/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*************************************************************************************
	OpenGL source interessante 
	de Microsoft: http://msdn.microsoft.com/library/default.asp?url=/library/en-us/opengl/oglport_17jm.asp

	Compilation: le rep "commun" doit faire partie de include de base
	/VS.NET aller dans Outils/options/project/repertoire de Vc++/fichier include
	/VS 6.0 aller dans ???
********************************************************************************** */

#include <Windows.h>
#include "controlEngine.h"
#include "MyString.h"
#include "MyList.h"
#include "input_instance.h"

#include "MyDebug.h"
#include "control_instance.h"
#include "button.h"
#include "texture_instance.h"
#include "textBox.h"
#include "title.h"
#include "container.h"
#include "window.h"
#include "effect.h"

#include "msgBox.h"
#include "tip.h"
#include "treeView.h"
#include "slider.h"
#include "calculator.h"

ControlEngine ce; // forcement en variable global

using namespace sigslot;  // pour signal (evit l'ecriture sigslot::signal?<...>)

/*!
\brief
Evenement de parametre et d'evenement de l'application
*/

class Tutorial : public Window
{
public:
	TextBox info;
	Button btnReturn;
	EffectApparition effect;
	Tip tip;

	Button btnUniversDemo;
	Button btnFaceDemo;
	Button btnBaseDemo;
	Button btnLayoutDemo;
	Button btnWindowDemo;
	Button btnApplicationDemo;

	class Demo : public has_slots<>
	{
	public:
		Tutorial *parent;

		Demo(Tutorial *_parent)
		{
			parent=_parent;
			parent->btnReturn.onButton.click.disconnect_all();
			parent->title.OnMin();
			ce.univers->HomePosition();
		}

		void Ending()
		{
			parent->title.OnOriginal();
			parent->SetTutorialTipText();
			parent->btnReturn.onButton.click.disconnect_all();
			parent->btnReturn.onButton.click.connect(parent,Tutorial::OnEnd);
			parent->effect.Bind(parent->faceNode);
			parent->effect.Start(FALSE,1);
		}
	};

	class FaceDemo  : public Demo
	{
	public:
		Title *titleBasic;
		Control *controlBasic;
		Title *titleText;
		Control *controlText;
		Title *titleCurving;
		Control *controlCurving;

		Timer timer;
		MyList<FaceNode *> nodes;

		FaceDemo(Tutorial *_parent) : Demo (_parent)
		{
			parent->tip.SetText("Face is the base graphics primitive; for example a Box is just on combination of Face, and by extand, it was the same for all control");
			parent->btnReturn.onButton.click.connect(this,OnEnd);
			ce.Push(ce.univers);
			FaceNode *node;
			Face *face;

			// groupe "Basic"
			controlBasic= new Control;
			controlBasic->faceNode = ce.primitive.AddFace("basic container",controlBasic->id,0,FALSE);
			face = controlBasic->faceNode->elem;
			face->position.x=-170;
			face->position.y=170;
			face->size.x=300;
			face->size.y=300;
			face->line=1;
			face->lineMaterial.SetDS(0.5,0.5,0.5);
			ce.Push(controlBasic);
			titleBasic = new Title;
			titleBasic->SetUses(FALSE,FALSE,FALSE,FALSE);
			titleBasic->Create(controlBasic);
			titleBasic->SetText("Basic: Outline,Red,Texture,Blend");
			titleBasic->XDockCenter(2,0);
			titleBasic->YDockTop(2,30);


			// outline
			node = ce.primitive.AddFace("outline",controlBasic->id,1,FALSE);
			nodes.i+=node;
			face = node->elem;
			face->position.x=-50;
			face->position.y=50;
			face->size.x=50;
			face->size.y=50;
			face->angle.Set(22,22,0);
			face->line=1;
			face->lineMaterial.SetDS(0,1,0);

			// basic
			node = ce.primitive.AddFace("red",controlBasic->id,2,TRUE);
			nodes.i+=node;
			face = node->elem;
			face->position.x=50;
			face->position.y=50;
			face->size.x=50;
			face->size.y=50;
			face->angle.Set(22,22,0);
			face->material.SetDS(1,0,0);

			// textured
			node = ce.primitive.AddFace("textured",controlBasic->id,3,TRUE);
			nodes.i+=node;
			face = node->elem;
			face->position.x=-50;
			face->position.y=-50;
			face->size.x=50;
			face->size.y=50;
			face->angle.Set(22,22,0);
			face->material.SetDS(1,1,1);
			face->texture=&ce.texture.iconBase;

			// blending
			node = ce.primitive.AddFace("blending",controlBasic->id,4,TRUE);
			nodes.i+=node;
			face = node->elem;
			face->position.x=50;
			face->position.y=-50;
			face->size.x=50;
			face->size.y=50;
			face->angle.Set(22,22,0);
			face->material.SetDS(1,0,0.5);
			node = ce.primitive.AddFace("blending2",controlBasic->id,5,TRUE);
			nodes.i+=node;
			face = node->elem;
			face->position.x=50+10;
			face->position.y=-50+10;
			face->position.z=20;
			face->size.x=40;
			face->size.y=40;
			face->blending=TRUE;
			face->material.SetDS(0,1,0,0.5);
			ce.Pop();

			// groupe "Text"
			controlText = new Control;
			controlText->faceNode = ce.primitive.AddFace("text container",controlText->id,0,FALSE);
			face = controlText->faceNode->elem;
			face->position.x=-170;
			face->position.y=-170;
			face->size.x=300;
			face->size.y=300;
			face->line=1;
			face->lineMaterial.SetDS(0.5,0.5,0.5);
			ce.Push(controlText);
			titleText = new Title;
			titleText->SetUses(FALSE,FALSE,FALSE,FALSE);
			titleText->Create(controlText);
			titleText->SetText("Text & Alignement");
			titleText->XDockCenter(2,0);
			titleText->YDockTop(2,30);


			// text
			int x,y=68;
			char xText[4][10]={"left","center","right","justify"};
			char yText[4][10]={"top","center","bottom","justify"};
			MyString text;
			for (int yAlign=0;yAlign<4;yAlign++)
			{
				x=-118;
				for (int xAlign=0;xAlign<4;xAlign++)
				{
					node = ce.primitive.AddFace("text",controlText->id,xAlign+yAlign*4,TRUE);
					nodes.i+=node;
					face = node->elem;
					face->position.x=x;
					face->position.y=y;
					face->size.x=60;
					face->size.y=50;
					face->angle.Set(22,22,0);
					face->text=new Text3D;
					face->text->SetJustifyLimit(100,100);
					face->text->Setup(ce.fontTexture,ft_texture);
					text<<clear<<xText[xAlign]<<"\n"<<yText[yAlign];
					face->text->SetText(text);
					face->text->SetAlign((TextAlign)xAlign,(TextAlign)yAlign);
					face->text->SetWrapping(TRUE,FALSE,TRUE);
					face->text->material.SetDS(0,0,1);
					face->line=1;
					face->lineMaterial.SetDS(1,0,0);
					x+=60;
				}
				y-=50;
			}
			ce.Pop();

			// groupe "Curving"
			controlCurving= new Control;
			controlCurving->faceNode = ce.primitive.AddFace("curving container",controlCurving->id,0,FALSE);
			face = controlCurving->faceNode->elem;
			face->position.x=170;
			face->position.y=-170;
			face->size.x=300;
			face->size.y=300;
			face->line=1;
			face->lineMaterial.SetDS(0.5,0.5,0.5);
			ce.Push(controlCurving);
			titleCurving = new Title;
			titleCurving->SetUses(FALSE,FALSE,FALSE,FALSE);
			titleCurving->Create(controlCurving);
			titleCurving->SetText("Curving");
			titleCurving->XDockCenter(2,0);
			titleCurving->YDockTop(2,30);


			// curving
			Material material[4];
			material[0].SetDS(1,0.5,0);
			material[1].SetDS(0.5,1,0);
			material[2].SetDS(0.5,0,1);
			material[3].SetDS(0.5,0.5,0.5);
			int num=0;

			y=50;
			for (float xCenter=0;xCenter<=1;xCenter+=0.5)
			{
				x=-50;
				for (float yCenter=0;yCenter<=1;yCenter+=0.5)
				{
					node = ce.primitive.AddFace("curving",controlCurving->id,num++,TRUE);
					nodes.i+=node;
					face = node->elem;
					face->position.x=x;
					face->position.y=y;
					face->size.x=50;
					face->size.y=50;
					face->material=material[num%4];
					if (xCenter==0.5&&yCenter==0.5) face->curve.Set(75,6,6);
					else face->curve.Set(50,yCenter==0.5?1:6,xCenter==0.5?1:6);
					face->curve.center.x=1-yCenter;
					face->curve.center.y=xCenter;
					x+=60;
				}
				y-=60;
			}

			ce.Pop();
			ce.Pop();

			BindTimer(timer,OnTimer,40);
		}

		int OnTimer(Timer *timer)
		{
			double time = timer->GetDeltaTime();
			double decal = 0;

			int num;
			// les 4*4+5 premiers sont les nodes du text
			for (nodes.i=0,num=0;num<21;nodes.i++,num++)
			{
				FaceNode *node = nodes.i.GetElem();
				node->elem->angle.Set(sin(0.6*time*PI+decal)*20,cos(0.6*time*PI+decal)*20,0);
				decal+=PI/3.0;
			}

			// les 3*3 dernier sont les nodes du curving
			decal = 0;
			for (;nodes.i.More();nodes.i++)
			{
				FaceNode *node = nodes.i.GetElem();
				node->elem->curve.center.z=sin(time*PI+decal)*50;
				decal+=PI/3.0;
			}
			return 0;
		}

		int OnEnd()
		{
			timer.onTimer.disconnect_all();
			Ending();
			delete titleBasic; // !attention, du a l'ordre de supression
			delete titleText;
			delete titleCurving;
			delete controlBasic;
			delete controlText;
			delete controlCurving;
			return 1;
		}
	};

	class UniversDemo  : public Demo
	{
	public:
		Control *control;
		Button *button[4*6];
		Timer timer;
		DockPos slideDirection; // type de deplacement
		double countDirection; // nombre de deplacement

		UniversDemo(Tutorial *_parent) : Demo(_parent) 
		{
			parent->tip.SetText("The Univers is the basic container, to move inside you can:\npush left button on the mouse and moving or using key HOME, directionnal (add Shift to rotate) or PRIOR/NEXT");
			parent->btnReturn.onButton.click.connect(this,OnEnd);

			ce.Push(ce.univers);
			control= new Control;
			control->faceNode = ce.primitive.AddBox("Box test",control->id,100,TRUE);
			Face *face = control->faceNode->elem;
			face->size.x=100;
			face->size.y=100;
			face->position.z=-50; // centr dans les z aussi

			SetFace(FACE_BACK,1,0,0,"Face\nBack");
			SetFace(FACE_FRONT,0,1,0,"Face\nFront");
			SetFace(FACE_LEFT,0,0,1,"Face\nLeft");
			SetFace(FACE_RIGHT,1,1,0,"Face\nRight");
			SetFace(FACE_TOP,1,0,1,"Face\nTop");
			SetFace(FACE_BOTTOM,0,1,1,"Face\nBottom");

			ce.Pop();
		}

		void SetFace(int numFace,float r,float g,float b,char *text)
		{
			Face *face = control->faceNode->Elems(numFace);
			face->material.SetDS(r,g,b);
			face->text= new Text3D;
			face->text->Setup(ce.fontTexture);
			face->text->SetAlign(align_center,align_center);
			face->text->SetText(text);
			face->text->material.SetDS(0,0,0);
			ce.primitive.GoTo(control->faceNode);
			ce.primitive.GoTo(numFace);
			for (int i=numFace;i<numFace+4;i++)
			{
				button[i] = new Button;
				button[i]->materialOnUp.SetDS(0.5,0.5,0.5,0.5);
				button[i]->materialOnDown.SetDS(1,1,1,0.5);
				button[i]->Create();
				button[i]->faceNode->Elems(FACE_FRONT)->texture = &ce.texture.arrow;
				button[i]->SetBlending(TRUE);
			}

			button[numFace]->XDockLeft(3,15);
			button[numFace]->YDockCenter(15);

			button[numFace+1]->XDockRight(3,15);
			button[numFace+1]->YDockCenter(15);

			button[numFace+2]->XDockCenter(15);
			button[numFace+2]->YDockTop(3,15);

			button[numFace+3]->XDockCenter(15);
			button[numFace+3]->YDockBottom(3,15);

			button[numFace+1]->RotateTo(0,0,180);
			button[numFace+2]->RotateTo(0,0,-90);
			button[numFace+3]->RotateTo(0,0,90);

			button[numFace+1]->MoveTo(-18,0,0);
			button[numFace+2]->MoveTo(9,-9,0);
			button[numFace+3]->MoveTo(9,9,0);

			button[numFace]->onButton.click.connect(this,OnLeft);
			button[numFace+1]->onButton.click.connect(this,OnRight);
			button[numFace+2]->onButton.click.connect(this,OnTop);
			button[numFace+3]->onButton.click.connect(this,OnBottom);
		}

		int OnTimer(Timer *timer)
		{
			double delta = 5.0;
			Face *face = control->faceNode->elem;

			countDirection-=delta;
			if (countDirection<=0)
			{
				timer->onTimer.disconnect(this);
				delta = -countDirection;
			}
			switch (slideDirection)
			{
			case left:	control->RotateQuaternion(delta,0,0);	break;
			case right: control->RotateQuaternion(-delta,0,0);	break;
			case top:	control->RotateQuaternion(0,delta,0);	break;
			case bottom:control->RotateQuaternion(0,-delta,0);	break;
			}	
			return 0;
		}

		int OnLeft()
		{
			ConfigSlide(left);
			return 1;
		}

		int OnRight()
		{
			ConfigSlide(right);
			return 1;
		}

		int OnTop()
		{
			ConfigSlide(top);
			return 1;
		}

		int OnBottom()
		{
			ConfigSlide(bottom);
			return 1;
		}

		void ConfigSlide(DockPos direction)
		{
			slideDirection=direction;
			countDirection=50; // il dois fonction en gradian, mais pourkoi ?!
			timer.onTimer.disconnect(this);
			BindTimer(timer,OnTimer,20);
		}

		int OnEnd()
		{
			Ending();
			//for (int i=0;i<4*6;i++) delete button[i];
			delete control;
			return 1;
		}
	};

	class BaseDemo : public Demo
	{
	public:
		Button *button;
		Title *title;
		TextBox *textBox ;
		Window *winInfo;
		Window *winSlider;
		MsgBox *msgBox;
		Slider *xSlider,*ySlider;
		Tip *tipSlider;
		double xPercent,yPercent;

		BaseDemo(Tutorial *_parent) : Demo(_parent)
		{
			parent->tip.SetText("Basic Control, like Button, TextBox, Title, Window and MsgBox");
			parent->btnReturn.onButton.click.connect(this,OnEnd);
			ce.taskWindow->title.OnOriginal();

			ce.Push(ce.univers);
			button = new Button;
			button->Create();
			button->SetText("Button");
			button->text.SetAlign(align_justify,align_begin);
			button->Size(80,50);
			button->MoveTo(0,-100,0);

			title = new Title;
			title->Create();
			title->SetText("Title");
			title->Size(200,20);
			title->MoveTo(0,0,0); 

			textBox = new TextBox;
			textBox->Create();
			textBox->SetText("Text");
			textBox->Size(50,20);
			textBox->MoveTo(0,100,0);

			winSlider = new Window;
			winSlider->Create();
			winSlider->title="Slider";
			winSlider->Size(200,200);
			winSlider->MoveTo(0,200,0);

			ce.Push(&winSlider->client);
				ce.primitive.GoTo(FACE_FRONT);
				xPercent=yPercent=0.5;
				xSlider = new Slider;
				xSlider->Create(10);
				xSlider->XDockCenter(2,15);
				xSlider->YDockBottom(2,15);
				xSlider->onSlidder.moving.connect(this,OnXSliderMove);

				ySlider = new Slider;
				ySlider->Create(10);
				xSlider->XDockCenter(2,15);
				xSlider->YDockBottom(18,15);
				ySlider->onSlidder.moving.connect(this,OnYSliderMove);

				tipSlider = new Tip;
				tipSlider->Create();
				tipSlider->XDockCenter(100);
				tipSlider->YDockCenter(100);
				tipSlider->text.SetWrapping(FALSE,FALSE,FALSE);
				tipSlider->Move(0,0,5);
			ce.Pop();

			winInfo = new Window;
			winInfo->Create();
			winInfo->title="Window";
			winInfo->Size(200,200);
			winInfo->MoveTo(-200,0,0);
			winInfo->RotateTo(0,35,0);

			winInfo->client.faceNode->Elems(FACE_FRONT)->texture=&ce.texture.tropique;

			winInfo->client.onControl.mouse.move.connect(this,OnWinMove);
			//winInfo->client.onControl.over.out.connnect(this,OnWinOut); // pb a la compil pkoi ?!
			Face *face = winInfo->client.faceNode->Elems(FACE_FRONT);
			face->material.SetDS(1,1,1,0.5f);
			face->blending=TRUE;
			face->text = new Text3D;
			face->text->Setup(ce.fontTexture);
			face->text->SetAlign(align_center,align_center);
			face->text->material.SetDS(1,1,0);
			face->text->SetWrapping(FALSE,FALSE,FALSE);
			face->text->SetText("Moving inside\nGive local positions");

			msgBox = new MsgBox;
			msgBox->Create();
			msgBox->Size(250,200);
			msgBox->title="3D Controls - MsgBox";
			msgBox->txtInfo="Hello Word !!\nVoici un example de l'utilisation de MsgBox";
			msgBox->txtTranche="3D Controls";
			msgBox->MoveTo(200,0,0);
			msgBox->RotateTo(0,-35,0);
			ce.Pop();
		}

		int OnXSliderMove(double min,double max,double percent)
		{
			xPercent=percent;
			MyString temp;
			temp.Format("X=%.0lf%%\nY=%.0lf%%",xPercent*100.0,yPercent*100.0);
			tipSlider->SetText(temp);
			return 0;
		}

		int OnYSliderMove(double min,double max,double percent)
		{
			yPercent=percent;
			MyString temp;
			temp.Format("X=%.0lf%%\nY=%.0lf%%",xPercent*100.0,yPercent*100.0);
			tipSlider->SetText(temp);
			return 0;
		}

		int OnWinOut (Control *,int)
		{
			Face *face = winInfo->client.faceNode->Elems(FACE_FRONT);
			face->text->SetText("Moving inside\nGive local positions");
			return 0;
		}

		int OnWinMove(StateMouse *mouse,int,int,int numFace)
		{
			MyString text;
			Point2D<double> pos(mouse->pos.x,mouse->pos.y);
			Point2D<double> localPos;
			localPos = winInfo->client.GetLocalPos(numFace,pos);

			Point3D<double> corners[4];
			winInfo->client.GetCorners(numFace,corners);
			text.Format("Coordonate {x,y}\n%.2f , %.2f",localPos.x,localPos.y);
			winInfo->client.faceNode->Elems(FACE_FRONT)->text->SetText(text);
			return 0;
		}

		int OnEnd()
		{
			ce.taskWindow->title.OnMin();
			Ending();
			delete button;
			delete textBox;
			delete winInfo;
			delete winSlider;
			delete title;
			delete msgBox;
			delete xSlider;
			delete ySlider;
			delete tipSlider;
			return 1;
		}
	};

	class LayoutDemo  : public Demo
	{
	public:
		Button btnToogleAll;
		Button btnToogle;
		Button btnResetAll;
		Button btnReset;
		Button btnSetAll;
		Button btnSet;
		Window window;
		TextBox text;
		Button btnTree;
		Button *lastSelected;
		MyList<Button> buttons;

		LayoutDemo(Tutorial *_parent) : Demo(_parent)
		{
			// general
			lastSelected=NULL;
			parent->tip.SetText("Layout cant be use to specify \"contact\" bitwin somme other Control\nIt's the easy way to create\"composite\" Controls");
			parent->btnReturn.onButton.click.connect(this,OnEnd);

			ce.Push(ce.univers);
			// affichage de l'arbre a boutton pushed like
			btnTree.Create(40);
			btnTree.XDockCenter(80);
			btnTree.YDockCenter(40);
			btnTree.ModePushLike(TRUE);
			btnTree.depth_min=btnTree.depth_max;
			btnTree.ModeAllButtons(TRUE);
			btnTree.faceNode->Elems(FACE_FRONT)->view=TRUE;
			ce.Push(&btnTree);
				CreateTreeButton(3);
			ce.Pop();

			// affichage du text 
			window.Create();
			window.title.SetText("Composite controls");
			window.XDockCenter(300);
			window.YDockTop(10,130);
			window.faceNode->Elems(FACE_FRONT)->material.SetNull();
			window.title.OnMax();
			ce.Push(&window.client);
				ce.primitive.GoTo(FACE_FRONT);
				text.Create();
				text.SetText("Sample composite botton that use other button inside");
				text.XDockCenter(3,0);
				text.YDockTop(3,40);


				text.text.SetAlign(align_begin,align_center);

				// boutton imbriqu "select"
				btnToogle="Toogle";
				btnToogle.Create();
				btnToogle.XDockCenter(80);
				btnToogle.YDockBottom(3,40);
				btnToogle.text.SetAlign(align_begin,align_center);
				btnToogle.onButton.click.connect(this,OnToogle);
				btnToogleAll.onButton.click.connect(this,OnToogleAll);
				ce.Push(&btnToogle);
					ce.primitive.GoTo(FACE_FRONT);
					btnToogleAll="All";
					btnToogleAll.Create();
					btnToogleAll.XDockRight(3,30);
					btnToogleAll.YDockCenter(2,2);
				ce.Pop();	

				// boutton imbriqu "set"
				btnSet="Set";
				btnSet.Create();
				btnSet.XDockLeft(3,80);
				btnSet.YDockBottom(3,40);
				btnSet.text.SetAlign(align_begin,align_center);
				btnSet.onButton.click.connect(this,OnSet);
				btnSetAll.onButton.click.connect(this,OnSetAll);
				ce.Push(&btnSet);
					ce.primitive.GoTo(FACE_FRONT);
					btnSetAll="All";
					btnSetAll.Create();
					btnSetAll.XDockRight(3,30);
					btnSetAll.YDockCenter(2,2);
				ce.Pop();	

				// boutton imbriqu "reset"
				btnReset="Reset";
				btnReset.Create();
				btnReset.XDockRight(3,80);
				btnReset.YDockBottom(3,40);
				btnReset.text.SetAlign(align_begin,align_center);
				btnReset.onButton.click.connect(this,OnReset);
				btnResetAll.onButton.click.connect(this,OnResetAll);
				ce.Push(&btnReset);
					ce.primitive.GoTo(FACE_FRONT);
					btnResetAll="All";
					btnResetAll.Create();
					btnResetAll.XDockRight(3,30);
					btnResetAll.YDockCenter(2,2);
				ce.Pop();	
			ce.Pop();	
		}

		void CreateTreeButton(int level)
		{
			if (level)
			{
				Button *button;
				if (!(level%2)) 
				{	//si impaire ne fait qu'un boutton					
					button = buttons.i.AddNewLast();
					ce.primitive.GoTo(FACE_FRONT);
					ConfigTreeButton(button,level*6);	
					ce.Push(button);
						CreateTreeButton(level-1);
					ce.Pop();
				}
				else
				{
					for (int i=0;i<6;i++)
					{
						button = buttons.i.AddNewLast();
						ce.primitive.GoTo(i);
						ConfigTreeButton(button,level*6+i);							
						ce.Push(button);
							CreateTreeButton(level-1);
						ce.Pop();
						ce.primitive.GoToParent();
					}
				}
			}
		}

		void ConfigTreeButton (Button *button,int num)
		{
			button->Create(40);
			button->ModePushLike(TRUE);
			button->ModeAllButtons(TRUE);
			button->XDockCenter(5,5);
			button->YDockCenter(5,5);
			//button->onButton.click_sender.connect(this,OnTreeButtonClick);
			MyString txtButton;
			txtButton = materials[num%NB_MATERIAL].name;
			//txtButton<<"n"<<num;
			button->SetBlending(TRUE);
			button->faceNode->Elems(FACE_BACK)->material.As((TypeMaterial)num);
			ConfigTextFace(&button->text,txtButton);

			for (int i=1;i<5;i++)
			{
				Text3D *text = new Text3D;
				button->faceNode->Elems(i)->text = text;
				ConfigTextFace(text,txtButton);
			}
		}

		void ConfigTextFace(Text3D *text,char *textValue)
		{
			text->Setup(ce.fontTexture);
			text->SetText(textValue);
			text->material.SetDS(1,0,0);
			text->scale=0.5;
			text->SetWrapping(FALSE,FALSE,TRUE);
			text->SetAlign(align_center,align_center);
		}

		int OnTreeButtonClick(Control *sender)
		{
			Button *button = (Button *)sender;
			lastSelected = button;
			return 1;
		}

		int OnToogle()
		{
			if (!lastSelected) return 1;
			lastSelected->SetPushToogle();
			return 1;
		}

		int OnToogleAll()
		{
			for (buttons.i=0;buttons.i.More();buttons.i++)
			{
				buttons.i.GetElemPtr()->SetPushToogle();
			}
			return 1;
		}

		int OnReset()
		{
			if (!lastSelected) return 1;
			lastSelected->SetPush(FALSE);
			return 1;
		}

		int OnResetAll()
		{
			for (buttons.i=0;buttons.i.More();buttons.i++)
				buttons.i.GetElemPtr()->SetPush(FALSE);
			return 1;
		}

		int OnSet()
		{
			if (!lastSelected) return 1;
			lastSelected->SetPush(TRUE);
			return 1;
		}

		int OnSetAll()
		{
			for (buttons.i=0;buttons.i.More();buttons.i++)
				buttons.i.GetElemPtr()->SetPush(TRUE);
			return 1;
		}

		int OnEnd()
		{
			btnTree.Delete();
			window.Delete();
			Ending();
			return 1;
		}
	};

	class WindowDemo  : public Demo
	{
	public:
		Window *windowLeft;
		Window *windowRight;
		Window *windowTop;
		Window *windowBottom;

		WindowDemo(Tutorial *_parent) : Demo(_parent)
		{
			parent->tip.SetText("The Window as a docking Title, he can:\nMinimize, Maximize, Moving, Close and Rotate");
			parent->btnReturn.onButton.click.connect(this,OnEnd);

			ce.taskWindow->title.OnOriginal();

			ce.Push(ce.univers);

			Face *face;
			windowLeft=new Window;
			windowLeft->titleDock=left;
			windowLeft->title.SetText("Left");
			windowLeft->Create();
			windowLeft->RotateTo(0,33,0);
			windowLeft->XDockLeft(10,150);
			windowLeft->YDockCenter(200);
			face = windowLeft->client.faceNode->Elems(FACE_FRONT);
			face->text = new Text3D;
			face->text->Setup(ce.fontTexture);
			face->text->SetAlign();
			face->text->material.SetDS(0,0,0);
			face->text->SetText("I can write text on client area !!");

			windowRight=new Window;
			windowRight->titleDock=right;
			windowRight->Create();
			windowRight->title.SetText("Right");
			windowRight->RotateTo(0,-33,0);
			windowRight->XDockRight(10,150);
			windowRight->YDockCenter(200);
			face = windowRight->client.faceNode->Elems(FACE_FRONT);
			face->text = new Text3D;
			face->text->Setup(ce.fontTexture);
			face->text->SetAlign();
			face->text->material.SetDS(0,0,0);
			face->text->SetText("I can write text on client area !!");

			windowTop=new Window;
			windowTop->titleDock=top;
			windowTop->Create();
			windowTop->title.SetText("Top");
			windowTop->RotateTo(33,0,0);
			windowTop->XDockCenter(200);
			windowTop->YDockTop(10,150);
			face = windowTop->client.faceNode->Elems(FACE_FRONT);
			face->text = new Text3D;
			face->text->Setup(ce.fontTexture);
			face->text->SetAlign();
			face->text->material.SetDS(0,0,0);
			face->text->SetText("I can write text on client area !!");

			windowBottom=new Window;
			windowBottom->titleDock=bottom;
			windowBottom->Create();
			windowBottom->title.SetText("bottom");
			face = windowBottom->client.faceNode->Elems(FACE_FRONT);
			face->text = new Text3D;
			face->text->Setup(ce.fontTexture);
			face->text->SetAlign();
			face->text->material.SetDS(0,0,0);
			face->text->SetText("I can write text on client area !!");

			windowBottom->RotateTo(-33,0,0);
			windowBottom->XDockCenter(200);
			windowBottom->YDockBottom(10,150);

			ce.primitive.Pop();
		}

		int OnEnd()
		{
			ce.taskWindow->title.OnMin();
			Ending();
			delete windowLeft;
			delete windowRight;
			delete windowTop;
			delete windowBottom;
			return 1;
		}
	};

	class ApplicationDemo  : public Demo
	{
	public:
		Calculator *calc;
		Window *winExplorer;
		Window *winBackground;
		TreeDirectory * tdBkg;
		TreeDirectory * tdExplorer;
		TextBox pathBox;

		ApplicationDemo(Tutorial *_parent) : Demo(_parent)
		{
			parent->tip.SetText("This is some sample of basic application using 3D Controls");
			parent->btnReturn.onButton.click.connect(this,OnEnd);
			ce.taskWindow->title.OnOriginal();
			calc=new Calculator;

			ce.Push(ce.univers);
				calc->Create();
				calc->XDockCenter(94);
				calc->YDockCenter(224);

				winBackground = new Window;
				winBackground->Create();
				winBackground->title.SetText("Backgrounds");
				winBackground->XDockLeft(20,200);
				winBackground->YDockCenter(400);
				ce.Push(&winBackground->client);
				{
					tdBkg = new TreeDirectory;
					tdBkg->Create();
					tdBkg->XDockCenter(2,2);
					tdBkg->YDockCenter(2,2);
					tdBkg->Config(TRUE,TRUE);
					tdBkg->Loading("texture/background",FALSE);
					tdBkg->Draw();
					tdBkg->onTreeView.select.connect(this,OnSelectBkg);
				}
				ce.Pop();

				winExplorer = new Window;
				winExplorer->Create();
				winExplorer->title.SetText("Explorer");
				winExplorer->XDockRight(20,200);
				winExplorer->YDockCenter(400);
				ce.Push(&winExplorer->client);
				{
					pathBox.Create();
					pathBox.XDockCenter(2,2);
					pathBox.YDockTop(2,60);
					pathBox.text.SetAlign(align_begin,align_begin);
					pathBox.text.SetWrapping(FALSE,FALSE,FALSE);

					tdExplorer = new TreeDirectory;
					tdExplorer->Create();
					tdExplorer->XDockCenter(2,2);
					tdExplorer->YDockBottom(2,300);
					tdExplorer->Config(FALSE,TRUE);
					tdExplorer->Loading("c:",TRUE);
					tdExplorer->Draw();
					tdExplorer->onTreeView.select.connect(this,OnSelectExplorer);
				}
				ce.Pop();
			ce.Pop();
		}

		int OnSelectBkg(TreeNode *treeNode)
		{
			MyString file;
			tdBkg->GetTreePath(treeNode,file);
			ce.texture.background.FromFile("background",file,TRUE);
			return 0;
		}

		int OnSelectExplorer(TreeNode *treeNode)
		{
			MyString file;
			tdExplorer->GetTreePath(treeNode,file);
			MyString complete;
			complete<<"Path:\n"<<file;
			pathBox.text.SetText(complete);
			return 0;
		}

		int OnEnd()
		{
			ce.taskWindow->title.OnMin();
			Ending();
			delete calc;
			return 1;
		}
	};
	
	FaceDemo *fd;
	UniversDemo *ud;
	BaseDemo *db;
	LayoutDemo *ld;
	WindowDemo *wd;
	ApplicationDemo *ad;

	Tutorial()
	{
		fd = NULL;
		ud = NULL;
		db = NULL;
		ld = NULL;
		wd =NULL;
		ad = NULL;
		ce.taskWindow->title.OnMin();

		title="3D Controls - Tutorial";
		title.useClose=FALSE;
		title.useMax=FALSE;
		title.useMin=FALSE;

		info="Here are 6 steps to familiarize with the 3D Controls.";
		info.text.SetAlign(align_justify,align_center);

		btnUniversDemo="1) How to move on the desktop:\nThe \"Univers\" concept.";
		btnUniversDemo.text.SetAlign(align_justify,align_center);

		btnFaceDemo="2) Basic graphical concepts:\nThe \"Face\" concept.";
		btnFaceDemo.text.SetAlign(align_justify,align_center);

		btnBaseDemo="3) See the basic controls.";
		btnBaseDemo.text.SetAlign(align_justify,align_center);

		btnLayoutDemo="4) How to position a control next to another ?";
		btnLayoutDemo.text.SetAlign(align_justify,align_center);

		btnWindowDemo="5) The Window is the basic container of the application.";
		btnWindowDemo.text.SetAlign(align_justify,align_center);

		btnApplicationDemo="6) Examples of applications for 3D Controls.";
		btnApplicationDemo.text.SetAlign(align_justify,align_center);
		
		btnUniversDemo.onButton.click.connect(this,OnUniversDemo);
		btnFaceDemo.onButton.click.connect(this,OnFaceDemo);
		btnBaseDemo.onButton.click.connect(this,OnBaseDemo);
		btnLayoutDemo.onButton.click.connect(this,OnLayoutDemo);
		btnWindowDemo.onButton.click.connect(this,OnWindowDemo);
		btnApplicationDemo.onButton.click.connect(this,OnApplicationDemo);
	}

	~Tutorial()
	{
		ce.taskWindow->title.OnOriginal();
	}

	void Create()
	{
		ce.Push(ce.masterContainer);

		Window::Create();
		canResize=FALSE;
		canRotate=FALSE;

		XDockCenter(300);
		YDockCenter(394);
		ce.Push(&client);
			ce.primitive.GoTo(FACE_FRONT);
			info.Create();
			int yPos = 2;
			info.XDockCenter(2,0);
			info.YDockTop(yPos,50);

			yPos+=52;
			btnUniversDemo.Create();
			btnUniversDemo.XDockCenter(2,0);
			btnUniversDemo.YDockTop(yPos,50);

			yPos+=52;
			btnFaceDemo.Create();
			btnFaceDemo.XDockCenter(2,0);
			btnFaceDemo.YDockTop(yPos,50);

			yPos+=52;
			btnBaseDemo.Create();
			btnBaseDemo.XDockCenter(2,0);
			btnBaseDemo.YDockTop(yPos,50);

			yPos+=52;
			btnLayoutDemo.Create();
			btnLayoutDemo.XDockCenter(2,0);
			btnLayoutDemo.YDockTop(yPos,50);

			yPos+=52;
			btnWindowDemo.Create();
			btnWindowDemo.XDockCenter(2,0);
			btnWindowDemo.YDockTop(yPos,50);

			yPos+=52;
			btnApplicationDemo.Create();
			btnApplicationDemo.XDockCenter(2,0);
			btnApplicationDemo.YDockTop(yPos,50);

			yPos+=52;

			ce.Push(ce.masterContainer);
			tip.Create();
			tip.XDockCenter(10,10);
			tip.YDockBottom(10,50);
			ce.Pop();
			SetTutorialTipText();
		ce.Pop();

		btnReturn.Create();
		btnReturn.SetText("<< Back");
		btnReturn.XDockRight(10,65);		
		btnReturn.YDockTop(10,20);
		btnReturn.onButton.click.connect(this,OnEnd);
		ce.Pop();

		effect.Bind(faceNode);
		effect.Start(FALSE,1);
	}

	void SetTutorialTipText()
	{
		tip.SetText("Welcom to \"3D Controls\" user tutorial");
	}

	void DestroyDemos()
	{
		if (fd) { delete fd; fd=NULL;}
		if (ud) { delete ud; ud=NULL;}
		if (db) { delete db; db=NULL;}
		if (ld) { delete ld; ld=NULL;}
		if (wd) {delete wd; wd=NULL;}
		if (ad) {delete ad; ad=NULL;}
	}

	int OnUniversDemo()
	{
		OnUniversDemoStart();
		//effect.onEnding.connect(this,OnUniversDemoStart);
		//effect.Start(TRUE,1);
		return 1;
	}

	int OnFaceDemo()
	{
		OnFaceDemoStart();
		//effect.onEnding.connect(this,OnFaceDemoStart);
		//effect.Start(TRUE,1);
		return 1;
	}

	int OnBaseDemo()
	{
		OnBaseDemoStart();
		//effect.onEnding.connect(this,OnBaseDemoStart);
		//effect.Start(TRUE,1);
		return 1;
	}

	int OnLayoutDemo()
	{
		OnLayoutDemoStart();
		//effect.onEnding.connect(this,OnLayoutDemoStart);
		//effect.Start(TRUE,1);
		return 1;
	}

	int  OnWindowDemo()
	{
		OnWindowDemoStart();
		//effect.onEnding.connect(this,OnWindowDemoStart);
		//effect.Start(TRUE,1);
		return 1;
	}

	int OnApplicationDemo()
	{
		OnApplicationDemoStart();
		//effect.onEnding.connect(this,OnApplicationDemoStart);
		//effect.Start(TRUE,1);
		return 1;
	}
	void OnUniversDemoStart()
	{
		DestroyDemos();
		UniversDemo * ud = new UniversDemo(this);
	}

	void OnFaceDemoStart()
	{
		DestroyDemos();
		FaceDemo *fd = new FaceDemo(this);
	}

	void OnBaseDemoStart()
	{
		DestroyDemos();
		BaseDemo *bd = new BaseDemo(this);
	}

	void OnLayoutDemoStart()
	{
		DestroyDemos();
		LayoutDemo *ld = new LayoutDemo(this);
	}

	void OnWindowDemoStart()
	{
		DestroyDemos();
		WindowDemo *wd = new WindowDemo(this);
	}

	void OnApplicationDemoStart()
	{
		DestroyDemos();
		ApplicationDemo *ad = new ApplicationDemo(this);
	}
	int OnEnd()
	{
		ce.univers->ExitQuestion();
		return 1;
	}
};

class Demo : public has_slots<>
{
public:
	Timer timer;
	Tip tip[6];

	Demo()
	{
		ce.taskWindow->title.OnMin();
		ce.univers->faceNode->elem->position.z=-700;

		char text[][100]={
			"Il y a bien longtemp",
			"Tres tres longtemp",
			"Trop mme ...",
			"Mais c'est finis !",
			"Quoi ???",
			"DE TROP PARLER"
		};

		ce.Push(ce.masterContainer);
		for (int i=0;i<6;i++)
		{
			tip[i].Create();
			tip[i].XDockCenter(200);
			tip[i].YDockCenter(200);
			tip[i].SetText(text[i]);
			tip[i].faceNode->elem->position.z=-i*800;
			tip[i].SetAlphaBlending(0);
			tip[i].text.material.SetDS(0,0,0,0.8f);
		}
		ce.Pop();

		BindTimer(timer,OnTimer,10);
	}

	int OnTimer(Timer *)
	{
		ce.univers->faceNode->elem->position.z.constValue +=10;
		ce.univers->Rotate(0.1,0.2,0);
		for (int i=0;i<6;i++)
		{
			tip[i].faceNode->elem->position.z.constValue+=10;
		}
		return 0;
	}
};

int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{	
	MyDebug::Start();
	ce.Start(hInstance);

	//Demo demo;
	Tutorial tutorial;
	tutorial.Create();

	ce.Loop();
	return 0;
}