/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file Inertia.h
\brief Gestion du mode Inertia
*/

#ifndef INCLUDE_INERTIA_H
#define INCLUDE_INERTIA_H

#include "windows.h"
#include "MyList.h"
#include "input.h"

/*!
\brief
Element du stack d'inertia
*/
class ElemStack
{
public:
	Point2D<int> elem;
	double timing;

	void operator=(ElemStack &src)
	{
		elem = src.elem;
		timing = src.timing;
	}
};

/*!
\brief Donne le vecteur correspondant a la moyenne de deplacement donne par longeur de cycl donne.

Remarque :
	- le moteur de timer doit deja etre lanc par "TimerEvent::Start();")
	- l'element template T doit avoir definie les operateurs =,/=,+=,-= avec lui meme

Principe de distrubution:
	- chaque pick de valeur est suppose dure tInertie
	- si un au pick arrive avant ca la fin tInertie du precedent elle le poursuis 

Ex.:
\code
Inertia<int,double> inertia;
int moy;

inertia.Start(5,10);	// 5sec
...
inertia.Add(2);			
...
inertia.Add(5);			
...
moy=GetMoyValue();
\endcode
*/
class Inertia: public has_slots<>
{
private:
	MyStack<ElemStack> stack;
	Timer timer;
	double viewTime;
	double inertiaTime;

public:
	Inertia();
	void Start(double _viewTime=0.5,double _inertiaTime=0.1);
	void Add(Point2D<int> element);
	Point2D<double> GetMoyValue();
};

class Control;

/*!
\brief Evenement lie a inertia
*/
class EventInertia : public has_slots<>
{
private:
	StateMouse *mouseState;
	Point2D<double> mousePos;
	int numButton;
	Timer timer;
	int face;
	Point2D<double> vectorInertia;

public:
	BOOL enable;

	EventInertia();
	void operator=(EventInertia &src);
	void Init(int _numButton);
	void Start(int _face,Point2D<double> _vectorInertia);
	void Stop();
	void SwitchButton();
	int OnInertia(Timer *timer);
};

#endif