/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file MyObject.h
\brief Element generique d'un list ou d'un arbre
*/

#ifndef INCLUDE_MY_OBJECT_H
#define INCLUDE_MY_OBJECT_H

class Object;
typedef void * (Object::*Obj_CallBack) (void *);

/*!
\brief "Objet" generique, utilise pour faire des list ou arbre d'element de type different

Ex.: "list a element variable"
\code
// chaque class voulant etre CBase faire son init avec
// BaseObj<CMyClass,CBASE_INDEX_CUSTOM> obj_myClass;

MyList<Obj> objs;
{
	CBase_int var0;
	var0.value = 12;
	objs+=var0;
	CBase_double var1;
	var1.value = 12.345;
	objs+=var1;
	CBase_string var2;
	var2.value = "coucou";
	objs+=var2;
	// le element on boen ete recopier car a la	fin	du } var0,var1,var2	sont des-alloue
}	
CBase_string varStatic;
varStatic.value="fait le hibou";
objs+=&varStatic; // ici on	affect le ptr par d'utilisation	de recopy

MyString res;
for (objs=0;objs.More();objs++)
{
	Obj *ptrObj = objs();
	switch (ptrObj->type)
	{
		case CBASE_INDEX_INT:
		{
			CBase_int *ptr = (CBase_int *)ptrObj->thisParent;
			res<<" Int:	"<<ptr->value;
			break;
		}
		case CBASE_INDEX_DOUBLE: 
		{
			CBase_double *ptr = (CBase_double *)ptrObj->thisParent;
			res<<" Double: "<<ptr->value;
			break;
		}
		case CBASE_INDEX_STRING: 
		{
			CBase_string *ptr = (CBase_string *)ptrObj->thisParent;
			res<<" String: "<<ptr->value;
			break;
		}
	}
}

MessageBox(NULL,res,"Resultat",0);
\endcode
*/
class Object 
{
public:
	void *thisParent;
	int type;
	LONG param; // parametre suplementaire
	Obj_CallBack callBackAlloc;

public:
	Object()
	{
		param = 0;
		thisParent=NULL;
		type=-1;
		callBackAlloc=NULL;
	}

	Object(int _type,void *_thisParent)
	{
		type = _type;
		thisParent = _thisParent;
	}

	void operator= (Object &source)
	{
		param = source.param;
		type = source.type;	
		callBackAlloc = source.callBackAlloc;
		thisParent = (*this.*callBackAlloc)(source.thisParent);
	}
};

/*!
\brief Objet de base.

Si la class T est elle meme une template faire prealablement:
	-  typedef CMaTemplate<type> CMaTemplate_type;
	- #define CBASE_INDEX_MATEPLATE_TYPE CBASE_INDEX_CUSTOM+num
	- #define CBase_MaTemplate_type BaseObj<CMaTemplate_type,CBASE_INDEX_MATEPLATE_TYPE>
*/
template <class T,int V> class BaseObj : public Object
{
public:
	T elem;

	BaseObj() : Object(V,(void *)this) 
	{
		callBackAlloc = (Obj_CallBack) Duplicate;
	}

	void * Duplicate (void *_in)
	{
		BaseObj<T,V> *in = (BaseObj<T,V> *)_in;
		BaseObj<T,V> *out = new BaseObj<T,V>;
		*out = *in;
		return (void *)out;
	}

	void operator=(BaseObj<T,V> &source)
	{
		elem = source.elem;
	}
};

#define CBASE_INDEX_INT		0
#define CBASE_INDEX_FLOAT	1
#define CBASE_INDEX_DOUBLE	2
#define CBASE_INDEX_STRING	3
#define CBASE_INDEX_CUSTOM	4

#define CBase_int BaseObj<int,CBASE_INDEX_INT>
#define CBase_float BaseObj<float,CBASE_INDEX_FLOAT>
#define CBase_double BaseObj<double,CBASE_INDEX_DOUBLE>
#define CBase_string BaseObj<MyString,CBASE_INDEX_STRING>

/* Visuel de class virtuel ******************************************************
 class A
 {
 public:
	 int var_a;

	 virtual void meth_a()
	 {
		 var_a=10;
	 }

	 void meth_a2()
	 {
		 meth_a();
	 }
 };

 class B: public A
 {
 public:
	 int var_b;

	 void meth_a()
	 {
		 var_b=20;
	 }
 };

 // test:
 A a;
 B b;

 a.meth_a();
 a.meth_a2();
 b.meth_a();
 b.meth_a2();
 A *c = &b;
 c->meth_a2();

 // classe de text pour les object
 class CTest
 {
 public:
 int i;
 MyString text;

 CTest()
 {
 text="";
 i=0;
 }
 void operator=(CTest &source)
 {
 i = source.i;
 text = source.text;
 }
 };
*/

#endif