/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file MyString.h
\brief Chaine de caractere dynamique
*/

#ifndef INCLUDE_MYSTRING_H
#define INCLUDE_MYSTRING_H

#include <windef.h>

// pour gestion de manipulateur
class MyString;
typedef void (*FncManipulateur) (MyString&);

/*!
\brief Gestion de chaine de caractere dynamique

Ex.:
\code
MyString temp,a,b,c;
a = "coucou";
b = "toi";
c = a;
temp = a + "lui";
c = "hello " + b;
temp = a + b;
\endcode
*/
class MyString
{
	private:
		void Delete();
		BOOL Resize();
		BOOL Resize(int newSize,BOOL recopy=FALSE);

	public:
		MyString();
		virtual ~MyString() { Delete(); }

		BOOL IsEmpty() { return (size == 0)||(size==1&&value[0]=='\0'); } // \0 par default

		MyString(char letter);
		MyString(int integer);
		MyString(long linteger);
		MyString(float floating);
		MyString(double lfloating);
		MyString(char *string);
		MyString(MyString &string);

		void Zero() { *this='\0'; };
		void operator+=(char letter);
		void operator+=(int integer);
		void operator+=(long linteger);
		void operator+=(float floating);
		void operator+=(double lfloating);
		void operator+=(char *string);
		void operator+=(MyString &string) { *this+=string.value; }

		MyString & operator=(char letter);
		MyString & operator=(int integer);
		MyString & operator=(long linteger);
		MyString & operator=(float floating);
		MyString & operator=(double lfloating);
		MyString & operator=(char *string);
		MyString & operator=(MyString &string);

		BOOL operator==(char letter);
		BOOL operator==(int integer);
		BOOL operator==(long linteger);
		BOOL operator==(float floating);
		BOOL operator==(double lfloating);
		BOOL operator==(char *string);
		BOOL operator==(MyString &string);

		BOOL operator!=(char letter) { return !operator==(letter); };
		BOOL operator!=(int integer) { return !operator==(integer); };
		BOOL operator!=(long linteger) { return !operator==(linteger); };
		BOOL operator!=(float floating) { return !operator==(floating); };
		BOOL operator!=(double lfloating) { return !operator==(lfloating); };
		BOOL operator!=(char *string) { return !operator==(string); };
		BOOL operator!=(MyString &string) { return !operator==(string); };

		BOOL AddFormat(char *format,...);
		BOOL Format(char *format,...);

		long	ToInteger();
		float	ToFloat();
		double	ToDouble();

		friend MyString operator+(MyString &string,char letter);
		friend MyString operator+(char letter,MyString &string);

		friend MyString operator+(MyString &string,int integer);
		friend MyString operator+(int integer,MyString &string);

		friend MyString operator+(MyString &string,long linteger);
		friend MyString operator+(long linteger,MyString &string);

		friend MyString operator+(MyString &string,float floating);
		friend MyString operator+(float floating,MyString &string);

		friend MyString operator+(MyString &string,double lfloating);
		friend MyString operator+(double lfloating,MyString &string);

		friend MyString operator+(MyString &string1,char *string2);
		friend MyString operator+(char *string1,MyString &string2);

		friend MyString operator+(MyString &string1,MyString &string2);

		char & operator [] (int num);
		operator char *() const { return value; }

		friend MyString & operator << (MyString& string , char letter);
		friend MyString & operator << (MyString& string , int integer);
		friend MyString & operator << (MyString& string , long linteger);
		friend MyString & operator << (MyString& string , float floating);
		friend MyString & operator << (MyString& string , double lfloating);
		friend MyString & operator << (MyString& string1 , char *string2);
		friend MyString & operator << (MyString& string1 , MyString &string2);
		friend MyString & operator << (MyString& in ,FncManipulateur fnc);

		void SplitPath(MyString *drive,MyString *dir,MyString *file,MyString *ext,MyString *driveDir,MyString *fileExt);
		BOOL SubStringBeforToken(int numSubString,MyString &result,char *lstToken);

		char * GetValue(int index);
		BOOL CopyTo(MyString &result,int index,int size);

		BOOL FormatToNoCtrl();
		BOOL FormatToLower();
		BOOL FormatToUpper();

		void AddRepetitiveFormat(char *matrise,int nbCopy);
		
		BOOL IsVisibleText();

		int GetPositionFromLeft(int pos);
		int GetPositionFromRight(int pos);

		BOOL Delete(int pos,BOOL after,char letterToFind);
		BOOL Delete(int pos,BOOL after,int nbLetter);

		BOOL Add(int pos,BOOL after,char letter);
		BOOL Add(int pos,BOOL after,int integer);
		BOOL Add(int pos,BOOL after,long linteger);
		BOOL Add(int pos,BOOL after,float floating);
		BOOL Add(int pos,BOOL after,double lfloating);
		BOOL Add(int pos,BOOL after,char *string);
		BOOL Add(int pos,BOOL after,MyString &string);
		
		int FindLetter(int pos,BOOL after,char letter);
		int FindLetter(char letter);
		MyString Mids(int beginPos,int sizeToCopy);
		BOOL IsBeginWith(char *model);
		char * TestSubString(char *text,BOOL caseSensitive=TRUE);

		void EnterHexaNumber(int value);
		void EnterBinNumber(int value);
		void Reverse();

		long GetSize();
		char *GetValue();

	private:
		char *value;
		long size;
		long allocSize;
};

// liste des manipulateurs
void clear (MyString &in); // permet l'ecriture: myString<<clear<<"coucou";

#endif