/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "3d.h"
#include "material.h"
#include "object3d.h"

#include "controlEngine.h"
extern ControlEngine ce;

/* ****************************************************************************
**************************************************************************** */

BOOL Object3D::DrawVectorMinMax(double x,double y,double minIntensity,double maxIntensity)
{
	double intensity = sqrt(x*x+y*y);
	//MyDebug::SendEvent("draw vector","x:%lf y:%lf i:%lf",x,y,intensity);
	
	if (intensity<minIntensity) return FALSE;
	if (intensity>maxIntensity) 
	{
		double angle=atan2(y,-x);
		x=cos(angle)*maxIntensity;
		y=sin(angle)*maxIntensity;
	}
	DrawVector(x,y);
	return TRUE;
}

/*!
\brief Affiche un vecteur
\todo blending, mais la j'ai pas reussi
*/
void Object3D::DrawVector(double x,double y)
{
	glDisable(GL_TEXTURE_2D);
	//glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	double angle=atan2(y,-x)*(180.0/PI);
	double intensity = sqrt(x*x+y*y)/25.0;

	Point3D<double> rotate(0,0,angle);

	Material material;
	material.SetNull();
	//material.SetDS(0,0,1,0.5);
	material.emission.Set(0,0,1);
	material.Use();

	glPushMatrix();
	rotate.glRotated();
	glBegin(GL_POLYGON);
	glVertex2d(0,0);
	glVertex2d(intensity,0);
	glVertex2d(intensity-0.05,-0.05);
	glVertex2d(intensity-0.05,0.05);
	glVertex2d(intensity,0);
	glEnd();
	glPopMatrix();
	//glDisable(GL_BLEND);
}

void Object3D::DrawArrow(Color color)
{
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	Material material;

	material.SetNull();
	material.emission = color;
	material.Use();

	// tracement des regions
	glBegin(GL_POLYGON);
	glVertex2d(0,0);
	glVertex2d(0,-0.2);
	glVertex2d(0.05,-0.15);
	glVertex2d(0.12,-0.3);
	glVertex2d(0.20,-0.22);
	glVertex2d(0.10,-0.10);
	glVertex2d(0.15,-0.06);
	glVertex2d(0,0);
	glEnd();

	// les contours
	material.emission.SetNull();
	material.Use();
	glBegin(GL_LINE_LOOP);
	glVertex2d(0,0);
	glVertex2d(0,-0.2);
	glVertex2d(0.05,-0.15);
	glVertex2d(0.12,-0.3);
	glVertex2d(0.2,-0.22);
	glVertex2d(0.10,-0.10);
	glVertex2d(0.15,-0.06);
	glEnd();
}

// ********************************************************************************

void Object3D::DrawRepere()
{
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);

	Material material;
	material.SetNull();
	
	GLUquadric *newQuadric = gluNewQuadric();
	gluQuadricNormals(newQuadric,GLU_NONE); // sinon influ de maniere misterieuse les autre normales
	double size=0.03;
	double baseSize=0.8;
	double arrowSize=0.3;
	double baseArrowSize=0.15;

	glPushMatrix(); // les Z
		material.emission.Set(1,0,0);
		material.Use();
		gluCylinder(newQuadric,size,size,baseSize,3,3);
		glTranslated(0,0,baseSize);
		gluCylinder(newQuadric,baseArrowSize,0,arrowSize,3,3);
		//glTranslated(0,0,0.5);
		//DrawText("Z");
	glPopMatrix();

	glPushMatrix(); // les Y
		material.emission.Set(0,1,0);
		material.Use();
		glRotated(-90,1,0,0);
		gluCylinder(newQuadric,size,size,baseSize,3,3);
		glTranslated(0,0,baseSize);
		gluCylinder(newQuadric,baseArrowSize,0,arrowSize,3,3);
		//glTranslated(0,0,0.5);
		//DrawText("Y");
	glPopMatrix();
	
	glPushMatrix(); // les X
		material.emission.Set(0,0,1);
		material.Use();
		glRotated(90,0,1,0);
		gluCylinder(newQuadric,size,size,baseSize,3,3); 
		glTranslated(0,0,baseSize);
		gluCylinder(newQuadric,baseArrowSize,0,arrowSize,3,3);
		//glTranslated(0,0,0.5);
		//DrawText("X");
	glPopMatrix();

	gluDeleteQuadric(newQuadric);
	
	material.emission.SetNull();
	material.Use();
}

// ********************************************************************************

void Object3D::DrawText(char *text)
{
	//ce.fontTexture->Render(text);

	wglUseFontBitmaps (hDC, 0, 256, 1000); 
	glListBase (1000); 
	glRasterPos3d(0,0,0);
	glCallLists ((int)strlen(text),GL_BYTE,text); 
	glDeleteLists(1000,1);
}
