/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "angles.h"

void Angles::Zero()
{
	isUpdatedEuler=TRUE;
	isUpdatedQuaternion=TRUE;
	euler.Zero();
}

Angles::Angles()
{
	Zero();
}

void Angles::operator= (Angles &src)
{
	quaternion=src.quaternion;
	euler=src.euler;
	isUpdatedEuler=src.isUpdatedEuler;
	isUpdatedQuaternion=src.isUpdatedQuaternion;
}

Point3D<double> Angles::GetEuler()
{
	if (!isUpdatedEuler) 
	{
		euler = quaternion.to_euler();
		isUpdatedEuler=TRUE;
	}
	return euler;
}

Angles::Angles(Point3D<double> &relativeAngles)
{
	Zero();
	Set(relativeAngles);
}

Angles::Angles(double x,double y,double z) 
{
	Zero();
	Set(x,y,z);
}

void Angles::Rotate(Point3D<double> &relativeAngles)
{
	if (!isUpdatedQuaternion) 
	{
		quaternion.from_euler(euler);
		isUpdatedQuaternion=TRUE;
	}
	Quaternion qrotate;
	qrotate.from_euler(relativeAngles);
	quaternion*=qrotate;
}

void Angles::Rotate(double x,double y,double z)
{
	if (!isUpdatedQuaternion) 
	{
		quaternion.from_euler(euler);
		isUpdatedQuaternion=TRUE;
	}
	Quaternion qrotate;
	Point3D<double> rotate(x,y,z);
	qrotate.from_euler(rotate);
	quaternion*=qrotate;
	isUpdatedEuler=FALSE;
}

void Angles::Set(Point3D<double> &relativeAngles)
{
	euler=relativeAngles;
	isUpdatedQuaternion=FALSE;
}

void Angles::Set(double x,double y,double z)
{
	euler.Set(x,y,z);
	isUpdatedQuaternion=FALSE;
}

void Angles::Add(Point3D<double> &relativeAngles)
{
	if (relativeAngles.IsNotNull()) 
	{
		euler+=relativeAngles;
		isUpdatedQuaternion=FALSE;
	}
}

void Angles::Add(double x,double y,double z)
{
	if (x||y||z) 
	{
		euler.Add(x,y,z);
		isUpdatedQuaternion=FALSE;
	}
}