/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file array.h
\brief tableau dynamique

Utilisation
\code
Array<double> tabDouble1(10),tabDouble2(10);
tabDouble1.Set(0,0.111);
tabDouble1.Set(1,1.111);
//-ou-
tabDouble1.values[0]=0.111;
tabDouble2=tabDouble1;
\endcode
\todo allocation par block pour optimiser les perfs
*/

#ifndef INCLUDE_ARRAY_H
#define INCLUDE_ARRAY_H

#include <stdio.h>
#include <windows.h>

template <class T> 
class Array
{
public:
	int size;
	T *values;

	Array()
	{
	}

	Array(int _size)
	{
		Init(_size);
	}

	~Array()
	{
		Delete();
	}

	void operator = (Array &src)
	{
		size=src.size;
	}

	void Init(int _size)
	{
		size=_size;
		values=NULL;
	}

	void New()
	{
		Delete();
		values = new T[size];
		Zero();
	}

	void Zero()
	{
		operator=(0);
	}

	//! Permet l'ecriture \code var=0; \endcode
	void operator=(int val)
	{
		for (int i=0;i<size;i++) values[i]=0;
	}

	void Delete()
	{
		if (values)
		{
			delete values;
			Init(size);
		}
	}

	BOOL Set(int num,T value)
	{
		if (!values||num>=size) return FALSE;
		values[num]=value;
		return TRUE;
	}
};

#endif