/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file box.h
\brief Gestion de Boite 3D
*/

#ifndef INCLUDE_3D_BOX
#define INCLUDE_3D_BOX

#include "coordonate.h"
#include "material.h"

/*!
\brief
Affiche une boite en 3D
*/
class Box3D
{
public:
	Material material;
	Coordonate<GLfloat> coor;
	Coordonate<GLfloat> delta;

public:
	Box3D()
	{
		Zero();
	}

	void Zero()
	{
		coor.Zero();
		delta.Zero();
		material.Zero();
	}

	void operator=(Box3D &source)
	{
		material=source.material;
		coor=source.coor;
		delta=source.delta;
	}

	void Draw()
	{
		float x1 = coor.x;
		float x2 = x1+delta.x;
		float y1 = coor.y;
		float y2 = y1+delta.y;
		float z1 = coor.z;
		float z2 = z1+delta.z;

		material.Use();
		glDisable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);
		glBegin(GL_QUADS);

		glVertex3f(x1,y1,z1);
		glVertex3f(x2,y1,z1);
		glVertex3f(x2,y2,z1);
		glVertex3f(x1,y2,z1);

		glVertex3f(x1,y1,z1);
		glVertex3f(x1,y1,z2);
		glVertex3f(x1,y2,z2);
		glVertex3f(x1,y2,z1);

		glVertex3f(x1,y1,z1);
		glVertex3f(x2,y1,z1);
		glVertex3f(x2,y1,z2);
		glVertex3f(x1,y1,z2);

		glVertex3f(x2,y1,z1);
		glVertex3f(x2,y1,z2);
		glVertex3f(x2,y2,z2);
		glVertex3f(x2,y2,z1);

		glVertex3f(x1,y2,z1);
		glVertex3f(x2,y2,z1);
		glVertex3f(x2,y2,z2);
		glVertex3f(x1,y2,z2);

		glVertex3f(x1,y1,z2);
		glVertex3f(x2,y1,z2);
		glVertex3f(x2,y2,z2);
		glVertex3f(x1,y2,z2);
		glEnd();
	}

	void CenterIt()
	{
		coor.x-=delta.x/2.0f;
		coor.y-=delta.y/2.0f;
		coor.z-=delta.z/2.0f;
	}
};

#endif