/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "3d.h"
#include "input.h"
#include "controlEngine.h"
#include "myList.h"
#include "texture.h"
#include "equations.h"
#include "button.h"

extern ControlEngine ce;

Button::Button()	
{
	// init variable
	name="Button";
	depth_min = DEFAULT_DEPTH_MIN_BUTTON;
	depth_max = DEFAULT_DEPTH_MAX_BUTTON;
	depth_cur = depth_max;
	timeOut = TIME_OUT_PUSH_DOWN_BUTTON; 
	isPushed = FALSE;
	pushLikeMode = FALSE;
	allButtonMode = FALSE;
	materialOnUp.SetDS(0.6f,0.6f,0.6f);
	materialOnDown.SetDS(0.9f,0.9f,0.9f);
	materialCurrent = &materialOnUp;

	//init du text
	text.SetAlign(align_center,align_center);
	//text.Setup(ce.fontPolygon,ft_polygon); // marche
	//text.Setup(ce.fontOutline,ft_outline); // marche
	//text.Setup(ce.fontExtrude,ft_extrude); // affichage pb de lumiere
	//text.depth=10;
	text.Setup(ce.fontTexture,ft_texture); // affiche que les textures dans un quads
	text.material.SetDS(0,0,0);

	// gestion des evenements
	onControl.mouse.left.down.connect(this,OnPushDown);
	onControl.mouse.left.up.connect(this,OnPushUp);
	onControl.over.out.connect(this,OnOverOut);
	onControl.over.in.connect(this,OnOverIn);
}

Button::~Button()
{
}

// ********************************************************************************

void Button::Create(double depth)
{
	// init general
	SetParent();
	depth_max = depth;
	depth_cur = depth;
	faceNode = ce.primitive.AddBox(name,id,&depth_cur,FALSE); 
	faceNode->Elems(FACE_FRONT)->text=&text; // attache le text
	faceNode->elem->material=*materialCurrent;
}

// ********************************************************************************

int Button::OnPushDown(StateMouse *mouse,int face)
{
	DebugMouse2("Button::OnPushDown",mouse,face);
	ce.cursor.ApplyTypes(cursorPush,cursorArrow);

	if (pushLikeMode)
	{
		SetPushToogle();
		onButton.checked(isPushed);
		return 0;
	}
	if (isPushed) timerPushDown.GetDeltaTime(TRUE);
	else
	{
		isPushed=TRUE;
		materialCurrent = &materialOnDown;
		faceNode->elem->material = *materialCurrent;
		BindTimer(timerPushDown,OnTimerPushDown,20); 
	}
	return 0;
}

// ********************************************************************************

int Button::OnTimerPushDown(Timer *timer)
{
	Debug0Param("Button::OnTimerPushDown()");
	double time = timer->GetDeltaTime();
	double depth_delta = depth_max-depth_min;

	if (time<timeOut) depth_cur=depth_min+Equ_1SURX(1.0-(time/timeOut))*depth_delta;
	else 
	{
		depth_cur=depth_min;
		timer->Stop();
	}
	return 0;
}

// ********************************************************************************

void Button::SetPush(BOOL enable)
{
	isPushed=enable;
	depth_cur=isPushed?depth_min:depth_max;
}

void Button::SetPushToogle()
{
	SetPush(!isPushed);
}

//! determine si le button est appuill ou pas (vaut aussi en mode pushLike)
BOOL Button::GetPush()
{
	return isPushed;
}

int Button::OnPushUp(StateMouse *mouse,int face)
{
	DebugMouse2("Button::OnPushUp",mouse,face);
	ce.cursor.ApplyTypes(cursorArrow,cursorPush);
	BOOL isPushedCopy = isPushed;
	if (!pushLikeMode) PushUp();
	if (isPushedCopy) onButton.click();
	return 0;
}

//! gestion du mode pushLike (toogle appuis relach a chaque click)
void Button::ModePushLike(BOOL enable)
{
	pushLikeMode = enable;
}

//! Tout les bouttons peuvent activ le click
void Button::ModeAllButtons(BOOL enable)
{
	if (allButtonMode==enable) return;
	allButtonMode = enable;
	if (enable)
	{
		onControl.mouse.mid.down.connect(this,OnPushDown);
		onControl.mouse.mid.up.connect(this,OnPushUp);
		onControl.mouse.right.down.connect(this,OnPushDown);
		onControl.mouse.right.up.connect(this,OnPushUp);
	}
	else
	{
		onControl.mouse.mid.down.disconnect(this);
		onControl.mouse.mid.up.disconnect(this);
		onControl.mouse.right.down.disconnect(this);
		onControl.mouse.right.up.disconnect(this);
	}
}

/*!
\brief Si on quit le control avec le boutton appuy ==> arret
*/

void Button::PushUp()
{
	if (isPushed)
	{
		materialCurrent = &materialOnUp;
		faceNode->elem->material = *materialCurrent;
		timerPushDown.Stop();
		depth_cur=depth_max;
		isPushed=FALSE;
	}
}

// ********************************************************************************

int Button::OnOverIn(Control *oldControl,int face)
{
	Debug2Param("Button::OnOverIn(oldCtrl:%s,face:%d)",(char *)oldControl->name,face);
	ce.cursor.ApplyTypes(cursorArrow,cursorPush,cursorNothing,cursorNothing);
	return 0;
}

int Button::OnOverOut(Control *newControl,int face)
{
	Debug2Param("Button::OnOverOut(newCtrl:%s,face:%d)",(char *)newControl->name,face);
	if (!pushLikeMode) PushUp();
	return 0;
}

// ********************************************************************************

void Button::operator=(char *newCaption)
{
	text.SetText(newCaption);
}

void Button::SetText(char *newCaption)
{
	text.SetText(newCaption);
}

// ********************************************************************************

void Button::ToString(MyString &str)
{
	str.AddFormat("name: %s caption:%s",name,caption);
}

// ********************************************************************************