/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file coordonate.h
\brief Gestion des coordonne en 3D
*/

#ifndef INCLUDE_3D_COORDONATE_H
#define INCLUDE_3D_COORDONATE_H

/*!
\brief Gestion des coordonne en 3D (compatible avec OpenGL)

valeurs: value ou (x,y,z,w)
*/
template <class T> class Coordonate
{
public:	
	union 
	{
		T value[4];
		struct 
		{
			T x;
			T y;
			T z;
			T w;
		};
	};
	BOOL isEnable;

public:
	Coordonate()
	{
		Zero();
	}

	void Zero()
	{
		x=0;
		y=0;
		z=0;
		isEnable = FALSE;
	}

	void operator=(Coordonate &src)
	{
		isEnable = src.isEnable;
		x = src.x;
		y = src.y;
		z = src.z;
		w = src.w;
	}

	void Set(T _x,T _y,T _z=0,T _w=0)
	{
		x = _x;
		y = _y;
		z = _z;
		w = _w;
		isEnable = TRUE;
	}

	BOOL operator!=(Coordonate &src)
	{
		return !operator==(src);
	}

	BOOL operator==(Coordonate &src)
	{
		return isEnable==src.isEnable&&x==src.x&&y==src.y&&z==src.z&&w==src.w;
	}

	operator T *() const 
	{
		return (T *)&value;
	}

	void ToString (MyString &source)
	{
		source<<"[enable,x,y,z,w]: "<<isEnable<<" , "<<x<<" , "<<y<<" , "<<z<<" , "<<w;
	}
};

#endif