/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

#include "3d.h"
#include "face.h"
#include "effect.h"
#include "controlEngine.h"

extern ControlEngine ce;

Effect::Effect()
{
	Bind(NULL);
}

void Effect::Bind(FaceNode *_faceNode)
{
	faceNode=_faceNode;
	isRunning=FALSE;
}

EffectApparition::EffectApparition()
{
	nbSeconds = 0;
}

EffectApparition::~EffectApparition() 
{
	Stop();
}

void EffectApparition::Start(BOOL _isFadeIn,double _nbSeconds)
{
	if (isRunning)
	{
		Stop();
	}
	isFadeIn = _isFadeIn;
	ce.timerRefresh.onTimer.connect(this,OnTimer);
	ce.timerRefresh.GetDeltaTime(TRUE,&beginTime); // conteur de temp au debut

	nbSeconds = _nbSeconds;
	isRunning = TRUE;
	oldTree = faceNode;
}

void EffectApparition::Stop()
{
	if (!isRunning) return;
	ce.timerRefresh.onTimer.disconnect(this);
	progress=1;
	UnApply(faceNode,oldTree.GetRoot(),0);
	onEnding();
	onEnding.disconnect(this);
}

int EffectApparition::OnTimer(Timer *timer)
{
	double nbSecondsCur = timer->GetDeltaTime(FALSE,&beginTime);
	progress = nbSecondsCur/nbSeconds; 
	// progress=0% si nbMilliSecondsCur=0
	// progress=100% si nbMilliSecondsCur=nbMilliseconds
	if (progress>=1) Stop();
	else Apply(faceNode,oldTree.GetRoot(),0);	
	return 0;
}

void EffectApparition::Apply(FaceNode *faceNode,FaceNode *oldFaceNode,int level)
{
	if (!faceNode) return;
	Apply(faceNode->elem,oldFaceNode->elem,level);
	for (oldFaceNode->childs.i=0,faceNode->childs.i=0;
		 faceNode->childs.i.More();
		 faceNode->childs.i++,oldFaceNode->childs.i++)
	{
		Apply(faceNode->childs.i.GetElem(),oldFaceNode->childs.i.GetElem(),level+1);
	}
}

void EffectApparition::UnApply(FaceNode *faceNode,FaceNode *oldFaceNode,int level)
{
	if (!faceNode) return;
	UnApply(faceNode->elem,oldFaceNode->elem,level);
	for (oldFaceNode->childs.i=0,faceNode->childs.i=0;
		faceNode->childs.i.More();
		faceNode->childs.i++,oldFaceNode->childs.i++)
	{
		UnApply(faceNode->childs.i.GetElem(),oldFaceNode->childs.i.GetElem(),level+1);
	}
}

void EffectApparition::Apply(Face *face,Face *oldFace,int level)
{
	double progressCur = isFadeIn? progress : 1-progress;
	face->position.z.constValue =oldFace->position.z.constValue+20.0*(level+1)*progressCur;
	if (!level) face->angle.y=oldFace->angle.y+60.0*sin(progressCur*2.0*PI);
}

void EffectApparition::UnApply(Face *face,Face *oldFace,int level)
{
	face->position.z.constValue =oldFace->position.z.constValue;
	if (!level) face->angle.y=oldFace->angle.y;
}
