/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file primitive.h
\brief Effet de transition des controls
*/

#ifndef INCLUDE_EFFECT_H
#define INCLUDE_EFFECT_H

#include "3d.h"
#include "face.h"
#include "sigslot.h"
#include "input.h"

/*! effet de transition des conrtols

Ex.:
\code
EffectAppariation effect;
effect.Setup(TRUE,control->faceNode);
effect.Start(1000);
\endcode
*/

class Effect : public has_slots<>
{
protected:
	FaceNode *faceNode;			//!< faceNode sur lequel l'effet dois etre effectu
	BOOL isFadeIn;				//!< effet FadeIn ou FadeOut
	BOOL isRunning;				//!< effet a demarer (Start)
	MyTree<Face> oldTree; //!< ancien arbre
	LARGE_INTEGER beginTime;	//!< debut du temp


public:
	signal0<> onEnding;		//!< appel a la fin de la transition

	Effect();
	void Bind(FaceNode *_faceNode);
};

class EffectApparition : public Effect
{
private:
	double nbSeconds;
	double progress;
	int OnTimer(Timer *timer);
	void Apply(FaceNode *faceNode,FaceNode *oldFace,int level);
	void Apply(Face *face,Face *oldFace,int level);
	void UnApply(FaceNode *faceNode,FaceNode *oldFace,int level);
	void UnApply(Face *face,Face *oldFace,int level);
public:

	EffectApparition();
	~EffectApparition();
	void Start(BOOL _isFadeIn,double _nbSeconds);
	void Stop();
};

/* effet a faire
EffectShow
EffectStreetch
EffectHomewold
EffectCompress
EffectRotate
EffectExplosed
*/

#endif