/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file face.h
\brief Gestion de Face
*/

#ifndef INCLUDE_3D_FACE_H
#define INCLUDE_3D_FACE_H

#include "3d.h"
#include "texture.h"
#include "curve.h"
#include "material.h"
#include "layout.h"
#include "text3d.h"

typedef Layout<GLdouble> LayoutDouble;
/*!
\brief Une face est la brique de base de l'affichage 3D

Parametres:
	- Identifi (if)
	- A des modes d'affichage (view,blending,fog,antialias)
	- Textur
	- A un materiel
	- A une position et angle par rapoort a sont "pere" (layout,angle)
	- Peut etre incurv
	- Peut etre detour (line)
*/
class Face
{
public:
	// params
	Recorder childRecorder;	//!< objet 3D a affich quelque soit le mode d'affichage de la face (peremet d'afficher le curseur par example)
	int idFace,idControl;	//!< identifiant de la face ne cours
	BOOL noPicking;			//!< ne pas utiliser dans le picking
	BOOL view;				//!< doit etre visualise
	BOOL blending;		    //!< 0=non 1=oui -1=ne change rien a l'ancien
	MyString name;			//!< info pour le picking
	Text3D *text;			//!< text qui dois etre affich
	Material material;		//!< couleurs
	Texture *texture;		//!< pointeur sur une texture
	Curve curve;			//!< parametre d'incurvation
	float line; //!< taille de la ligne de detour =0 pas de ligne, si anti alise active la taille ne change rien donc 0 ou 1 dans ce cas
	Material lineMaterial;	//!< couleurs de la ligne
	BOOL antialias;			//!< antialiase active
	BOOL fog;				//!< brouillard active

	// positionement
	Point2D<LayoutDouble> size;		//!< taille logic de la face
	Point2D<int> center;			//!< centre logic par rapport a la face courante {-1,0,1}
	Point3D<LayoutDouble> position;	//!< centre logic par rapport a la face conteneur
	Point3D<GLdouble> scale;		//!< redimentionement de la face
	Point3D<GLdouble> angle;		//!< angle de cette face par rapport a la face conteneur
	Point2D<GLdouble> computeSize;	//!< taile une fois calcul
	Point2D<GLdouble> computeCenter;//!< valeur du centre calcul
	Point3D<GLdouble> computePosition;//!< valeur de la position calcul
	double ratioXY;					//!< ratio entre les X et Y

public:
	Face();
	~Face();
	void Set (char *name,int idControl,int idFace,BOOL view);
	BOOL ConfigDynamicLayout(Face *father,double xCenter,double yCenter,double xSize,double ySize);
	void ConfigStaticLayout(double xCenter,double yCenter,double zCenter,double xSize,double ySize);
	void ConfigCenter(int xCenter,int yCenter);
	void Move(double x,double y,double z);
	void MoveTo(double x,double y,double z);
	void MovingInside();
	void Zero();
	void operator=(Face &source);
	void Draw(BOOL onPicking);
	void GetCorners(Point3D<double> *corners);
	void ComputeLayout();

};

typedef MyTreeNode<Face> FaceNode; // pour simplifier l'ecriture

#endif