/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file fog.h
\brief Gestion du brouillard
*/

#ifndef INCLUDE_3D_FOG_H
#define INCLUDE_3D_FOG_H

/*!
\brief Gestion du brouillard (charque face peut choisir ou pas de l'utiliser)

Les parametres:
	- actife (isEnable)
	- position (start,end)
	- densit (density)
	- mode
	- couleur (color)
*/

typedef enum 
{
	fogLinear = GL_LINEAR,
	fogExponential = GL_EXP,
	fogExponential2 = GL_EXP2
} FogMode; 
/*! \todo
template class <T> class IVS
{
	int index
	T value
	MyString description;
}
*/


class Fog
{
public:
	BOOL  isEnable;
	float start;
	float end;
	float density;
	FogMode mode;
	Color color;

public:
	Fog()
	{
		isEnable=FALSE;
		start=0;
		end=0;
		density=0;
		mode=fogLinear;
		color.Set(0,0,0,1.0);
	}

	void Enable(BOOL set=-1)
	{
		if (set!=-1) isEnable = set;
		if (isEnable) glEnable(GL_FOG);
		else glDisable(GL_FOG);
	}

	void Use()
	{
		Enable();
		glFogfv(GL_FOG_COLOR,color);
		glFogf(GL_FOG_START,start);
		glFogf(GL_FOG_END,end); 
		glFogf(GL_FOG_DENSITY,density); 
		glFogi(GL_FOG_MODE,(int)mode);
	}

	void GetStringMode (MyString &result)
	{
		switch (mode)
		{
		case fogLinear: result="Linear"; break;
		case fogExponential: result="Exponential"; break;
		case fogExponential2: result="Exponential"; break;
		default: result="Error";
		}
	}

	int GetIndexMode()
	{
		switch (mode)
		{
		case fogLinear: return 0;
		case fogExponential: return 1;
		case fogExponential2: return 2;
		default: return -1;
		}
	}

	void SetIndexMode(int index)
	{
		FogMode listMode[]={fogLinear,fogExponential,fogExponential2};
		mode = listMode[index];
	}
};

#endif