#include "Matrix.h"

#include "Vector.h"

#include <math.h>

const matrix O(0, 0, 0, 0,
               0, 0, 0, 0,
               0, 0, 0, 0,
               0, 0, 0, 0);   // Empty matrix

const matrix I(1, 0, 0, 0,
               0, 1, 0, 0,
               0, 0, 1, 0,
               0, 0, 0, 1);   // Identity matrix

matrix::matrix(float i)
{
   m11 = i; m12 = 0; m13 = 0; m14 = 0;
   m21 = 0; m22 = i; m23 = 0; m24 = 0;
   m31 = 0; m32 = 0; m33 = i; m34 = 0;
   m41 = 0; m42 = 0; m43 = 0; m44 = i;
}

matrix::matrix(float Am11, float Am12, float Am13, 
					float Am21, float Am22, float Am23, 
					float Am31, float Am32, float Am33)
{
   m11 = Am11; m12 = Am12; m13 = Am13; m14 = 0;
   m21 = Am21; m22 = Am22; m23 = Am23; m24 = 0;
   m31 = Am31; m32 = Am32; m33 = Am33; m34 = 0;
   m41 = 0;    m42 = 0;    m43 = 0;    m44 = 1;
}

matrix::matrix(float Am11, float Am12, float Am13, float Am14, 
					float Am21, float Am22, float Am23, float Am24, 
					float Am31, float Am32, float Am33, float Am34, 
					float Am41, float Am42, float Am43, float Am44)
{
   m11 = Am11; m12 = Am12; m13 = Am13; m14 = Am14;
   m21 = Am21; m22 = Am22; m23 = Am23; m24 = Am24;
   m31 = Am31; m32 = Am32; m33 = Am33; m34 = Am34;
   m41 = Am41; m42 = Am42; m43 = Am43; m44 = Am44;
}

matrix matrix::operator+() const
{
   return *this;
}

matrix matrix::operator-() const
{
   return matrix(-m11, -m12, -m13, -m14,
                 -m21, -m22, -m23, -m24,
                 -m31, -m32, -m33, -m34,
                 -m41, -m42, -m43, -m44);
}

matrix matrix::operator!() const
{
   matrix A;

   float M3344 = m33 * m44 - m43 * m34;
   float M2344 = m23 * m44 - m43 * m24;
   float M2334 = m23 * m34 - m33 * m24;
   float M3244 = m32 * m44 - m42 * m34;
   float M2244 = m22 * m44 - m42 * m24;
   float M2234 = m22 * m34 - m32 * m24;
   float M3243 = m32 * m43 - m42 * m33;
   float M2243 = m22 * m43 - m42 * m23;
   float M2233 = m22 * m33 - m32 * m23;
   float M1344 = m13 * m44 - m43 * m14;
   float M1334 = m13 * m34 - m33 * m14;
   float M1244 = m12 * m44 - m42 * m14;
   float M1234 = m12 * m34 - m32 * m14;
   float M1243 = m12 * m43 - m42 * m13;
   float M1233 = m12 * m33 - m32 * m13;
   float M1324 = m13 * m24 - m23 * m14;
   float M1224 = m12 * m24 - m22 * m14;
   float M1223 = m12 * m23 - m22 * m13;

   A.m11 = m22 * M3344 - m32 * M2344 + m42 * M2334;
   A.m21 = -m21 * M3344 + m31 * M2344 - m41 * M2334;
   A.m31 = m21 * M3244 - m31 * M2244 + m41 * M2234;
   A.m41 = -m21 * M3243 + m31 * M2243 - m41 * M2233;

   A.m12 = -m12 * M3344 + m32 * M1344 - m42 * M1334;
   A.m22 = m11 * M3344 - m31 * M1344 + m41 * M1334;
   A.m32 = -m11 * M3244 + m31 * M1244 - m41 * M1234;
   A.m42 = m11 * M3243 - m31 * M1243 + m41 * M1233;

   A.m13 = m12 * M2344 - m22 * M1344 + m42 * M1324;
   A.m23 = -m11 * M2344 + m21 * M1344 - m41 * M1324;
   A.m33 = m11 * M2244 - m21 * M1244 + m41 * M1224;
   A.m43 = -m11 * M2243 + m21 * M1243 - m41 * M1223;

   A.m14 = -m12 * M2334 + m22 * M1334 - m32 * M1324;
   A.m24 = m11 * M2334 - m21 * M1334 + m31 * M1324;
   A.m34 = -m11 * M2234 + m21 * M1234 - m31 * M1224;
   A.m44 = m11 * M2233 - m21 * M1233 + m31 * M1223;

   A /= m11 * A.m11 +
        m21 * A.m12 +
        m31 * A.m13 +
        m41 * A.m14;   // Determinant

   return A;
}

matrix matrix::operator~() const
{
   return matrix(m11, m21, m31, m41,
                 m12, m22, m32, m42,
                 m13, m23, m33, m43,
                 m14, m24, m34, m44);
}

matrix &matrix::operator=(float i)
{
   m11 = i; m12 = i; m13 = i; m14 = i;
   m21 = i; m22 = i; m23 = i; m24 = i;
   m31 = i; m32 = i; m33 = i; m34 = i;
   m41 = i; m42 = i; m43 = i; m44 = i;

	return *this;
}

matrix &matrix::operator=(const matrix &A)
{
   m11 = A.m11; m12 = A.m12; m13 = A.m13; m14 = A.m14;
   m21 = A.m21; m22 = A.m22; m23 = A.m23; m24 = A.m24;
   m31 = A.m31; m32 = A.m32; m33 = A.m33; m34 = A.m34;
   m41 = A.m41; m42 = A.m42; m43 = A.m43; m44 = A.m44;

   return *this;
}

matrix &matrix::operator+=(const matrix &A)
{
   m11 += A.m11; m12 += A.m12; m13 += A.m13; m14 += A.m14;
   m21 += A.m21; m22 += A.m22; m23 += A.m23; m24 += A.m24;
   m31 += A.m31; m32 += A.m32; m33 += A.m33; m34 += A.m34;
   m41 += A.m41; m42 += A.m42; m43 += A.m43; m44 += A.m44;

   return *this;
}

matrix &matrix::operator-=(const matrix &A)
{
   m11 -= A.m11; m12 -= A.m12; m13 -= A.m13; m14 -= A.m14;
   m21 -= A.m21; m22 -= A.m22; m23 -= A.m23; m24 -= A.m24;
   m31 -= A.m31; m32 -= A.m32; m33 -= A.m33; m34 -= A.m34;
   m41 -= A.m41; m42 -= A.m42; m43 -= A.m43; m44 -= A.m44;

   return *this;
}

matrix &matrix::operator*=(float r)
{
   m11 *= r; m12 *= r; m13 *= r; m14 *= r;
   m21 *= r; m22 *= r; m23 *= r; m24 *= r;
   m31 *= r; m32 *= r; m33 *= r; m34 *= r;
   m41 *= r; m42 *= r; m43 *= r; m44 *= r;

   return *this;
}

matrix &matrix::operator*=(const matrix &A)
{
   return *this = *this * A;
}

matrix &matrix::operator/=(float r)
{
   m11 /= r; m12 /= r; m13 /= r; m14 /= r;
   m21 /= r; m22 /= r; m23 /= r; m24 /= r;
   m31 /= r; m32 /= r; m33 /= r; m34 /= r;
   m41 /= r; m42 /= r; m43 /= r; m44 /= r;

   return *this;
}

matrix &matrix::operator/=(const matrix &A)
{
   return *this = *this * !A;
}

float matrix::operator[](int i) const
{
   return *((float*)(&m11 + i));
}

vector &matrix::operator()(int i) const
{
   return *((vector*)(&m11 + (((i - 1) & 3) << 2)));
}

float &matrix::operator()(int i, int j) const
{
   return *((float*)(&m11 + (((i - 1) & 3) << 2) + ((j - 1) & 3)));
}

bool operator==(const matrix &A, const matrix &B)
{
   if(A.m11 == B.m11 && A.m12 == B.m12 && A.m13 == B.m13 && A.m14 == B.m14 &&
      A.m21 == B.m21 && A.m22 == B.m22 && A.m23 == B.m23 && A.m24 == B.m24 &&
      A.m31 == B.m31 && A.m32 == B.m32 && A.m33 == B.m33 && A.m34 == B.m34 &&
      A.m41 == B.m41 && A.m42 == B.m42 && A.m43 == B.m43 && A.m44 == B.m44)
      return true;
   else
      return false;
}

bool operator!=(const matrix &A, const matrix &B)
{
   if(A.m11 == B.m11 && A.m12 == B.m12 && A.m13 == B.m13 && A.m14 == B.m14 &&
      A.m21 == B.m21 && A.m22 == B.m22 && A.m23 == B.m23 && A.m24 == B.m24 &&
      A.m31 == B.m31 && A.m32 == B.m32 && A.m33 == B.m33 && A.m34 == B.m34 &&
      A.m41 == B.m41 && A.m42 == B.m42 && A.m43 == B.m43 && A.m44 == B.m44)
      return false;
   else
      return true;
}

matrix operator+(const matrix &A, const matrix &B)
{
   return matrix(A.m11 + B.m11, A.m12 + B.m12, A.m13 + B.m13, A.m14 + B.m14,
                 A.m21 + B.m21, A.m22 + B.m22, A.m23 + B.m23, A.m24 + B.m24,
                 A.m31 + B.m31, A.m32 + B.m32, A.m33 + B.m33, A.m34 + B.m34,
                 A.m41 + B.m41, A.m42 + B.m42, A.m43 + B.m43, A.m44 + B.m44);
}

matrix operator-(const matrix &A, const matrix &B)
{
   return matrix(A.m11 - B.m11, A.m12 - B.m12, A.m13 - B.m13, A.m14 - B.m14,
                 A.m21 - B.m21, A.m22 - B.m22, A.m23 - B.m23, A.m24 - B.m24,
                 A.m31 - B.m31, A.m32 - B.m32, A.m33 - B.m33, A.m34 - B.m34,
                 A.m41 - B.m41, A.m42 - B.m42, A.m43 - B.m43, A.m44 - B.m44);
}

matrix operator*(const matrix &A, float r)
{
   return matrix(A.m11 * r, A.m12 * r, A.m13 * r, A.m14 * r,
                 A.m21 * r, A.m22 * r, A.m23 * r, A.m24 * r,
                 A.m31 * r, A.m32 * r, A.m33 * r, A.m34 * r,
                 A.m41 * r, A.m42 * r, A.m43 * r, A.m44 * r);
}

matrix operator*(float r, const matrix &A)
{
   return matrix(r * A.m11, r * A.m12, r * A.m13, r * A.m14,
                 r * A.m21, r * A.m22, r * A.m23, r * A.m24,
                 r * A.m31, r * A.m32, r * A.m33, r * A.m34,
                 r * A.m41, r * A.m42, r * A.m43, r * A.m44);
}

matrix operator*(const matrix &A, const matrix &B)
{
   return matrix(A.m11 * B.m11 + A.m12 * B.m21 + A.m13 * B.m31 + A.m14 * B.m41, A.m11 * B.m12 + A.m12 * B.m22 + A.m13 * B.m32 + A.m14 * B.m42, A.m11 * B.m13 + A.m12 * B.m23 + A.m13 * B.m33 + A.m14 * B.m43, A.m11 * B.m14 + A.m12 * B.m24 + A.m13 * B.m34 + A.m14 * B.m44,
                 A.m21 * B.m11 + A.m22 * B.m21 + A.m23 * B.m31 + A.m24 * B.m41, A.m21 * B.m12 + A.m22 * B.m22 + A.m23 * B.m32 + A.m24 * B.m42, A.m21 * B.m13 + A.m22 * B.m23 + A.m23 * B.m33 + A.m24 * B.m43, A.m21 * B.m14 + A.m22 * B.m24 + A.m23 * B.m34 + A.m24 * B.m44,
                 A.m31 * B.m11 + A.m32 * B.m21 + A.m33 * B.m31 + A.m34 * B.m41, A.m31 * B.m12 + A.m32 * B.m22 + A.m33 * B.m32 + A.m34 * B.m42, A.m31 * B.m13 + A.m32 * B.m23 + A.m33 * B.m33 + A.m34 * B.m43, A.m31 * B.m14 + A.m32 * B.m24 + A.m33 * B.m34 + A.m34 * B.m44,
                 A.m41 * B.m11 + A.m42 * B.m21 + A.m43 * B.m31 + A.m44 * B.m41, A.m41 * B.m12 + A.m42 * B.m22 + A.m43 * B.m32 + A.m44 * B.m42, A.m41 * B.m13 + A.m42 * B.m23 + A.m43 * B.m33 + A.m44 * B.m43, A.m41 * B.m14 + A.m42 * B.m24 + A.m43 * B.m34 + A.m44 * B.m44);
}

matrix operator/(const matrix &A, float r)
{
   return matrix(A.m11 / r, A.m12 / r, A.m13 / r, A.m14 / r,
                 A.m21 / r, A.m22 / r, A.m23 / r, A.m24 / r,
                 A.m31 / r, A.m32 / r, A.m33 / r, A.m34 / r,
                 A.m41 / r, A.m42 / r, A.m43 / r, A.m44 / r);
}

matrix operator/(float r, const matrix &A)
{
   return r * !A;
}

matrix operator/(const matrix &A, const matrix &B)
{
   return A * !B;
}

vector &operator*=(vector &V, const matrix &A)
{
   return V = V * A;
}

vector operator*(const matrix &A, const vector &V)
{
   return vector(A.m11 * V.x + A.m12 * V.y + A.m13 * V.z + A.m14, A.m21 * V.x + A.m22 * V.y + A.m23 * V.z + A.m24, A.m31 * V.x + A.m32 * V.y + A.m33 * V.z + A.m34);
}

vector operator*(const vector &V, const matrix &A)
{
   return vector(V.x * A.m11 + V.y * A.m21 + V.z * A.m31 + A.m41, V.x * A.m12 + V.y * A.m22 + V.z * A.m32 + A.m42, V.x * A.m13 + V.y * A.m23 + V.z * A.m33 + A.m43);
}

float det(const matrix &A)
{
   float M3344 = A.m33 * A.m44 - A.m43 * A.m34;
   float M2344 = A.m23 * A.m44 - A.m43 * A.m24;
   float M2334 = A.m23 * A.m34 - A.m33 * A.m24;
   float M1344 = A.m13 * A.m44 - A.m43 * A.m14;
   float M1334 = A.m13 * A.m34 - A.m33 * A.m14;
   float M1324 = A.m13 * A.m24 - A.m23 * A.m14;

   return A.m11 * (A.m22 * M3344 - A.m32 * M2344 + A.m42 * M2334) -
          A.m21 * (A.m12 * M3344 - A.m32 * M1344 + A.m42 * M1334) +
          A.m31 * (A.m12 * M2344 - A.m22 * M1344 + A.m42 * M1324) -
          A.m41 * (A.m12 * M2334 - A.m22 * M1334 + A.m32 * M1324);
}

float det(float Am11)
{
	return Am11;
}

float det(float Am11, float Am12,
			 float Am21, float Am22)
{
	return Am11 * Am22 - Am12 * Am21; 
}

float det(float Am11, float Am12, float Am13, 
			 float Am21, float Am22, float Am23, 
			 float Am31, float Am32, float Am33)
{
   return Am11 * (Am22 * Am33 - Am32 * Am23) -
          Am21 * (Am12 * Am33 - Am32 * Am13) +
          Am31 * (Am12 * Am23 - Am22 * Am13);
}

float det(float Am11, float Am12, float Am13, float Am14, 
			 float Am21, float Am22, float Am23, float Am24, 
			 float Am31, float Am32, float Am33, float Am34, 
			 float Am41, float Am42, float Am43, float Am44)
{
   float M3344 = Am33 * Am44 - Am43 * Am34;
   float M2344 = Am23 * Am44 - Am43 * Am24;
   float M2334 = Am23 * Am34 - Am33 * Am24;
   float M1344 = Am13 * Am44 - Am43 * Am14;
   float M1334 = Am13 * Am34 - Am33 * Am14;
   float M1324 = Am13 * Am24 - Am23 * Am14;

   return Am11 * (Am22 * M3344 - Am32 * M2344 + Am42 * M2334) -
          Am21 * (Am12 * M3344 - Am32 * M1344 + Am42 * M1334) +
          Am31 * (Am12 * M2344 - Am22 * M1344 + Am42 * M1324) -
          Am41 * (Am12 * M2334 - Am22 * M1334 + Am32 * M1324);
}

float tr(const matrix &A)
{
	return A.m11 + A.m22 + A.m33 + A.m44;
}