#ifndef MATRIX_H
#define MATRIX_H

class vector;

class matrix
{
   float m11; float m12; float m13; float m14;
   float m21; float m22; float m23; float m24;
   float m31; float m32; float m33; float m34;
   float m41; float m42; float m43; float m44;

public:
   matrix(){};
	matrix(float i);
   matrix(float Am11, float Am12, float Am13,
          float Am21, float Am22, float Am23,
          float Am31, float Am32, float Am33);
   matrix(float Am11, float Am12, float Am13, float Am14,
          float Am21, float Am22, float Am23, float Am24,
          float Am31, float Am32, float Am33, float Am34,
          float Am41, float Am42, float Am43, float Am44);

   matrix operator+() const;
   matrix operator-() const;

   matrix operator!() const;   // Inverse
   matrix operator~() const;   // Transpose

	matrix &operator=(float i);
   matrix &operator=(const matrix &A);

   matrix &operator+=(const matrix &A);
   matrix &operator-=(const matrix &A);
   matrix &operator*=(float r);
   matrix &operator*=(const matrix &A);
   matrix &operator/=(float r);
   matrix &operator/=(const matrix &A);

	float operator[](int i) const;   // Access elements sequentially (row major), starting with 0

	vector &operator()(int i) const;   // Row vector
   float &operator()(int i, int j) const;   // Access elements (row, col) starting with (1, 1)

   friend bool operator==(const matrix &A, const matrix &B);
   friend bool operator!=(const matrix &A, const matrix &B);

   friend matrix operator+(const matrix &A, const matrix &B);
   friend matrix operator-(const matrix &A, const matrix &B);
   friend matrix operator*(const matrix &A, float r);
   friend matrix operator*(float r, const matrix &A);
   friend matrix operator*(const matrix &A, const matrix &B);
   friend matrix operator/(const matrix &A, float r);

   friend matrix operator/(float r, const matrix &A);

   friend matrix operator/(const matrix &A, const matrix &B);

   friend vector& operator*=(vector &V, const matrix &A);

   friend vector operator*(const matrix &A, const vector &V);
   friend vector operator*(const vector &V, const matrix &A);

   friend float det(const matrix &A);
	friend float det(float Am11);
	friend float det(float Am11, float Am12,
	                 float Am21, float Am22);
   friend float det(float Am11, float Am12, float Am13,
                    float Am21, float Am22, float Am23,
                    float Am31, float Am32, float Am33);
   friend float det(float Am11, float Am12, float Am13, float Am14,
                    float Am21, float Am22, float Am23, float Am24,
                    float Am31, float Am32, float Am33, float Am34,
                    float Am41, float Am42, float Am43, float Am44);

	friend float tr(const matrix &A); 
};

extern const matrix O;   // Empty matrix
extern const matrix I;   // Identity matrix

#endif

