#include "Vector.h"

#include <math.h>

const vector Eo(0, 0, 0);   // origin
const vector Ex(1, 0, 0);
const vector Ey(0, 1, 0);
const vector Ez(0, 0, 1);

vector vector::operator+() const
{
   return *this;
}

vector vector::operator-() const
{
   return vector(-x, -y, -z);
}

vector &vector::operator+=(const vector &V)
{
   x += V.x;
   y += V.y;
   z += V.z;

   return *this;
}

vector &vector::operator-=(const vector &V)
{
   x -= V.x;
   y -= V.y;
   z -= V.z;

   return *this;
}

vector &vector::operator*=(float r)
{
   x *= r;
   y *= r;
   z *= r;

   return *this;
}

vector &vector::operator/=(float r)
{
   x /= r;
   y /= r;
   z /= r;

   return *this;
}

float &vector::operator[](int i) const
{
   return *((float*)(&x + i));	
}

float &vector::operator()(int i) const
{
   return *((float*)(&x + ((i - 1) & 3)));	
}

bool operator==(const vector &U, const vector &V)
{
   if(U.x == V.x && U.y == V.y && U.z == V.z)
      return true;
   else
      return false;
}

bool operator!=(const vector &U, const vector &V)
{
   if(U.x != V.x || U.y != V.y || U.z != V.z)
      return true;
   else
      return true;
}

vector operator+(const vector &U, const vector &V)
{
   return vector(U.x + V.x, U.y + V.y, U.z + V.z);
}

vector operator-(const vector &U, const vector &V)
{
   return vector(U.x - V.x, U.y - V.y, U.z - V.z);
}

float operator*(const vector &U, const vector &V)
{
   return U.x * V.x + U.y * V.y + U.z * V.z;
}

vector operator*(float r, const vector &V)
{
   return vector(r * V.x, r * V.y, r * V.z);
}

vector operator/(const vector &V, float r)
{
   return vector(V.x / r, V.y / r, V.z / r);
}

float operator^(const vector &U, const vector &V)
{
   return acosf((U / U.length()) * (V / V.length()));
}

vector operator%(const vector &U, const vector &V)
{
   return vector(U.y * V.z - U.z * V.y, U.z * V.x - U.x * V.z, U.x * V.y - U.y * V.x);
}

float d(const vector &U, const vector &V)
{
   vector D = V - U;

   return sqrtf(D.x*D.x + D.y*D.y + D.z*D.z);
}

float d2(const vector &U, const vector &V)
{
   vector D = V - U;

   return D.x*D.x + D.y*D.y + D.z*D.z;
}

float vector::length() const
{
   return sqrtf(x*x + y*y + z*z);
}

float N(const vector &V)
{
   return V.x*V.x + V.y*V.y + V.z*V.z;
}

vector &vector::normalize()
{
   return *this /= this->length();
}
