#ifndef VECTOR_H
#define VECTOR_H

class vector
{
public:
	float x;
	float y;
	float z;

   vector(){};
	vector(float i);
   vector(const vector &V);
   vector(float Vx, float Vy, float Vz);

   vector operator+() const;
   vector operator-() const;

	vector &operator=(float i);
   vector &operator=(const vector &V);

   vector &operator+=(const vector &V);
   vector &operator-=(const vector &V);
   vector &operator*=(float r);
   vector &operator/=(float r);

	float &operator[](int j) const;

	float &operator()(int j) const;

   friend bool operator==(const vector &U, const vector &V);
   friend bool operator!=(const vector &U, const vector &V);

   friend vector operator+(const vector &U, const vector &V);
   friend vector operator-(const vector &U, const vector &V);
   friend float operator*(const vector &U, const vector &V);   // Dot product
   friend vector operator*(float r, const vector &V);
   friend vector operator/(const vector &V, float r);
   friend float operator^(const vector &U, const vector &V);   // Angle between vectors

   friend vector operator%(const vector &U, const vector &V);   // Cross product

   friend float d(const vector&, const vector&);   // Distance between two points
	friend float d2(const vector&, const vector&);   // Squared distance between two points
	friend float N(const vector &V);   // Norm

   float length() const;

   vector &normalize();
};

inline vector::vector(float i)
{
   x = i;
   y = i;
   z = i;
}

inline vector::vector(const vector &V)
{
   x = V.x;
   y = V.y;
   z = V.z;
}

inline vector &vector::operator=(float i)
{
   x = y = z = i;

   return *this;
}

inline vector &vector::operator=(const vector &V)
{
   x = V.x;
   y = V.y;
   z = V.z;

   return *this;
}

inline vector::vector(float Vx, float Vy, float Vz)
{
	x = Vx;
	y = Vy;
	z = Vz;
}

extern const vector Eo;   // origin
extern const vector Ex;
extern const vector Ey;
extern const vector Ez;

#endif
