#ifndef QUATERNION_H
#define QUATERNION_H

#include "vector.h"
#include "../trigo.h"

class matrix;

class Quaternion
{
public:
	float w;
	union
	{
		struct
		{
			float x;
			float y;
			float z;
		};
		struct
		{
			vector v;
		};
	};

	Quaternion(){};
	Quaternion(float s);
	Quaternion(const vector &V);
	Quaternion(const Quaternion &Q);
	Quaternion(float s, const vector &V);
	Quaternion(const matrix &A);   // Must be a rotation matrix
	Quaternion(float Qw, float Qx, float Qy, float Qz);

	operator float();
	operator vector();
	operator matrix();

	Quaternion operator+() const;
	Quaternion operator-() const;

	Quaternion operator!() const;   // Inverse
	Quaternion operator~() const;   // Conjugate

	Quaternion &operator=(const Quaternion &Q);

	Quaternion &operator+=(const Quaternion &V);
	Quaternion &operator-=(const Quaternion &V);
	Quaternion &operator*=(float r);
	Quaternion &operator*=(const Quaternion &Q);
	Quaternion &operator/=(float r);
	Quaternion &operator/=(const Quaternion &Q);

	friend bool operator==(const Quaternion &Q1, const Quaternion &Q2);
	friend bool operator!=(const Quaternion &Q1, const Quaternion &Q2);

	friend Quaternion operator+(const Quaternion &Q1, const Quaternion &Q2);
	friend Quaternion operator-(const Quaternion &Q1, const Quaternion &Q2);
	friend Quaternion operator*(const Quaternion &Q1, const Quaternion &Q2);
	friend Quaternion operator*(float r, const Quaternion &Q);
	friend Quaternion operator*(const Quaternion &Q, float r);
	friend Quaternion operator/(const Quaternion &Q, float r);
	friend Quaternion operator/(const Quaternion &Q1, const Quaternion &Q2);

	friend float N(const Quaternion &Q);   // Norm

	Quaternion &normalize();

	void from_euler(Point3D<double> &axis);
	void from_axisangle(vector &axis,double angle);
	Point3D<double> to_euler();
	//void to_euler(Point3D<double> *axis,double *angle);
};

extern const Quaternion i;   // Imaginary unit
extern const Quaternion j;
extern const Quaternion k;

#endif