/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file layout.h
\brief Gestion de positionnement
*/

#ifndef INCLUDE_LAYOUT_H
#define INCLUDE_LAYOUT_H

/*!
\brief 	Gestion des positionnements des controls

Ex.:
\code
Layout var;
double base;
var.Set(0,1,&base);
float res = var.Compute(); // res = 1; (0% de 100 + +1)
var.Set(0.5,-1);
base=100;
float res = var.Compute(); // res = 49; (50% de 100 + -1)
\endcode
*/
template <class T> class Layout // T peut etre de preferance float ou double
{
public:
	T constValue;
	double percentValue;
	T *baseValue;

public:
	Layout()
	{
		Zero();
	}

	void Zero()
	{
		constValue=0;
		percentValue=0;
		baseValue=NULL;
	}

	/*!
	\brief permet d'acceder directement a la composant constante dy layout
	\code
	onLayout=0;
	\endcode
	Remarque: Indispensable dans les templates qui initialise par "value=0;" (cf.: Point3D<Layout>)
	*/
	void operator=(int value)
	{
		constValue = value;
	}

	void operator=(GLdouble value)
	{
		constValue = value;
	}

	void operator=(Layout &source)
	{
		constValue = source.constValue;
		percentValue = source.percentValue;
		baseValue = source.baseValue;
	}

	void Set(T _constValue,double _percentValue=0,T *_baseValue=NULL) 
	{
		constValue = _constValue;
		SetPercent(_percentValue,_baseValue);
	}

	void SetPercent(double _percentValue=0,T *_baseValue=NULL)
	{
		percentValue = _percentValue;
		baseValue = _baseValue;
	}

	BOOL IsDynamic()
	{
		return baseValue?TRUE:FALSE;
	}

	/*! \todo a revoir completement
	void Move(T delta)
	{
		constValue+=delta;
	}

	void MoveTo(T value)
	{
		constValue=value;
	}

	void Scale(T delta)
	{
		percentValue*=(double)delta;
	}
*/
	T Compute()
	{
		if (!percentValue || !baseValue) return constValue;
		return constValue+*baseValue*percentValue;
	}
};

#endif