/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html	
*********************************************************************************** */

#include "light.h"

BOOL Light::useLighting;

void Light::SetLighting(BOOL set)
{
	useLighting = set;
	if (set) glEnable(GL_LIGHTING);
	else glDisable(GL_LIGHTING);
}

BOOL Light::IsLighting() 
{ 
	return useLighting; 
}

Light::Light()
{
	Zero();
	id = 0;
}

Light::Light(int _id)
{
	Zero();
	id = _id;
}

void Light::Zero()
{
	id = GL_LIGHT0;
	isEnable = TRUE;
	material.Zero();
	direction.Zero();
	position.Zero();
	cutoff.isEnable=FALSE;
	exponent.isEnable=FALSE;
	attenuationCst.isEnable=FALSE;
	attenuationLin.isEnable=FALSE;
	attenuationQuad.isEnable=FALSE;
}

void Light::operator=(Light &source)
{
	isEnable = source.isEnable;
	material = source.material;

	direction = source.direction;
	position = source.position;

	cutoff = source.cutoff;
	exponent = source.exponent;

	attenuationCst = source.attenuationCst;
	attenuationLin = source.attenuationLin;
	attenuationQuad = source.attenuationQuad;
}

void Light::Enable(BOOL active) // -1 affiche ou pas suivant le mode precedent , 0 desactive et 1 active
{
	if (active!=-1) isEnable = active;
	if (isEnable) glEnable (id);
	else glDisable (id);
}

BOOL Light::IsEnabled() 
{ 
	return isEnable; 
} 

void Light::Toggle () 
{ 
	Enable(!IsEnabled()); 
}

void Light::Use()
{
	Enable();
	if (!isEnable) return;
	if (material.ambient.isEnable)	glLightfv(id,GL_AMBIENT,material.ambient);
	if (material.diffuse.isEnable)	glLightfv(id,GL_DIFFUSE,material.diffuse);
	if (material.specular.isEnable)	glLightfv(id,GL_SPECULAR,material.specular);
	if (cutoff.isEnable)			glLightf(id,GL_SPOT_CUTOFF,cutoff.elem);
	if (exponent.isEnable)			glLightf(id,GL_SPOT_EXPONENT,exponent.elem);
	if (attenuationCst.isEnable)	glLightf(id,GL_CONSTANT_ATTENUATION,attenuationCst.elem);
	if (attenuationLin.isEnable)	glLightf(id,GL_CONSTANT_ATTENUATION,attenuationLin.elem);
	if (attenuationQuad.isEnable)	glLightf(id,GL_CONSTANT_ATTENUATION,attenuationQuad.elem);
	if (direction.isEnable)			glLightfv(id,GL_SPOT_DIRECTION,direction);
	if (position.isEnable)			glLightfv(id,GL_POSITION,position);
}