/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
http://www.gnu.org/copyleft/gpl.html	
*********************************************************************************** */

/*! \file light.h
\brief Gestion des lumieres
*/

#ifndef INCLUDE_3D_LIGHT_H
#define INCLUDE_3D_LIGHT_H

#include "3d.h"
#include "variable.h"
#include "material.h"
#include "coordonate.h"

/*!
\brief Gestion d'une lumiere

Les parametres:
	- cutoff
	- exponent
	- attenuation
	- position
	- direction
*/
class Light
{
public:	
	static BOOL useLighting;
	int id;
	BOOL isEnable;
	Enabling<GLfloat> cutoff;
	Enabling<GLfloat> exponent;
	Enabling<GLfloat> attenuationCst;
	Enabling<GLfloat> attenuationLin;
	Enabling<GLfloat> attenuationQuad;

public:
	Material material;
	Coordonate<GLfloat> direction;
	Coordonate<GLfloat> position;

	static void SetLighting(BOOL set);
	static BOOL IsLighting();
	Light();
	Light(int _id);
	void Zero();
	void operator=(Light &source);
	void Enable(BOOL active=-1); //!<< -1 affiche ou pas suivant le mode precedent , 0 desactive et 1 active
	BOOL IsEnabled();
	void Toggle();
	void Use();
};


#endif