/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

#include "material.h"

Material::Material()
{
	Zero();
}

void Material::Zero()
{
	typeFace = GL_FRONT_AND_BACK;
	ambient.Zero();
	//ambient.opacity=&opacity;
	diffuse.Zero();
	//diffuse.opacity=&opacity;
	specular.Zero();
	//specular.opacity=&opacity;
	emission.Zero();
	//emission.opacity=&opacity;
	shiness.elem=0;
	//opacity=1; // gestion a faire
}

void Material::operator=(Material &source)
{
	typeFace = source.typeFace;
	ambient = source.ambient;
	diffuse = source.diffuse;
	specular = source.specular;
	emission = source.emission;
	shiness = source.shiness;
	opacity = source.opacity;
}

void Material::SetDS (Color color)
{
	ambient.SetNull();
	diffuse=color;
	specular=diffuse;
	emission.SetNull();
}

void Material::SetDS (float *source)
{
	SetDS(source[0],source[1],source[2],source[3]);
}

void Material::SetAlpha(float alpha)
{
	ambient.a = alpha;
	diffuse.a = alpha;
	specular.a = alpha;
	emission.a = alpha;
}

/*! set diffuse and specular */
void Material::SetDS (float r,float g,float b,float a) 
{
	ambient.SetNull();
	diffuse.Set(r,g,b,a);
	specular=diffuse;
	emission.SetNull();
}

void Material::SetDS (float r,float g,float b) 
{
	ambient.SetNull();
	diffuse.Set(r,g,b);
	specular=diffuse;
	emission.SetNull();
}

void Material::SetNull ()
{
	ambient.SetNull();
	diffuse.SetNull();
	specular.SetNull();
	emission.SetNull();
}

void Material::SetFull(BOOL includeEmission)
{
	ambient.SetFull();
	diffuse.SetFull();
	specular.SetFull();
	if (includeEmission) emission.SetFull();
}
void Material::Use()
{
	if (ambient.isEnable)	glMaterialfv(typeFace,GL_AMBIENT,ambient);
	if (diffuse.isEnable)	glMaterialfv(typeFace,GL_DIFFUSE,diffuse);
	if (specular.isEnable)	glMaterialfv(typeFace,GL_SPECULAR,specular);
	if (emission.isEnable)	glMaterialfv(typeFace,GL_EMISSION,emission);
	if (shiness.isEnable) 	glMateriali (typeFace,GL_SHININESS,shiness.elem);
}

void Material::Record()
{
	recorder.BeginRecord(FALSE);
	Use();
	recorder.EndRecord();
}

void Material::Play()
{
	recorder.Play();
}

/*!
\todo shiness mistere
*/

void Material::As(TypeMaterial mat) 
{
	int num = mat%NB_MATERIAL;
	AsMaterial *material = &materials[num];
	ambient=material->ambient;
	diffuse=material->diffuse;
	specular=material->specular;
	//shiness=material->shininess; // sinon fait des trucs bizard
	shiness.Set(0);
}