/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file material.h
\brief Gestion des materiaux (couleur,type etc ...)
*/

#ifndef INCLUDE_3D_MATERIAL_H
#define INCLUDE_3D_MATERIAL_H

#include "3d.h"
#include "color.h"
#include "variable.h"
#include "mystring.h"
#include "recorder.h"


//! element de materials
typedef struct 
{
	MyString name;
	float ambient[4];
	float diffuse[4];
	float specular[4];
	float shininess;
} AsMaterial;

#define NB_MATERIAL 24
//! type de materials, pour la methode "As"
typedef enum
{
	mat_brass = 0,
	mat_bronze,
	mat_polished_bronze,
	mat_chrome,
	mat_copper,
	mat_polished_copper,
	mat_gold,
	mat_polished_gold,
	mat_pewter,
	mat_silver,
	mat_polished_silver,
	mat_emerald,
	mat_jade,
	mat_obsidian,
	mat_pearl,
	mat_ruby,
	mat_turquoise,
	mat_black_plastic,
	mat_black_rubber,
	mat_cyan_rubber,
	mat_green_rubber,
	mat_red_rubber,
	mat_white_rubber,
	mat_yellow_rubber
} TypeMaterial;

/*!
\brief Gestion du materiel (notion OpenGL)

Parametres:
- couleur: Ambiente,Speculaire,Diffuse,Emission
- luminosit (shiness)
- typeFace [GL_FRONT_AND_BACK]

\todo gestion opacity a faire
*/

class Material
{
public:	
	int typeFace;
	Color ambient;
	Color diffuse;
	Color specular;
	Color emission;
	Enabling<GLint> shiness; //!<0 par default
	float opacity;
	Recorder recorder;		//!< recorder pour acceleration openGL
public:
	Material();
	void Zero();
	void operator=(Material &source);
	void SetDS (Color color);
	void SetDS (float *source);
	void SetDS (float r,float g,float b);
	void SetDS (float r,float g,float b,float a);
	void SetAlpha(float alpha);
	void SetNull ();
	void SetFull(BOOL includeEmission=FALSE);
	void Use();
	void As(TypeMaterial mat);
	void Record();
	void Play();
};

/* source http://www.cs.utk.edu/~kuck/materials_ogl.htm -et-
http://devernay.free.fr/cours/opengl/materials.html */
//! list de tout les materiel classique
static AsMaterial materials[]= 
{ 
	{	
		"Brass", // laiton
		{0.329412f,		0.223529f,	0.027451f,	1.0f},
		{0.780392f,		0.568627f,	0.113725f,	1.0f},
		{0.992157f,		0.941176f,	0.807843f,	1.0f}, 
		27.8974f
	},{
		"Bronze", // bronze
		{0.2125f,	0.1275f,	0.054f,		1.0f},
		{0.714f,	0.4284f,	0.18144f,	1.0f},
		{0.393548f, 0.271906f,	0.166721f,	1.0f},
		25.6f
	},{
		"Polished Bronze", // bronze polie
		{0.25f,		0.148f,		0.06475f,	1.0f},
		{0.4f,		0.2368f,	0.1036f,	1.0f},
		{0.774597f, 0.458561f,	0.200621f,	1.0f},
		76.8f
	},{
		"Chrome", // chrome
		{0.25f,		0.25f,		0.25f,		1.0f},
		{0.4f,		0.4f,		0.4f,		1.0f},
		{0.774597f, 0.774597f,	0.774597f,	1.0f},
		76.8f
	},{
		"Copper", // cuivre 
		{0.19125f,	0.0735f,	0.0225f,	1.0f},
		{0.7038f,	0.27048f,	0.0828f,	1.0f},
		{0.256777f, 0.137622f,	0.086014f,	1.0f},
		12.8f
	},{
		"Polished Copper", // cuivre polie
		{0.2295f,	0.08825f,	0.0275f,	1.0f},
		{0.5508f,	0.2118f,	0.066f,		1.0f},
		{0.580594f, 0.223257f,	0.0695701f, 1.0f},
		51.2f
	},{
		"Gold", // or
		{0.24725f,	0.1995f,	0.0745f,	1.0f},
		{0.75164f,	0.60648f,	0.22648f,	1.0f},
		{0.628281f,	0.555802f,	0.366065f,	1.0f},
		51.2f
	},{
		"Polished Gold", // or polie
		{0.24725f,	0.2245f,	0.0645f,	1.0f},
		{0.34615f,	0.3143f,	0.0903f,	1.0f},
		{0.797357f,	0.723991f,	0.208006f,	1.0f},
		83.2f
	},{
		"Pewter", // etain
		{0.105882f,	0.058824f,	0.113725f,	1.0f},
		{0.427451f, 0.470588f,	0.541176f,	1.0f}, 
		{0.333333f, 0.333333f,	0.521569f,	1.0f},
		9.84615f
	},{
		"Silver", // argent
		{0.19225f,	0.19225f,	0.19225f,	1.0f},
		{0.50754f,	0.50754f,	0.50754f ,	1.0f},
		{0.508273f,	0.508273f,	0.508273f,	1.0f},
		51.2f
	},{
		"Polished Silver", // argent polie
		{0.23125f,	0.23125f,	0.23125f,	1.0f},
		{0.2775f,	0.2775f,	0.2775f,	1.0f},
		{0.773911f, 0.773911f,	0.773911f,	1.0f},
		89.6f
	},{
		"Emerald", // emeraude
		{0.0215f,	0.1745f,	0.0215f,	0.55f},
		{0.07568f,	0.61424f,	0.07568f,	0.55f},
		{0.633f,	0.727811f,	0.633f,		0.55f},
		76.8f
	},{
		"Jade", // jade
		{0.135f,	0.2225f,	0.1575f,	0.95f},
		{0.54f,		0.89f,		0.63f,		0.95f},
		{0.316228f, 0.316228f,	0.316228f,	0.95f},
		12.8f
	},{
		"Obsidian", // obsidien
		{0.05375f,	0.05f,		0.06625f,	0.82f},
		{0.18275f,	0.17f,		0.22525f,	0.82f},
		{0.332741f, 0.328634f,	0.346435f,	0.82f},
		38.4f
	},{
		"Pearl", // perle
		{0.25f,		0.20725f,	0.20725f,	0.922f},
		{1.0f,		0.829f,		0.829f,		0.922f},
		{0.296648f, 0.296648f,	0.296648f,	0.922f},
		11.264f
	},{
		"Ruby", // ruby
		{0.1745f,	0.01175f,	0.01175f,	0.55f},
		{0.61424f,	0.04136f,	0.04136f,	0.55f},
		{0.727811f, 0.626959f,	0.626959f,	0.55f},
		76.8f
	},{
		"Turquoise", // turquoise
		{0.1f,		0.18725f,	0.1745f,	0.8f},
		{0.396f,	0.74151f,	0.69102f,	0.8f},
		{0.297254f, 0.30829f,	0.306678f,	0.8f},
		12.8f
	},{
		"Black Plastic", // plasique noir
		{0.0f,		0.0f,		0.0f,		1.0f},
		{0.01f,		0.01f,		0.01f,		1.0f},
		{0.50f,		0.50f,		0.50f,		1.0f},
		32.0f
	},{
		"Black Rubber", // caoutchouc noir
		{0.02f,		0.02f,		0.02f,		1.0f},
		{0.01f,		0.01f,		0.01f,		1.0f},
		{0.4f,		0.4f,		0.4f,		1.0f},
		10.0f
	},{	
		"cyan plastic", // plasitique cyan
		{0.0f,			0.1f,			0.06f,			1.0f},
		{0.0f,			0.50980392f,	0.50980392f,	1.0f},
		{0.50196078f,	0.50196078f,	0.50196078f,	1.0f},
		32.0f
	},{
		"green plastic", // plastique vert
		{0.0f,		0.0f,		0.0f,		1.0f},
		{0.1f,		0.35f,		0.1f,		1.0f}, 
		{0.45f,		0.55f,		0.45f,		1.0f},
		32.0f
	},{
		"red plastic",	// plastique rouge
		{0.0f,		0.0f,		0.0f,		1.0f},
		{0.5f,		0.0f,		0.0f,		1.0f},
		{0.7f,		0.6f,		0.6f,		1.0f},
		32.0f
	},{
		"white plastic", // plastique blanc
		{0.0f,		0.0f,		0.0f,		1.0f},
		{0.55f,		0.55f,		0.55f,		1.0f},
		{0.70f,		0.70f,		0.70f,		1.0f},
		32.0f
	},{
		"yellow plastic", // plastique jaune
		{0.0f,		0.0f,		0.0f,		1.0f},
		{0.5f,		0.5f,		0.0f,		1.0f},
		{0.60f,		0.60f,		0.50f,		1.0f},
		32.0f
	}, {
		"cyan rubber", // caoutchouc cyan
		{0.0f,		0.05f,		0.05f,		1.0f},
		{0.4f,		0.5f,		0.5f,		1.0f},
		{0.04f,		0.7f,		0.7f,		1.0f},
		10.0f
	}, {
		"green rubber", // caoutchouc vert
		{0.0f,		0.05f,		0.0f,		1.0f},
		{0.4f,		0.5f,		0.4f,		1.0f},
		{0.04f,		0.7f,		0.04f,		1.0f},
		10.0f
	}, {
		"red rubber", // caoutchouc rouge
		{0.05f,		0.0f,		0.0f,		1.0f},
		{0.5f,		0.4f,		0.4f,		1.0f},
		{0.7f,		0.04f,		0.04f,		1.0f},
		10.0f
	}, {
		"white rubber", // caoutchouc blanc
		{0.05f,		0.05f,		0.05f,		1.0f},
		{0.5f,		0.5f,		0.5f,		1.0f},
		{0.7f,		0.7f,		0.7f,		1.0f},
		10.0f
	}, {
		"yellow rubber", // caoutchouc jaune
		{0.05f,		0.05f,		0.0f,		1.0f},
		{0.5f,		0.5f,		0.4f,		1.0f},
		{0.7f,		0.7f,		0.04f,		1.0f},
		10.0f
	}
};

#endif