/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html	
*********************************************************************************** */

#include "perfMon.h"

BOOL PerfMonitor::Save()
{
	FILE *fich = fopen(fileName,"wt");
	if (!fich) return FALSE;

	for (points.i=0;points.i.More();points.i++)
	{
		PerfMonitorPoint *point = points.i.GetElemPtr();
		fprintf(fich,"%s \t",(char *)point->name);
	}

	for (records.i=0;records.i.More();records.i++)
	{
		fprintf(fich,"\n");
		MyListOfDouble *recordCur = records.i.GetElemPtr();
		for (recordCur->i=0;recordCur->i.More();recordCur->i++)
			fprintf(fich,"%lf \t",recordCur->i.GetElem());
	}
	return TRUE;
}

PerfMonitor::PerfMonitor()
{
	enable=TRUE;
	fileName="perfMon.txt";
}

PerfMonitor::~PerfMonitor()
{
	if (enable) Save();
}

PerfMonitorPoint * PerfMonitor::AddTitle(char *name)
{
	PerfMonitorPoint *point = points.i.AddNewLast();
	point->Set(name,this);
	return point;
}

void PerfMonitor::AddRecord()
{
	if (! enable) return;
	MyListOfDouble *listValue = records.i.AddNewLast();

	// ajout dans la list des records
	for (points.i=0;points.i.More();points.i++) 
		listValue->i.AddLast(points.i.GetElemPtr()->value);
}

//! Reset de toute les values

void PerfMonitor::Reset()
{
	for (points.i=0;points.i.More();points.i++) 
		points.i.GetElemPtr()->Reset();
}

PerfMonitorPoint * PerfMonitor::Find(char *name,BOOL addIfNotExist)
{
	PerfMonitorPoint *pt;
	for (points.i=0;points.i.More();points.i++)
	{
		pt = points.i.GetElemPtr();
		if (pt->name==name) return pt;
	}

	if (addIfNotExist) return AddTitle(name);
	return NULL;
}

void PerfMonitor::Reset(char *name)
{
	if (!enable) return;
	PerfMonitorPoint * pt = Find(name,TRUE);
	pt->Reset();
}

void PerfMonitor::Begin(char *name,BOOL reset)
{
	if (!enable) return;
	PerfMonitorPoint * pt = Find(name,TRUE);
	pt->Begin(reset);
}

void PerfMonitor::End(char *name)
{
	if (!enable) return;
	PerfMonitorPoint * pt = Find(name,TRUE);
	pt->End();
}

// **************************************************************************

PerfMonitorPoint::PerfMonitorPoint()
{
	monitor=NULL;
	name="not used";
	Reset();
}
void PerfMonitorPoint::operator=(PerfMonitorPoint &src)
{
	name=src.name;
	timer=src.timer;
	monitor=src.monitor;
	value=src.value;
	nbValue=src.nbValue;
}

PerfMonitorPoint::PerfMonitorPoint(char *_name,PerfMonitor *_monitor)
{
	Set(_name,_monitor);			
}

void PerfMonitorPoint::Set(char *_name,PerfMonitor *_monitor)
{
	name=_name;
	monitor=_monitor;
}

void PerfMonitorPoint::Reset()
{
	value=0;
	nbValue=0;
}

void PerfMonitorPoint::Begin(BOOL reset)
{
	if (reset) Reset();
	timer.GetDeltaTime(TRUE);
}

void PerfMonitorPoint::End()
{
	value+=timer.GetDeltaTime();
	nbValue++;
}