/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
http://www.gnu.org/copyleft/gpl.html	
*********************************************************************************** */

/*! \file perfMon.h
\brief monteur de performance
*/

#ifndef INCLUDE_PERFMON_H
#define INCLUDE_PERFMON_H

#include "input.h"
#include "MyList.h"


/*!
system de monitoring de performance

Ex.1: le system le plus rapide, mais necessite le prototipage des variables ce qui peut etre fastidieux
\code
PerfMonitor perfMon;
PerfMonitorPoint *perf_first;
PerfMonitorPoint *perf_seconde;
perf_first = PerfMon.AddTitle("first");
perf_seconde = PerfMon.AddTitle("seconde");
//...
perf_first->Begin();
perf_seconde->Begin();
//...
perf_first->End();
//...
perf_seconde->End();
perfMon.AddRecord();
//...
\code

Ex.2: le system facil a utiliser => plus lent !attention: les denomination doivent etre exact
\code
PerfMonitor perfMon;
//...
perfMon->Begin("first");
perfMon->Begin("seconde");
//...
perfMon->End("first);
//...
perfMon->End("second");
perfMon.AddRecord();
//...
\code
*/

class PerfMonitorPoint;

typedef MyList<double> MyListOfDouble;
class PerfMonitor
{
private:
	MyList<MyListOfDouble> records;
	MyList<PerfMonitorPoint> points;
	BOOL Save();

public:
	BOOL enable;
	MyString fileName;

	PerfMonitor();
	~PerfMonitor();

	PerfMonitorPoint * AddTitle(char *name); //!< ajout une column en donne sont indice
	void AddRecord();
	void Reset(); 

	PerfMonitorPoint * Find(char *name,BOOL addIfNotExist);
	void Reset(char *name);
	void Begin(char *name,BOOL reset=FALSE);
	void End(char *name);
};

class PerfMonitorPoint
{
public:
	MyString name;
	Timer timer;
	PerfMonitor *monitor;
	double value;
	int nbValue;

public:
	PerfMonitorPoint();
	PerfMonitorPoint(char *_name,PerfMonitor *_monitor);
	void Set(char *_name,PerfMonitor *_monitor);
	void operator=(PerfMonitorPoint &src);

	void Reset();
	void Begin(BOOL reset=FALSE);
	void End();
};

#endif