/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file primitive.h
\brief Primitive graphics
*/

#ifndef INCLUDE_3D_PRIMITIVE_H
#define INCLUDE_3D_PRIMITIVE_H

#include "3d.h"
#include "layout.h"
#include "face.h"
#include "control.h"

/*!
\brief Gestion des Primitive graphics (ajout de face, de cube, etc ...)
*/

class Primitive
{
private:
	MyStack<FaceNode *> stack;
	FaceNode * AddBox (char *name,int idControl,BOOL isDynamic,GLdouble *depth,BOOL viewBack);	
	void GetCornersRecursive(FaceNode *faceNode,BOOL isObjective,Point3D<double> *points);

public:
	MyTree<Face> tree;

	Primitive();
	~Primitive();
	void Push();			//!< Empile le node en cours
	void Pop();				//!< Depile le node en cours
	FaceNode * Peek();		//!< Donne le node en cours

	FaceNode * GoTo(int childNumber);
	FaceNode * GoTo(FaceNode *node);
	FaceNode * GoToParent();

	void ViewDebug(FaceNode *node=NULL);
	BOOL Draw(BOOL isPicking);
	BOOL Draw(FaceNode *node,BOOL isPicking);
	FaceNode * Find(int idControl,int idFace,FaceNode *node=NULL);
	void BeginContainer(int idControl,double xConstCenter,double xPercentCenter,double yConstCenter,double yPercentCenter,double xConstSize,double xPercentSize,double yConstSize,double yPercentSize); // debut du conteneur
	void EndContainer(); // fin du conteneur
	FaceNode * AddFace (char *name,int idControl,int idFace,BOOL view);
	FaceNode * AddFace (char *name,int idControl,int idFace,BOOL view,double xConstCenter,double xPercentCenter,double yConstCenter,double yPercentCenter,double zConstCenter,double xConstSize,double xPercentSize,double yConstSize,double yPercentSize);
	FaceNode * AddBox (char *name,int idControl,GLdouble depth,BOOL viewBack);   // creation avec depth static
	FaceNode * AddBox (char *name,int idControl,GLdouble *depth,BOOL viewBack);  // creation avec depth dynamic
	Point2D<double> GetLocalPos(FaceNode *faceNode,Point2D<double> absolutePos);
	void GetCorners(FaceNode *faceNode,Point3D<double> *points);
	void ComputeLayout(FaceNode *faceNode);
	BOOL ReOrderForBlending(FaceNode *faceNode);
	void ReOrderForBlending();


};

#endif