/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file slider.h
\brief Control Slider
*/

#ifndef INCLUDE_SLIDER_H
#define INCLUDE_SLIDER_H

#include "control.h"
#include "input.h"

class EventSlider
{ 
public:
	signal3<double,double,double> moving;	//!< Moving(double min,double max,double percent)
};

#define MIN_SIZE_CURSOR 10
#define SIZE_BUTTON		12

/*!
\brief
Class de parametre et d'evenement d'un "Slider"
*/

class Slider : public Control
{
private:
	double maximum;
	double minimum;
	double curMax;
	double curMin;
	double stepButton;		//!< valeur a ajouter ou retirer sur appuis boutton
	BOOL useButtons;
	void ConfigFaces(Control *control,Texture *texture);
	void ApplyCurrent();
	void Init(double min,double max,double curMin,double curMax);
	BOOL SetMin(double value);
	BOOL SetMax(double value);
	BOOL SetCurrent(double min,double max);
	BOOL SetCurrent(double delta);
	void SetCurrentByPosition(double x);
	void SendMoving();

public:
	// variable membre
	EventSlider onSlidder;
	Button btnPlus;
	Button btnMinus;
	Control cursor;
	Control body;
	Timer timerAutoButton;
	
	Slider();
	~Slider();

	void Create(double depth);

	BOOL Step(double step);
	void Setup(double min,double max,double current,double size);
	BOOL Min(double value);
	BOOL Max(double value);
	BOOL Current(double min,double max);
	BOOL Current(double delta);
	void Refresh();

	int OnAutoMinus(Timer *);
	int OnAutoPlus(Timer *);
	int OnButtonsOverOut(Control *,int);
	int OnButtonsLMouseUp(StateMouse *,int);
	int OnMinusLMouseDown(StateMouse *,int);
	int OnPlusLMouseDown(StateMouse *,int);
	int OnLMouseDown(StateMouse *mouse,int face);
	int OnMouseMove(StateMouse *mouse,int deltaX,int deltaY,int face);

	void ToString(MyString &str);
};

#endif