/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include <windows.h>
#include "MyMenu.h"
#include "resource.h"
#include "systray.h"
#include "valueEditor.h"
#include "controlEngine.h"
#include "communTools.h"

extern ControlEngine ce;

LRESULT WINAPI SysTrayProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HICON hIcon;
	static NOTIFYICONDATA nid;

	const UINT trayId = 1;
	const UINT WM_MYTRAY = WM_APP + 1;

	HMENU hMenu;
	static CMyMenu menu;
	POINT pt;

	switch(message)
	{
	case WM_MEASUREITEM: menu.MeasureItem((LPMEASUREITEMSTRUCT)lParam); return TRUE;
	case WM_DRAWITEM: menu.DrawItem((LPDRAWITEMSTRUCT)lParam); return TRUE;
	case WM_CREATE:
		hIcon = (HICON)LoadImage(
			ce.hInst,
			MAKEINTRESOURCE(IDI_ICON_SYSTRAY),
			IMAGE_ICON,
			GetSystemMetrics(SM_CXSMICON),
			GetSystemMetrics(SM_CYSMICON),
			0);

		if (!hIcon) { ErrorMessage(TRUE, MB_ICONSTOP, NULL); return -1; }

		ZeroMemory(&nid, sizeof(NOTIFYICONDATA));
		nid.cbSize = sizeof(NOTIFYICONDATA);
		nid.hIcon = hIcon;
		nid.hWnd = hWnd;
		nid.uID = trayId;
		nid.uCallbackMessage = WM_MYTRAY;
		lstrcpy(nid.szTip, TEXT("3D Controls"));
		nid.uFlags = NIF_MESSAGE | NIF_TIP | NIF_ICON;
		Shell_NotifyIcon(NIM_ADD, &nid);
		return 0;

	case WM_QUIT:
	case WM_DESTROY:
		Shell_NotifyIcon(NIM_DELETE, &nid);
		PostQuitMessage(0);
		return 0;

	case WM_MYTRAY:
		switch (lParam)
		{
		case WM_RBUTTONDOWN:
			hMenu = LoadMenu(ce.hInst,MAKEINTRESOURCE(IDC_MENU));

			if(hMenu)
			{
				hMenu = GetSubMenu(hMenu, 0);
				menu = hMenu;
				menu.AddTitle(TITLE_NAME);
				//InsertMenu(hMenu,0, MF_BYPOSITION | MF_OWNERDRAW | MF_STRING | MF_DISABLED, 0, "ShortCutKey");
				if(hMenu)
				{
					//CheckMenuItem(hMenu,IDM_START,MF_BYCOMMAND | isCheck ? MF_CHECKED : MF_UNCHECKED);
					GetCursorPos(&pt);
					SetForegroundWindow(hWnd);
					TrackPopupMenu(
						hMenu,
						TPM_RIGHTALIGN | TPM_VCENTERALIGN | TPM_RIGHTBUTTON,
						pt.x,pt.y,0,
						hWnd,NULL);
				}
			}
			else ErrorMessage(TRUE, MB_ICONEXCLAMATION, NULL);
			break;

		case WM_LBUTTONDBLCLK:
			ShowWindow(CreateDialogParam(ce.hInst,MAKEINTRESOURCE(IDD_DEBUG_STRUCT),NULL,(DLGPROC)DebugStructProc,NULL),SW_SHOW);
			break;
		}
		return 0;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case ID_POPUP_DEBUGER:
			ShowWindow(CreateDialogParam(ce.hInst,MAKEINTRESOURCE(IDD_DEBUG_STRUCT),NULL,(DLGPROC)DebugStructProc,NULL),SW_SHOW);
			return 0;

		case ID_POPUP_ABOUT:
			DialogBox(ce.hInst,MAKEINTRESOURCE(IDD_ABOUT),NULL,(DLGPROC)AboutProc);
			return 0;

		case ID_POPUP_HELP:
			MyOpenBrowser(URL_3D_CONTROLS);
			return 0;

		case ID_POPUP_CONFIG:
			DialogBox(ce.hInst,MAKEINTRESOURCE(IDD_CONFIG),NULL,(DLGPROC)ConfigProc);
			return 0;

		case ID_POPUP_EXIT:
			PostMessage(hWnd, WM_CLOSE, 0, 0);
			return 0;
		}
		break;
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}

LRESULT WINAPI AboutProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		{
			MyString title;
			title.Format("3D Controls v%s - Copyright 2003-2004",VERSION);
			Static_SetText(GetDlgItem(hWnd,IDC_STATIC_VERSION),title);
		}		
		return TRUE;

	case WM_COMMAND:
		switch (wParam)
		{
		case IDC_BUTTON_MAILME:
			MailTo(MAIL_ADRESS);
			return TRUE;

		case IDC_BUTTON_URL:
			MyOpenBrowser(URL_3D_CONTROLS);
			return TRUE;

		case IDC_BUTTON_FORUM:
			MyOpenBrowser(URL_FORUM);
			return TRUE;

		case IDC_BUTTON_LICENCE:
			MyOpenBrowser(URL_LICENCE);
			return TRUE;

		case IDOK:
		case IDCANCEL: EndDialog(hWnd, LOWORD(wParam)); return TRUE;
		}
	}
	return FALSE;
}

// message custom
#define WM_REFRESH			WM_USER+1973
#define WM_REFRESH_EDIT		WM_USER+1974
#define WM_REFRESH_SPIN		WM_USER+1975
#define WM_READ_FROM_EDIT	WM_USER+1976

BOOL SetSpinPos(HWND hWnd,int id,int pos)
{
	return (BOOL)SendMessage(GetDlgItem(hWnd,id),UDM_SETPOS32,0,pos);
}
//! \todo delta de la fonction non implement
BOOL SetSpinRange(HWND hWnd,int id,int minPos,int maxPos,int delta)
{
	return (BOOL)SendMessage(GetDlgItem (hWnd,id), UDM_SETRANGE32,minPos,maxPos);
}

int GetSpinPos(HWND hWnd)
{
	return (BOOL)SendMessage(hWnd,UDM_GETPOS32,0,0);
}

LRESULT WINAPI ConfigProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static char temp[1024];
	HWND hWndCtrl;
	/*		
		enableAntiAliasing",&enableAntiAliasing
	*/
	switch (message)
	{
	case WM_REFRESH_SPIN:
		SetSpinPos(hDlg,IDC_SPIN_FOG_START,(int)ce.fog.start);
		SetSpinPos(hDlg,IDC_SPIN_FOG_END,(int)ce.fog.end);
		SetSpinPos(hDlg,IDC_SPIN_FOG_DENSITY,(int)ce.fog.density);
		SetSpinPos(hDlg,IDC_SPIN_XSIZE,(int)ce.dxWin);
		SetSpinPos(hDlg,IDC_SPIN_YSIZE,(int)ce.dyWin);
		SetSpinPos(hDlg,IDC_SPIN_FRAMERATE,ce.refreshTime);
		SetSpinPos(hDlg,IDC_SPIN_CPU,ce.cpuLimit);
		return TRUE;

	case WM_READ_FROM_EDIT:
		Edit_GetText(GetDlgItem(hDlg,IDC_RICHEDIT_FOG_START),temp,1024);
		sscanf(temp,"%f",&ce.fog.start);
		Edit_GetText(GetDlgItem(hDlg,IDC_RICHEDIT_FOG_END),temp,1024);
		sscanf(temp,"%f",&ce.fog.end);
		Edit_GetText(GetDlgItem(hDlg,IDC_RICHEDIT_FOG_DENSITY),temp,1024);
		sscanf(temp,"%f",&ce.fog.density);
		Edit_GetText(GetDlgItem(hDlg,IDC_RICHEDIT_FRAMERATE),temp,1024);
		sscanf(temp,"%d",&ce.timerRefresh);
		Edit_GetText(GetDlgItem(hDlg,IDC_RICHEDIT_CPU),temp,1024);
		sscanf(temp,"%d",&ce.cpuLimit);
		Edit_GetText(GetDlgItem(hDlg,IDC_RICHEDIT_XSIZE),temp,1024);
		sscanf(temp,"%d",&ce.dxWin);
		Edit_GetText(GetDlgItem(hDlg,IDC_RICHEDIT_YSIZE),temp,1024);
		sscanf(temp,"%d",&ce.dyWin);
		PostMessage(hDlg,WM_REFRESH_SPIN,NULL,NULL);
		return TRUE;

	case WM_REFRESH_EDIT:
		sprintf(temp,"%.2f",ce.fog.start);
		Edit_SetText(GetDlgItem(hDlg,IDC_RICHEDIT_FOG_START),temp);
		sprintf(temp,"%.2f",ce.fog.end);
		Edit_SetText(GetDlgItem(hDlg,IDC_RICHEDIT_FOG_END),temp);
		sprintf(temp,"%.2f",ce.fog.density);
		Edit_SetText(GetDlgItem(hDlg,IDC_RICHEDIT_FOG_DENSITY),temp);
		sprintf(temp,"%d",ce.timerRefresh);
		Edit_SetText(GetDlgItem(hDlg,IDC_RICHEDIT_FRAMERATE),temp);
		sprintf(temp,"%d",ce.cpuLimit);
		Edit_SetText(GetDlgItem(hDlg,IDC_RICHEDIT_CPU),temp);
		sprintf(temp,"%d",ce.dxWin);
		Edit_SetText(GetDlgItem(hDlg,IDC_RICHEDIT_XSIZE),temp);
		sprintf(temp,"%d",ce.dyWin);
		Edit_SetText(GetDlgItem(hDlg,IDC_RICHEDIT_YSIZE),temp);
		return TRUE;

	case WM_REFRESH:
		ComboBox_SetCurSel(GetDlgItem(hDlg,IDC_COMBO_FOG_MODE),ce.fog.GetIndexMode());
		ComboBox_SetCurSel(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_BACK),ce.polygonMode.back);
		ComboBox_SetCurSel(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_FRONT),ce.polygonMode.front);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_LIGHT),Light::IsLighting());
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_LIGHT0),ce.lights[0].isEnable);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_LIGHT1),ce.lights[1].isEnable);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_LIGHT2),ce.lights[2].isEnable);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_LIGHT0_MOVE),ce.lightsMove[0]);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_LIGHT1_MOVE),ce.lightsMove[1]);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_LIGHT2_MOVE),ce.lightsMove[2]);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_VIEW_BACKGROUND),ce.enableBackGround);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_VIEW_FPS),ce.enableFPS);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_VIEW_REPERE),ce.enableRepere);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_VIEW_INERTIA),ce.enableInertia);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_MONITOR),ce.monitor.enable);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_VIEW_LIGHT),ce.enableLightView);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_VIEW_REGION),ce.screenMain.region.enable);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_PICKING),ce.enablePicking);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_FOG),ce.fog.isEnable);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_TITLED),ce.screenMain.enableTitle);
		Button_SetCheck(GetDlgItem(hDlg,IDC_CHECK_INERTIA),ce.enableViewInertia);
		PostMessage(hDlg,WM_REFRESH_EDIT,NULL,NULL);
		return TRUE;

	case WM_INITDIALOG:
		ce.screenConfig.hWnd=hDlg;
		// config des SPIN
		SetSpinRange(hDlg,IDC_SPIN_FOG_START,-10,10,1);
		SetSpinRange(hDlg,IDC_SPIN_FOG_END,-10,10,1);
		SetSpinRange(hDlg,IDC_SPIN_FOG_DENSITY,-10,10,1);
		SetSpinRange(hDlg,IDC_SPIN_XSIZE,50,1280,50);
		SetSpinRange(hDlg,IDC_SPIN_YSIZE,50,1280,50);
		SetSpinRange(hDlg,IDC_SPIN_FRAMERATE,0,200,5);
		SetSpinRange(hDlg,IDC_SPIN_CPU,10,100,10);

		// config des combo
		ComboBox_ResetContent(GetDlgItem(hDlg,IDC_COMBO_FOG_MODE));
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_FOG_MODE),"linear");
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_FOG_MODE),"exp");
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_FOG_MODE),"exp");

		ComboBox_ResetContent(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_FRONT));
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_FRONT),"fill");
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_FRONT),"line");
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_FRONT),"dot");
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_FRONT),"nothing");

		ComboBox_ResetContent(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_BACK));
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_BACK),"fill");
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_BACK),"line");
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_BACK),"dot");
		ComboBox_AddString(GetDlgItem(hDlg,IDC_COMBO_POLYGONE_BACK),"nothing");
		PostMessage(hDlg,WM_REFRESH,NULL,NULL);
		PostMessage(hDlg,WM_REFRESH_SPIN,NULL,NULL);

		return TRUE;

	case WM_VSCROLL: // spin vertical
	case WM_HSCROLL: // spin horizontal
		if ((int)LOWORD(wParam) == SB_THUMBPOSITION) 
		{
			int num = GetSpinPos((HWND)lParam);
			switch (GetDlgCtrlID((HWND)lParam))
			{
			case IDC_SPIN_XSIZE: ce.dxWin = num; break;
			case IDC_SPIN_YSIZE: ce.dyWin = num; break;
			case IDC_SPIN_FRAMERATE: ce.refreshTime = num; break;
			case IDC_SPIN_CPU: ce.cpuLimit= num; break;
			case IDC_SPIN_FOG_START: ce.fog.start = (float)num; break;
			case IDC_SPIN_FOG_END: ce.fog.end = (float)num; break;
			case IDC_SPIN_FOG_DENSITY: ce.fog.density = (float)num; break;
			default: return FALSE;
			}
			PostMessage(hDlg,WM_REFRESH_EDIT,NULL,NULL);
			return TRUE;
		}
		break;

	case WM_COMMAND:
		int id;
		id = LOWORD(wParam);
		hWndCtrl = GetDlgItem(hDlg,id);
		switch (id)
		{
			// LIGHT
		case IDC_CHECK_LIGHT: Light::SetLighting(Button_GetCheck(hWndCtrl)); return TRUE;
		case IDC_CHECK_LIGHT0: ce.lights[0].isEnable=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_LIGHT1: ce.lights[1].isEnable=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_LIGHT2: ce.lights[2].isEnable=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_LIGHT0_MOVE: ce.lightsMove[0]=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_LIGHT1_MOVE: ce.lightsMove[1]=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_LIGHT2_MOVE: ce.lightsMove[2]=Button_GetCheck(hWndCtrl); return TRUE;
			// VIEW
		case IDC_CHECK_VIEW_INERTIA: ce.enableViewInertia=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_VIEW_BACKGROUND: ce.enableBackGround=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_VIEW_FPS: ce.enableFPS=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_VIEW_REPERE: ce.enableRepere=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_VIEW_LIGHT: ce.enableLightView=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_VIEW_REGION: ce.screenMain.region.enable=Button_GetCheck(hWndCtrl); return TRUE;
			// PICKING
		case IDC_CHECK_PICKING:ce.enablePicking=Button_GetCheck(hWndCtrl); return TRUE;
		//IDC_STATIC_PICKING_FULL:
		//IDC_STATIC_PICKING_SELECTED:
			// FOG
		case IDC_CHECK_FOG: ce.fog.Enable(Button_GetCheck(hWndCtrl)); return TRUE;
		case IDC_RICHEDIT_FOG_START:  
		case IDC_RICHEDIT_FOG_END: 
		case IDC_RICHEDIT_FOG_DENSITY: 
			PostMessage(hDlg,WM_READ_FROM_EDIT,NULL,NULL); return TRUE;
		case IDC_COMBO_FOG_MODE: //Edit_GetText(hWndCtrl,temp,1024); ce.fog.mode=atof(temp); return TRUE;
			return TRUE;
			// RENDER
		case IDC_COMBO_POLYGONE_FRONT:// ce.polygonMode.front
		case IDC_COMBO_POLYGONE_BACK:// ce.polygonMode.back
			return TRUE;
		case IDC_CHECK_TITLED: ce.screenMain.enableTitle=Button_GetCheck(hWndCtrl); ce.screenMain.Update(TRUE); return TRUE;
		case IDC_RADIO_FULLSCREEN: ce.screenMain.enableFullScreen=TRUE; ce.screenMain.Update(TRUE); return TRUE;
		case IDC_RADIO_CUSTOMSCREEN: ce.screenMain.enableFullScreen=FALSE; ce.screenMain.Update(TRUE); return TRUE;
		case IDC_RICHEDIT_XSIZE: 						
		case IDC_RICHEDIT_YSIZE:
		case IDC_RICHEDIT_FRAMERATE:
		case IDC_RICHEDIT_CPU:
			PostMessage(hDlg,WM_READ_FROM_EDIT,NULL,NULL); return TRUE;
			return TRUE;
			// TOOLS
		case IDC_CHECK_INERTIA: ce.enableInertia=Button_GetCheck(hWndCtrl); return TRUE;
		case IDC_CHECK_MONITOR: ce.monitor.enable=Button_GetCheck(hWndCtrl); return TRUE;
			// GENERAL
		case IDOK:
		case IDCANCEL: 
			ce.screenConfig.hWnd=NULL;
			EndDialog(hDlg, LOWORD(wParam)); 
			return TRUE;

		case IDC_BUTTON_DEFAULT:
			ce.lightsMove[0]=DEFAULT_LIGHT_MOVE0;
			ce.lightsMove[1]=DEFAULT_LIGHT_MOVE1;
			ce.lightsMove[2]=DEFAULT_LIGHT_MOVE2;
			ce.lights[0].isEnable=DEFAULT_LIGHT_ENABLE0;
			ce.lights[1].isEnable=DEFAULT_LIGHT_ENABLE1;
			ce.lights[2].isEnable=DEFAULT_LIGHT_ENABLE2;

			ce.fog.isEnable=DEFAULT_FOG_ENABLE;
			ce.fog.start=DEFAULT_FOG_START;
			ce.fog.end=DEFAULT_FOG_END;
			ce.fog.density=DEFAULT_FOG_DENSITY;
			ce.fog.mode=DEFAULT_FOG_MODE;

			ce.enableAntiAliasing=DEFAULT_ANTIALIASING;
			ce.enableRepere=DEFAULT_VIEW_REPERE;
			ce.enableLightView=DEFAULT_VIEW_LIGHT;
			ce.enablePicking=DEFAULT_PICKING;
			ce.enableDelta=DEFAULT_DELTA;
			ce.enableBackGround=DEFAULT_VIEW_BACKGROUND;
			ce.enableFPS=DEFAULT_VIEW_FPS;
			ce.enableInertia=DEFAULT_INERTIA;
			ce.monitor.enable=DEFAULT_MONITOR;
			ce.enableViewInertia=DEFAULT_VIEW_INERTIA;

			ce.screenMain.enableFullScreen=DEFAULT_FULLSCREEN;
			ce.screenMain.enableTitle=DEFAULT_TITLED;
			ce.screenMain.region.enable=DEFAULT_VIEW_REGION;

			ce.polygonMode.front=DEFAULT_POLYGON_FRONT;
			ce.polygonMode.back=DEFAULT_POLYGON_BACK;

			ce.dxWin=DEFAULT_XSIZE;
			ce.dyWin=DEFAULT_YSIZE;

			ce.refreshTime=DEFAULT_TIMER_REFRESH;
			ce.cpuLimit=DEFAULT_CPU_LIMIT;
			PostMessage(hDlg,WM_REFRESH,NULL,NULL);
			PostMessage(hDlg,WM_REFRESH_SPIN,NULL,NULL);
			return TRUE;
		}
	}
	return FALSE;
}