/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "3d.h"
#include "input.h"
#include "controlEngine.h"
#include "myList.h"
#include "texture.h"
#include "button.h"
#include "textBox.h"
#include "container.h"
#include "window.h"
#include "taskWindow.h"


extern ControlEngine ce;

TaskWindow::TaskWindow()
{
	name="TaskWindow";
	
	title.SetText("Task");
	title.useClose = FALSE;
	title.useMax = FALSE;

	txtTranche="Task";
}

TaskWindow::~TaskWindow()
{
}

void TaskWindow::Create()
{
	Window::Create(20);
	Size(200,200);
	XDockLeft(0,200);
	YDockBottom(0,150);
	Move(0,0,-25);
	Rotate(0,45,0);

	FaceNode *nodeCur;
	ce.Push(this);
	nodeCur = ce.primitive.GoTo(FACE_RIGHT);
		// text sur la tranche
		txtTranche.Create();
		txtTranche.text.scale=0.5;
		txtTranche.text.material.SetDS(0,0,1);
		txtTranche.faceNode->elem->angle.z=90;
		txtTranche.faceNode->elem->size.x.Set(0,1,&nodeCur->elem->computeSize.y);
		txtTranche.faceNode->elem->size.y.Set(0,1,&nodeCur->elem->computeSize.x);
	ce.Pop(); 
}

BOOL TaskWindow::Add(Title *title)
{
	ce.Push(&client);
	FaceNode *node = ce.primitive.GoTo(FACE_FRONT);
	ElemTaskWindow *elem = tasks.i.AddNewLast();
	elem->title = title;
	elem->button.Create();
	elem->button.SetText(title->caption->GetText());
	elem->button.text.SetWrapping(FALSE,FALSE,TRUE);
	elem->button.onButton.click.connect(title,Title::OnOriginal);
	ce.Pop();
	Refresh();
	return TRUE;
}

/*!
\brief Mise a jour de l'affichage des elements

\todo gerer les depassement de conteneur
*/

void TaskWindow::Refresh()
{
	ElemTaskWindow *elem;
	double xSize=70,ySize=30;
	double xPos=2,yPos=2;
	
	Face * face = client.faceNode->Elems(FACE_FRONT);
	double deltaX = face->computeSize.x;//200;
	double deltaY = face->computeSize.y;//200;
	for (tasks.i=0;tasks.i.More();tasks.i++)
	{
		elem = tasks.i.GetElemPtr();
		if ((xPos+xSize)>deltaX)
		{
			xPos=2;
			yPos+=ySize+2;
			//if (yPos>deltaY) return; // en dehors du conteneur
		}
		elem->button.XDockLeft(xPos,xSize);
		elem->button.YDockTop(yPos,ySize);
		xPos+=xSize+2;
	}
}

int TaskWindow::Find(Title *title)
{
	ElemTaskWindow *elem;
	for (tasks.i=0;tasks.i.More();tasks.i++)
	{
		elem = tasks.i.GetElemPtr();
		if (elem->title==title) return tasks.i.GetIndex();
	}
	return -1;
}

/*! Appel par un title pour retirer la button de la TaskWindow
*/
BOOL TaskWindow::Restore(Title *title)
{
	int num = Find(title);
	if (num==-1) return FALSE;

	ce.timerRefresh.onTimer.connect(this,OnTimerSupr);
	return TRUE;
}

int TaskWindow::OnTimerSupr(Timer *)
{
	Debug0Param("TaskWindow::OnTimerSupr()");
	ce.timerRefresh.onTimer.disconnect(this);
	ce.pickingCursor.ResetCurrent();
	tasks.i.Supr();
	Refresh();
	return 0;
}