/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "3d.h"
#include "MyString.h"
#include "control.h"
#include "input.h"
#include "controlEngine.h"
#include "mylist.h"
#include "text3d.h"
#include "textBox.h"

extern ControlEngine ce;

// ********************************************************************************

TextBox::TextBox()
{
	// intitialisation des variables
	name="TextBox";
	isReadOnly=FALSE;

	// init du text<
	text.Setup(ce.fontTexture,ft_texture); 
	text.SetAlign(align_center,align_center);
	text.material.SetDS(0,0,0);

	// gestion des evenements
	onControl.mouse.left.up.connect(this,OnLButtonUp);
	onControl.mouse.left.down.connect(this,OnLButtonDown);
	onControl.over.in.connect(this,OnOverIn);
}

TextBox::~TextBox()
{
}

// ********************************************************************************

void TextBox::Create(double depth)
{
	SetParent();
	faceNode = ce.primitive.AddBox(name,id,depth,FALSE); 
	faceNode->Elems(FACE_FRONT)->text=&text;
	if (isReadOnly) faceNode->Elems(FACE_BACK)->material.SetDS(.75,.75,.75);
	else faceNode->Elems(FACE_BACK)->material.SetDS(.5,.5,.5);
}

/*!
\brief Affectation du text

Permet l'ecriture
\code
TextBox txtInfo;
info="Ceci est mon text !";
\endcode
*/
void TextBox::operator=(char *newText)
{
	text.SetText(newText);
}

// ********************************************************************************

void TextBox::SetText(char *newText)
{
	text.SetText(newText);
}

/*! 
\brief on a clicker sur le control ==> curseur a l'emplacement de la souris

\todo SetCursor, parametre reel a envoyer
*/
int TextBox::OnLButtonDown(StateMouse *mouse,int face)
{
	DebugMouse2("TextBox::OnLButtonDown",mouse,face);
	ce.cursor.ApplyTypes(cursorText,cursorArrow);
	if (!isReadOnly) SetCursor(mouse->pos.x,mouse->pos.y);
	return 0;
}

int TextBox::OnLButtonUp(StateMouse *mouse,int face)
{
	DebugMouse2("TextBox::OnLButtonUp",mouse,face);
	ce.cursor.ApplyTypes(cursorArrow,cursorText);
	return 0;
}

int TextBox::OnOverIn(Control *oldControl,int face)
{
	Debug2Param("TextBox::OnOverIn(oldCtlr:%s,face:%d",(char *)oldControl->name,face);
	ce.cursor.ApplyTypes(cursorArrow,cursorText,cursorNothing,cursorNothing);
	return 1;
}

//! le curseur a ete positionne
void TextBox::SetCursor(int column,int line)
{
	Debug2Param("TextBox::OnSetCursor(col:%d,line:%d)",column,line);
	onTextBox.setCursor(column,line);
}

void TextBox::ToString(MyString &str)
{
	str.AddFormat("name: %s isReadOnly:%d",name,isReadOnly);
}

