/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file texture.h
\brief Gestion des textures 3D
*/

#ifndef INCLUDE_3D_TEXTURE_H
#define INCLUDE_3D_TEXTURE_H

#include "3d.h"
#include "color.h"

typedef enum
{
	texture1D = GL_TEXTURE_1D , 
	texture2D = GL_TEXTURE_2D,
	texture3D = GL_TEXTURE_3D
} TextureTarget; // a caster en GLint pour switch

typedef enum
{
	clamp = GL_CLAMP,
	repeat = GL_REPEAT
} TextureWrap; // a caster en GLint pour switch

typedef enum
{
	nearest = GL_NEAREST,
	linear = GL_LINEAR
} TextureFilter; // a caster en GLint pour switch

typedef enum 
{
	modulate = GL_MODULATE, 
	decal = GL_DECAL,
	blend = GL_BLEND, 
	replace = GL_REPLACE,
} TextureEnvMode; // a caster en GLint pour switch

/*!
\brief Gestion des textures pour la 3D

Les differentes faon de les generer sont:
	- Par un fichier graphique (CreateFromBitmap)
	- Par du text (CreateFromText)
*/
class Texture
{
private:
	GLubyte *dataOut;	// utilis dans CreateCurstomBegin / End

public:
	static MyList<Texture *> list;
	GLint id;
	MyString name;
	MyString source;
	BOOL useResizing;
	GLdouble xScale,yScale;
	int xSize,ySize;
	TextureTarget target;				
	TextureWrap wrapT,wrapS;			
	TextureFilter magFilter,minFilter;
	TextureEnvMode envMode;				 
	Color envColor;
	Color border;

public: // membre static
	static Texture * Find(char *name);
	static BOOL Delete(Texture *texture);

public:
	void VerifyId();
	void Config(); 
	void ConfigPixelStore();
	GLubyte * ConfigSize (int _xSize,int _ySize,BOOL allocate,BOOL resizeIt);
	void SetCoord(double x,double y);
	double GetXCoord(double x);
	double GetYCoord(double y);
	BOOL Delete();

public:
	Texture();
	~Texture();
	void Zero();
	void operator =(Texture &src);
	operator GLuint () const { return id; }

	BOOL FromNull();
	BOOL FromFile(char *_name,char *fileName,BOOL resizeIt=FALSE);
	BOOL FromBitmap(char *_name,char *fileName,BOOL resizeIt=FALSE);
	BOOL FromText(char *_name,char *text,COLORREF color,COLORREF background,int xFormat=0,int yFormat=0);

	BOOL Begin(char *_name,int deltaX,int deltaY);
	BOOL SetPixel(int x,int y,byte r,byte g,byte b);
	BOOL GetPixel(int x,int y,byte &r,byte &g,byte &b);
	BOOL AddText(char *text,COLORREF color,COLORREF background,int xFormat=0,int yFormat=0);
	BOOL End();
};

#endif