/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "3d.h"
#include "input.h"
#include "controlEngine.h"
#include "title.h"
#include "mylist.h"

extern ControlEngine ce;

// ********************************************************************************

Title::Title()	
{
	// init variable
	name="Title";
	focused=FALSE;
	state=original;
	container=NULL;

	SetUses(TRUE,TRUE,TRUE,TRUE);

	btnMax.name="btnMax";
	btnMin.name="btnMin";
	btnClose.name="btnClose";
	btnIcon.name="btnIcon";

	// config du caption text
	caption = &tipCaption.text;
	caption->Setup(ce.fontTexture,ft_texture); 
	caption->SetAlign(align_center,align_center);
	caption->material.SetDS(1,1,1);
	caption->SetWrapping(FALSE,FALSE,FALSE);

	// gestion des evenements
	btnMin.onButton.click.connect(this,OnMin);
	btnMax.onButton.click.connect(this,OnMaxOrOriginal);
	btnClose.onButton.click.connect(this,OnClose);
	onControl.mouse.move.connect(this,OnMouseMove);
	onControl.mouse.left.down.connect(this,OnLButtonDown);
	onControl.mouse.mid.down.connect(this,OnMButtonDown);
	onControl.mouse.left.up.connect(this,OnLMButtonUp);
	onControl.mouse.mid.up.connect(this,OnLMButtonUp);

	onControl.key.up.connect(this,OnKeyUp);
	onControl.key.down.connect(this,OnKeyDown);
	onControl.mouse.left.doubleClick.connect(this,OnMaxOrOriginal);
	onControl.mouse.wheel.connect(this,OnMouseWheel);

	onControl.over.in.connect(this,OnOverIn);
	onControl.over.out.connect(this,OnOverOut);
}

// ********************************************************************************

void Title::Create(Control *_container,double depth)
{
	SetParent();
	container= _container;
	faceNode = ce.primitive.AddBox(name,id,depth,FALSE); 

	int xLeft=2;
	ce.Push(this);
	ce.primitive.GoTo(FACE_FRONT);
	if (useIcon)
	{
		btnIcon.Create();
		btnIcon.faceNode->Elems(FACE_FRONT)->texture=&ce.texture.iconBase;
		btnIcon.XDockLeft(2,16);
		btnIcon.YDockCenter(2,2);
		xLeft+=16;
	}
	int xRight=2;
	if (useClose)
	{
		btnClose.Create();
		btnClose.faceNode->Elems(FACE_FRONT)->texture=&ce.texture.close;
		btnClose.XDockRight(xRight,16);
		btnClose.YDockCenter(2,2);
		xRight+=18;
	}
	if (useMax)
	{
		btnMax.Create();
		btnMax.faceNode->Elems(FACE_FRONT)->texture=&ce.texture.max;
		btnMax.XDockRight(xRight,16);
		btnMax.YDockCenter(2,2);
		xRight+=18;
	}
	if (useMin)
	{
		btnMin.Create();
		btnMin.faceNode->Elems(FACE_FRONT)->texture=&ce.texture.min;
		btnMin.XDockRight(xRight,16);
		btnMin.YDockCenter(2,2);
		xRight+=18;
	}
	tipCaption.Create();
	tipCaption.XDockCenter(xLeft+2,xRight+2);
	tipCaption.YDockCenter(0,0);
	tipCaption.SetAlphaBlending(0);
	tipCaption.faceNode->elem->noPicking=TRUE;
	ce.Pop();
	GotFocus();
}

Title::~Title()
{
	ce.timerRefresh.onTimer.disconnect(this);
}

//! A chaque rafraichisement graphique, verifie s'il y a des deplacements rapides
int Title::OnRefresh(Timer *timer)
{
	//Debug0Param("Title::OnRefresh()");
	if (!StateKey::GetState(VK_SHIFT))
	{
		double delta = timer->GetDeltaTime(TRUE)*500.0; // but 20 texel / seconde
		double x=0,y=0,z=0;
		if (StateKey::GetState(VK_UP))		y+=delta;
		if (StateKey::GetState(VK_DOWN))	y-=delta;
		if (StateKey::GetState(VK_LEFT))	x-=delta;
		if (StateKey::GetState(VK_RIGHT))	x+=delta;
		if (StateKey::GetState(VK_PRIOR))	z-=delta;
		if (StateKey::GetState(VK_NEXT))	z+=delta;
		if (container&&(x||y||z)) container->Move(x,y,z); 
	}
	return 0;
}

int Title::OnOverIn(Control *oldControl,int face)
{
	Debug2Param("Title::OnOverIn(oldCtlr:%s,face:%d",(char *)oldControl->name,face);
	ce.cursor.ApplyTypes(cursorArrow,cursorMoving,cursorRepere);
	ce.timerRefresh.onTimer.connect(this,OnRefresh);
	return 1;
}

int Title::OnOverOut(Control *newControl,int face)
{
	Debug2Param("Title::OnOverOut(newCtlr:%s,face:%d",(char *)newControl->name,face);
	ce.timerRefresh.onTimer.disconnect(this);
	return 1;
}

void Title::SetUses(BOOL _useMin,BOOL _useMax,BOOL _useClose,BOOL _useIcon)
{
	useMin=_useMin;
	useMax=_useMax;
	useClose=_useClose;
	useIcon=_useIcon;
}

void Title::SetText(char *newCaption)
{
	caption->SetText(newCaption);
}

/*!
\brief Swap soit en Maximize soit en OriginalSize
*/
int Title::OnMaxOrOriginal(StateMouse *mouse,int face)
{
	DebugMouse2("Title::OnMaxOrOriginal",mouse,face);
	switch (state)
	{
	case original: OnMax(); break;
	case maximize: OnOriginal(); break;
	}
	return 1;
}

int Title::OnMaxOrOriginal()
{
	Debug0Param("Title::OnMaxOrOriginal");
	switch (state)
	{
	case original: OnMax(); break;
	case maximize: OnOriginal(); break;
	}
	return 1;
}

/*!
\brief Affichage en taille par defaut

Action:
	- Change le boutton de Max en mode "original"
	- re-place le container a sa postion et d'origine
	- retir de taskWindow le boutton de restor
*/
int Title::OnOriginal()
{
	Debug0Param("Title::OnOriginal()");
	if (useMax) btnMax.faceNode->Elems(FACE_FRONT)->texture=&ce.texture.max;
	state=original;
	if (container)
	{
		container->faceNode->elem->angle = originalAngle;
		container->faceNode->elem->size= originalSize;
		container->faceNode->MoveTo(tempNode);
	}
	ce.taskWindow->Restore(this);
	return 1;
}

/*!
\brief La wheel configure le demplacement z
*/
int Title::OnMouseWheel(StateMouse *mouse,int face)
{
	DebugMouse2("Title::OnMouseWheel",mouse,face);
	if (!container) return 0;
	container->Move(0,0,-mouse->wheelDelta/10.0);
	return 1;
}

/*!
\brief Bouton min clicker

Sauvegarde la taille et position original de la fenetre et place le titre le plus bas a gauche du conteneur

\todo Seul l'univers pour le moment, mais dois etre etendu style fenetre MDI)
*/
int Title::OnMin()
{
	Debug0Param("Title::OnMin()");
	state=minimize;
	if (container)
	{
		originalAngle = container->faceNode->elem->angle;
		originalSize = container->faceNode->elem->size;
		tempNode=container->faceNode->parent;
		container->faceNode->MoveTo(ce.trash.GetRoot());
		ce.taskWindow->Add(this);
	}
	onTitle.minimize();
	return 1;
}

/*!
\brief Boutton max clicker

place le title en fullScreen mode
*/
int Title::OnMax()
{
	Debug0Param("Title::OnMax()");
	state=maximize;
	if (useMax) btnMax.faceNode->Elems(FACE_FRONT)->texture=&ce.texture.maxAlternate;
	if (container)
	{
		originalAngle = container->faceNode->elem->angle;
		originalSize = container->faceNode->elem->size;
		tempNode=container->faceNode->parent;
		container->RotateTo(0,0,0);
		container->faceNode->elem->angle.Zero();
		container->faceNode->MoveTo(ce.primitive.tree.GetRoot());
	}
	onTitle.maximize();
	return 1;
}

/*!
\brief si appuis de ESC sur le title ==> Close, HOME ==> angles=0
*/
int Title::OnKeyUp(StateKey *key,int face)
{
	DebugKey2("Title::OnKeyUp",key,face);
	switch(key->code)
	{
	case VK_ESCAPE: OnClose(); return 1;
	case VK_HOME: 
		if (!container) return 0;
		container->RotateTo(0,0,0);
		return 1;
	}
	return 0;
}

/*!
\brief si appuis de les touche de direction ==> move , si shift en meme temp ==> rotate
*/
int Title::OnKeyDown(StateKey *key,int face)
{
	DebugKey2("Title::OnKeyDown",key,face);
	if (!container) return 0;
	BOOL shift = StateKey::GetState(VK_SHIFT);
	if (shift)
	{
		switch(key->code)
		{
		case VK_UP:		container->RotateQuaternion(0,90/4.0,FALSE);	return 1;
		case VK_DOWN:	container->RotateQuaternion(0,-90/4.0,FALSE);	return 1;
		case VK_LEFT:	container->RotateQuaternion(90/4.0,0,FALSE);	return 1;
		case VK_RIGHT:	container->RotateQuaternion(-90/4.0,0,FALSE);	return 1;
		}
	}
	/* Remplacer par OnRefresh
	else
	{
		switch(key->code)
		{
		case VK_UP: Move(0,-15,0,); break;
		case VK_DOWN: Move(0,+15,0); break;
		case VK_LEFT: Move(-15,0,0); break;
		case VK_RIGHT: Move(15,0,0); break;
		case VK_PRIOR: Move(0,0,-15); break;
		case VK_NEXT: Move(0,0,15); break;
		}
	}
	*/
	return 0;
}

/*!
\brief La souris bouge
*/
int Title::OnMouseMove(StateMouse *mouse,int deltaX,int deltaY,int face)
{
	//DebugMouse2("Title::OnMouseMove",mouse,face);
	if (!container) return 0;
	//if (parent) parent->onControl.mouse.move(mouse,deltaX,deltaY,parentFace);	// heritage au parent pour la gestion des trackButtons	
	if (face!=FACE_FRONT) return 0; // pour le moment car la translation est local pour l'instnant et pas par rapport au point de vu
	if (mouse->GetNbButtonPush()==1)
	{
		if (mouse->leftBtn) container->Move(-deltaX,deltaY,0);
		if (mouse->midBtn)	container->RotateQuaternion(deltaX,deltaY,0);
		if (mouse->leftBtn||mouse->midBtn) return 1;
	}
	return 0;
}

int Title::OnLButtonDown(StateMouse *mouse,int face)
{
	DebugMouse2("Title::OnLButtonDown",mouse,face);
	oldMouse=*mouse;
	GotFocus();
	ce.cursor.ApplyTypes(cursorMoving,cursorArrow,cursorRepere);
	return 1;
}

int Title::OnMButtonDown(StateMouse *mouse,int face)
{
	DebugMouse2("Title::OnMButtonDown",mouse,face);
	oldMouse=*mouse;
	GotFocus();
	ce.cursor.ApplyTypes(cursorRepere,cursorMoving,cursorArrow);
	return 1;
}

int Title::OnLMButtonUp(StateMouse *mouse,int face)
{
	DebugMouse2("Title::OnLMButtonUp",mouse,face);
	ce.cursor.ApplyTypes(cursorArrow,cursorMoving,cursorRepere);
	return 0;
}

/*!
\brief Fenetre parente a le focus

\todo Passer nouveau parametre a onTitle.GotFocus(oldSelected), gestion des selected control precedent et suivant
*/
void Title::GotFocus()
{
	//focused^=1;
	//if (! focused){ LostFocus(); return; } // pour test
	Debug0Param("Title::GotFocus()");
	focused=TRUE;
	faceNode->Elems(FACE_FRONT)->texture=&ce.texture.degradeeFocus;
	faceNode->Elems(FACE_FRONT)->material.SetDS(1,1,1);
	faceNode->Elems(FACE_LEFT)->material.SetDS(10/256.0,36/256.0,106/256.0);
	faceNode->Elems(FACE_RIGHT)->material.SetDS(166/256.0,202/256.0,240/256.0);
	faceNode->Elems(FACE_TOP)->material.SetDS(0,0,0.4f);
	faceNode->Elems(FACE_BOTTOM)->material.SetDS(0,0,0.4f);

	onTitle.gotFocus();
}

/*!
\brief Fenetre parente n'a plus le focus
*/
void Title::LostFocus()
{
	Debug0Param("Title::LostFocus()");
	focused=FALSE;
	faceNode->Elems(FACE_FRONT)->texture=&ce.texture.degradeeUnFocus;
	faceNode->Elems(FACE_FRONT)->material.SetDS(1,1,1,0);
	faceNode->Elems(FACE_LEFT)->material.SetDS(128/256.0,128/256.0,128/256.0,0);
	faceNode->Elems(FACE_RIGHT)->material.SetDS(192/256.0,192/256.0,192/256.0,0);
	faceNode->Elems(FACE_TOP)->material.SetDS(.5,.5,.5);
	faceNode->Elems(FACE_BOTTOM)->material.SetDS(.5,.5,.5);

	onTitle.lostFocus();
}

/*!
\brief Bouton close clicker
*/
int Title::OnClose()
{
	Debug0Param("Title::OnClose()");
	//Delete(); // si pas de conteneur
	onTitle.close();
	return 1;
}

// ********************************************************************************

void Title::ToString(MyString &str)
{
	str.AddFormat("name: %s caption:%s focused:%d",
		name,faceNode->Elems(FACE_FRONT)->text->GetText(),focused);
}

// ********************************************************************************
