/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file title.h
\brief Control Title
*/

#ifndef INCLUDE_TITLE_H
#define INCLUDE_TITLE_H

#include "MyString.h"
#include "button.h"
#include "textBox.h"
//#include "container.h"

typedef enum { original , maximize , minimize } StateTitle;

using namespace sigslot;  // pour signal (evit l'ecriture sigslot::signal?<...>)

class EventTitle
{
public:
	signal0<> minimize;		//Call: void Minimize();
	signal0<> maximize;		//Call: void Maximize();
	signal0<> original;		//Call: void Original();
	signal0<> close;		//Call: void Close();
	signal0<> gotFocus;		//Call: void GotFocus();
	signal0<> lostFocus;	//Call: void Class::LostFocus();
};

/*!
\brief titre affiche une bare degrad titre avec des boutons (min,max,close)

Il n'a theoriquement pas d'interet seul, un conteneur peut s'en servire (Window,MsgBox,etc ...)
*/
class Title : public Control
{
public:
	// varaible membre
	Button btnMin,btnMax,btnClose;
	Button btnIcon;
	Text3D *caption;
	Tip	tipCaption;
	Control *container;					//!< control pere contenant 

	BOOL focused;
	BOOL useIcon;	//!< le bouton icone est il a utiliser
	BOOL useMin;	//!< le bouton min est il a utiliser
	BOOL useMax;	//!< le bouton max est il a utiliser
	BOOL useClose; //!< le bouton close est il a utiliser

	// gestion pour min / max / alternate
	Point3D<LayoutDouble> originalSize; // taille fenetre avant maximisation ou minimisation
	Point3D<GLdouble> originalAngle;	// angle fenetre avant maximisation ou minimisation
	FaceNode *tempNode;			// noeud temporaire, en max = masterContainer, en min dans trash
	StateTitle state;
	StateMouse oldMouse;
	EventTitle onTitle;

	Title();
	~Title();

	virtual void Create() { Create(NULL,DEFAULT_DEPTH_TITLE); } // Createur pour container
	void Create(Control *container,double depth=DEFAULT_DEPTH_TITLE); // !importent, le container dois etre le control contenant le title
	void ToString(MyString &str);
	void GotFocus();
	void LostFocus();
	void operator=(char *newCaption) { SetText(newCaption); };
	void SetText(char *newCaption);
	void SetUses(BOOL _useMin,BOOL _useMax,BOOL _useClose,BOOL _useIcon);

	int OnClick(StateMouse *mouse,int face);
	int OnLButtonDown(StateMouse *mouse,int face);
	int OnMButtonDown(StateMouse *mouse,int face);
	int OnLMButtonUp(StateMouse *mouse,int face);
	int OnMaxOrOriginal(StateMouse *mouse,int face);
	int OnMaxOrOriginal();
	int OnMouseMove (StateMouse *mouse,int deltaX,int deltaY,int face);
	int OnMouseWheel(StateMouse *mouse,int face);
	int OnMin();
	int OnMax();
	int OnOriginal();
	int OnClose();
	int OnKeyDown(StateKey *key,int face);
	int OnKeyUp(StateKey *key,int face);
	int OnRefresh(Timer *);
	int OnOverIn(Control *,int);
	int OnOverOut(Control *,int);
};

#endif