/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file trackButton.h
\brief Control Track Button
*/

#ifndef INCLUDE_TRACK_BUTTON_H
#define INCLUDE_TRACK_BUTTON_H

#include "control.h"

class EventTrackButton
{
public:
	signal0<> rotate; // call: void OnRotate();
	signal1<double> resize; // call: void OnResize(double nbTexel);
	signal1<double> translate; // call: void OnTranslate(double nbTexel);
};

/*!
\brief
Class de parametre et d'evenement d'un "Bouton"
*/

class TrackButton : public Control
{
private:
	void ApplyMode();
	void Config(int (TrackButton::*onClickFace)(StateMouse *,int),Texture *texture,double angleZ);
	BOOL canResize;
	BOOL canRotate;
	BOOL isMoving;

public:
	// varaible membre
	DockPos orientation;		//!< Orientation du boutton([haut]|[bas]|[gauche]|[droit])
	// evenements
	EventTrackButton onTrackButton;

	TrackButton();
	~TrackButton();

	virtual void Create() { Create(DEFAULT_SIZE_TRACK_BUTTON); } // Createur pour container
	BOOL Create(double size);
	void ToString(MyString &str);

	void SetMode(DockPos newOrientation);
	void SetResizeAndRotate(BOOL resize,BOOL rotate);


	int OnButtonClick();
	int OnMouseMove(StateMouse *mouse,int deltaX,int deltaY,int face);
	int OnOverOut(Control *newControl,int face);

	int OnClickLeft(StateMouse *,int);
	int OnClickRight(StateMouse *,int);
	int OnClickTop(StateMouse *,int);
	int OnClickBottom(StateMouse *,int);
	int OnClickLeftTop(StateMouse *,int);
	int OnClickLeftBottom(StateMouse *,int);
	int OnClickRightTop(StateMouse *,int);
	int OnClickRightBottom(StateMouse *,int);
};

#endif