/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "3d.h"
#include "input.h"
#include "controlEngine.h"
#include "treeView.h"

extern ControlEngine ce;

TreeView::TreeView()
{
	posY=0;
}

void TreeView::Create()
{
	faceNode = ce.primitive.AddBox("TreeView",id,10,FALSE);
}

void TreeView::Draw ()
{
	ce.Push(this);
	ce.primitive.GoTo(FACE_FRONT);
	if (!faceNode->elem->computeSize.y) ce.primitive.ComputeLayout(faceNode);
	posY=2; 
	DrawRecursif(data.GetRoot(),2,0);
	ce.Pop();
}

int TreeView::DrawRecursif (TreeNode *node,double margeY,int level)
{
	int nbElem=0;
	int nbElemFather;
	TreeNode *nodeChild;

	for (node->childs.i=0;node->childs.i.More();node->childs.i++)
	{
		nodeChild = node->childs.i.GetElem();
		if (posY+21>=faceNode->elem->computeSize.y) return nbElem;

		// creation du boutton
		Button *button;
		button = new Button;
		nodeChild->elem->idControl = button->id;
		button->materialOnUp.SetDS(0.5f+level*0.1f,0.5f+level*0.1f,0.5f+level*0.1f);
		button->materialOnDown.SetDS(0,0,1);
		button->Create();
		button->SetText(nodeChild->elem->text);
		button->text.SetWrapping(TRUE,FALSE,FALSE);
		button->XDockCenter(10,0);
		button->YDockTop(margeY,19);
		button->faceNode->Elems(FACE_RIGHT)->material=*button->materialCurrent;

		if (nodeChild->elem->contentSub)
		{	// a un sous repertoire (ou peut en posseder potentielement)
			button->onButton.click.connect(this,OnClick);
			button->text.SetAlign(align_begin,align_begin);
			AddTreeButton(button,nodeChild->elem->isCollaps); // ajout du boutton '+' / '-'
			if (!nodeChild->elem->isCollaps)
			{
				ce.Push(button);
					ce.primitive.GoTo(FACE_FRONT);
					nbElemFather=DrawRecursif(nodeChild,21,level+1);
				ce.Pop();
			}
			else nbElemFather=0;
			nbElem+=nbElemFather+1;
			margeY+=(nbElemFather+1)*21;
			posY+=21;
			button->faceNode->elem->size.y.constValue+=nbElemFather*21;
		}
		else
		{	// on est sur un noeud final
			button->onButton.click.connect(this,OnClick);
			button->text.SetAlign(align_begin,align_center);
			margeY+=21;
			posY+=21;
			nbElem++;
		}		
	}
	return nbElem;
}

//! Trouve le noeud data qui est utilis dans un control donne

TreeNode * TreeView::FindOnData(int idControl)
{
	return FindOnData(idControl,data.GetRoot());
}

TreeNode * TreeView::FindOnData(int idControl,TreeNode *node)
{
	if (!node) return NULL;
	if (node->elem->idControl==idControl) return node;
	TreeNode *nodeChild,*nodeFind;
	for (node->childs.i=0;node->childs.i.More();node->childs.i++)
	{
		nodeChild = node->childs.i.GetElem();
		nodeFind = FindOnData(idControl,nodeChild);
		if (nodeFind) return nodeFind;
	}
	return NULL;
}

int TreeView::OnClick()
{
	Control * control = ce.pickingCursor.controlHover;
	Test(control);
	return 0;
}

int TreeView::OnClickPlus()
{
	Control * control = ce.pickingCursor.controlHover;
	Test(control->parent);	
	return 0;
}

void TreeView::Test(Control *control)
{
	TreeNode *node = FindOnData(control->id);
	//char *text = control->faceNode->Elems(FACE_FRONT)->text->GetText();

	if (node->elem->contentSub) 
	{
		node->elem->isCollaps^=1;
		ce.timerRefresh.onTimer.connect(this,OnNextFrame); // ca sinon pb d'inter-blocage
	}
	onTreeView.select(node);
}

void TreeView::GetTreePath(TreeNode *treeNode,MyString &result,BOOL isFirst)
{
	if (!treeNode) 
	{
		result="";
		return;
	}
	GetTreePath(treeNode->parent,result,FALSE);
	result<<treeNode->elem->text;
	if (!isFirst) result<<'/';
}

int TreeView::OnNextFrame(Timer *timer)
{
	timer->onTimer.disconnect(this);
	ce.pickingCursor.ResetCurrent();
	DeleteSub();
	Draw();
	return 0;
}

void TreeView::ConfigFacesPlus(Control *control,BOOL isCollaps)
{
	MyListIterator<FaceNode *> i(&control->faceNode->childs);
	Face *face;
	for (i=0;i.More();i++)
	{
		face = i.GetElem()->elem;
		face->texture=isCollaps?&ce.texture.plus:&ce.texture.minus;
	}
}

void TreeView::AddTreeButton(Button *button,BOOL isCollaps)
{
	ce.Push(button);
	ce.primitive.GoTo(FACE_LEFT);
	Button *subButton;
	subButton = new Button;
	subButton->materialOnUp = button->materialOnUp;
	subButton->Create(12);
	subButton->XDockCenter(1);
	subButton->YDockTop(1,17);
	ConfigFacesPlus(subButton,isCollaps);
	ce.Pop();

	subButton->onButton.click.connect(this,OnClickPlus);
}

// *****************************************************************************

TreeDirectory::TreeDirectory()
{
	addFile=FALSE;
	addDirectory=TRUE;
	filter="*.*";
	beginPath="c:";
	onTreeView.select.connect(this,OnClickDirectory);
}

int TreeDirectory::OnClickDirectory(TreeNode *treeNode)
{
	if (treeNode->elem->contentSub)
	{
		ce.timerRefresh.onTimer.disconnect(this); // ca sinon pb d'inter-blocage
		MyString pathFile;
		GetTreePath(treeNode,pathFile);
		treeNode->SuprChilds(TRUE);
		Loading(treeNode,pathFile,TRUE);
		ce.timerRefresh.onTimer.connect((TreeView*)this,OnNextFrame); // ca sinon pb d'inter-blocage
	}
	return 0;
}

void TreeDirectory::Config(BOOL _addFile,BOOL _addDirectory,char *_filter)
{
	addFile = _addFile;
	addDirectory = _addDirectory;
	if (_filter) filter = _filter;
}

void TreeDirectory::Reset()
{
	data.SuprAll();
	DeleteSub();
}

void TreeDirectory::Loading(char *name,BOOL isCollaps)
{
	beginPath=name;
	// il faut au moin une racine
	if (!data.GetRoot()) 
	{
		TreeViewItem item;
		item.text=name;
		data.Add(item);
	}
	Loading(data.GetRoot(),name,isCollaps);
}

void TreeDirectory::Loading(TreeNode *node,char *name,BOOL isCollaps)
{
	HANDLE hFichCour;
	WIN32_FIND_DATA wfd ;
	MyString path;
	MyString temp;
	TreeViewItem item;
	TreeNode *newNode;

	path<<clear<<name<<"\\"<<filter;
	hFichCour = FindFirstFile (path, &wfd) ;
	if (hFichCour == INVALID_HANDLE_VALUE) return ;
	do 
	{
		if (wfd.cFileName[0]=='.') continue;
		BOOL isDir = (wfd.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY;
		if (isDir)
		{		
			if (!addDirectory) continue;
			temp<<clear<<name<<"\\"<<wfd.cFileName;
			item.isCollaps=isCollaps;
			item.contentSub=TRUE;
			item.text=wfd.cFileName;
			newNode = data.Add(item,node);
			item.askSubData=isCollaps;
			if (!isCollaps) Loading(newNode,temp,isCollaps);
		}
		else
		{
			if (!addFile) continue;
			item.isCollaps=FALSE;
			item.contentSub=FALSE;
			item.text=wfd.cFileName;
			data.Add(item,node);
		}
	} 
	while (FindNextFile (hFichCour, &wfd ));
	FindClose (hFichCour) ;
}