/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "trigo.h"

// borne a un angle de [0,2*PI[
double Bound2_PI(double angle)  
{
	int nbTours = (int)(angle/DOUBLE_PI);
	double angleRes = angle-nbTours*DOUBLE_PI;
	if (angleRes<0) angleRes = DOUBLE_PI+angleRes;
	return angleRes;
}

/*!
\brief Dit si les points sont les sens trigonometrique (invers des aiguille d'une montre)

Technique: prendre 3 points et determiner l'angle entre les 2 segments 
remarque: ne regard que 3 points, il peut y avoir inconsistance

\return resultat:
	- TRUE: trigonometrique
	- FALSE: horaire
*/
BOOL GetTrigoSens(POINT *pt)
{
	double angle1,angle2;
	int dx1=pt[1].x-pt[0].x;
	int dy1=pt[1].y-pt[0].y;
	int dx2=pt[2].x-pt[1].x;
	int dy2=pt[2].y-pt[1].y;
	angle1=atan2((double)dy1,(double)dx1);
	angle2=atan2((double)dy2,(double)dx2);
	return Bound2_PI(angle2-angle1)>PI?FALSE:TRUE;
}

/*!
\brief Dit si les points sont les sens trigonometrique (invers des aiguille d'une montre) 

Technique: prendre tout les points
\return 
	- TRUE:		trigonometrique
	- FALSE:	horaire
	- -1:		inconsistant
	- -2:		erreur: il faut au moin 3 points
*/
BOOL GetTrigoSens(POINT *pt,int nbPoints)
{
	BOOL trigo=-2;
	for (int i=0;i<nbPoints-2;i++)
	{
		BOOL trigoCur = GetTrigoSens(pt+i);
		if (trigo==-2) trigo = trigoCur;
		if (trigoCur!=trigo) 
			return -1;
	}
	return trigo;
}

/*!
\brief Inverse l'ordre des points ==> change de sens rotation
*/
void ChangeTrigoSens(POINT *pt,int nbPoints)
{
	POINT point;
	// si nb de point impaires, ne change rien
	for (int i=0,j=nbPoints-1;i<nbPoints/2;i++,j--)
	{
		point.x=pt[i].x;
		point.y=pt[i].y;
		pt[i].x=pt[j].x;
		pt[i].y=pt[j].y;
		pt[j].x=point.x;
		pt[j].y=point.y;
	}
}

/*!
\brief Donne le chiffre exposent de 2 englobant le nombre posse
*/
int GetNearExp2(int num) 
{
	int numExp2;
	for (numExp2=1;numExp2<num;numExp2<<=1);
	return numExp2;
}

/*! 
\brief (int)floor(log2(x)) 
*/
int IntFloorLog2(unsigned int x)
{
	int a=0;
	while (x>>=1) ++a;
	return a;
}

/*!
\brief true if x is a power of two 
*/
BOOL IsPow2(unsigned int x)
{
	return ((x>0) && ((x&(x-1))==0));
}