/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "3d.h"
#include "input.h"
#include "controlEngine.h"
#include "univers.h"
#include "mylist.h"
#include "polygonMode.h"
#include "valueEditor.h"
#include "msgBox.h"
#include "tip.h"

extern ControlEngine ce;

// ********************************************************************************

Univers::Univers()
{
	// init des variables
	isOnMenu=FALSE;
	isMenuUsing=FALSE;
	name="Univers";
	tipKey=NULL;

	// gestion des evenements
	onControl.key.down.connect(this,OnKeyDown);
	onControl.key.up.connect(this,OnKeyUp);
	onControl.mouse.move.connect(this,OnMouseMove);
	onControl.mouse.wheel.connect(this,OnMouseWheel);
	onControl.mouse.left.down.connect(this,OnLButtonDown);
	onControl.mouse.left.up.connect(this,OnLButtonUp);
	onControl.mouse.mid.down.connect(this,OnMButtonDown);
	onControl.mouse.mid.up.connect(this,OnMButtonUp);
	onControl.over.in.connect(this,OnOverIn);
	onControl.over.out.connect(this,OnOverOut);
}

Univers::~Univers()
{
	ce.timerRefresh.onTimer.disconnect(this);
}

// ********************************************************************************

void Univers::Create()
{	// creation graphics
	SetParent();
	faceNode = ce.primitive.AddFace(name,id,-1,FALSE,0,0,0,0,0,0,1,0,1);
}

// ********************************************************************************

int Univers::OnKeyDown (StateKey *key,int face)
{
	DebugKey2("Univers::OnKeyDown",key,face);
	if (isOnMenu)
	{
		isMenuUsing=TRUE;
		switch (key->code)
		{
		case 'C': ce.primitive.ViewDebug(); UpdateTip(); return 1;
		case 'D': ShowWindow(CreateDialogParam(ce.hInst,MAKEINTRESOURCE(IDD_DEBUG_STRUCT),NULL,(DLGPROC)DebugStructProc,NULL),SW_SHOW); return 1;
		case 'R': ce.screenMain.region.enable^=1; UpdateTip(); return 1;
		case 'F': ce.screenMain.enableFullScreen^=1 ; ce.screenMain.Update(TRUE); UpdateTip(); return 1;
		case 'T': ce.screenMain.enableTitle^=1; ce.screenMain.Update(TRUE); UpdateTip(); return 1;
		//case 'F': ce.screenMain.enableFullScreen=!ce.screenMain.enableFullScreen ; ce.Refresh(FALSE); ce.screenMain.ApplySize(); UpdateTip(); return 1;
		//case 'T': ce.screenMain.enableTitle=!ce.screenMain.enableTitle; ce.Refresh(TRUE); ce.screenMain.ApplySize(); UpdateTip(); return 1;
		case 'L': ce.enableLightView^=1; UpdateTip(); return 1;
		case 'B': ce.enableBackGround^=1; UpdateTip(); return 1;
		case 'A': ce.enableAntiAliasing^=1; UpdateTip(); return 1;
		case 'P': ce.enablePicking^=1; UpdateTip(); return 1;
		case 'E': ce.enableRepere^=1; UpdateTip(); return 1; // lettre 'R' deja pris !
		case 'S': ce.enableFPS^=1; UpdateTip(); return 1;
		case 'M': 
			{
				// ordre des differents modes
				PolygonModeType *mode,modes[]= {
					fill,fill, 
					line,line, 
					point,point, 
					fill,line, 
					fill,point, 
					line,fill, 
					line,point, 
					point,fill, 
					point,line};
				mode = modes;
				int i;
				for (i=0;i<9;i++)
				{
					if (mode[0]==ce.polygonMode.front && mode[1]==ce.polygonMode.back) break;
					mode+=2;
				}
				if (i==9) mode= modes;
				else mode+=2;
				ce.polygonMode.Use(mode[0],mode[1]);
				UpdateTip(); 
			}
			return 1;
		default:
			isOnMenu=FALSE;
			isMenuUsing=FALSE;
		}
	}
	BOOL shift = StateKey::GetState(VK_SHIFT);
	if (shift)
	{
		switch(key->code)
		{
		case VK_LEFT:	RotateQuaternion(90/4.0,0,0);	return 1;
		case VK_RIGHT:	RotateQuaternion(-90/4.0,0,0);	return 1;
		case VK_UP:		RotateQuaternion(0,90/4.0,0);	return 1;
		case VK_DOWN:	RotateQuaternion(0,-90/4.0,0);	return 1;
		//case VK_LEFT:	Rotate(0,-90/4.0);	return 1;
		//case VK_RIGHT:	Rotate(0,90/4.0);	return 1;
		//case VK_UP:		Rotate(-90/4.0,0);	return 1;
		//case VK_DOWN:	Rotate(90/4.0,0);	return 1;
		}
	}
	else
	{
		switch(key->code)
		{
		//case VK_UP:		Move(0,15,0);	break;
		//case VK_DOWN:	Move(0,-15,0);	break;
		//case VK_LEFT:	Move(-15,0,0); break;
		//case VK_RIGHT:	Move(15,0,0);	break;
		//case VK_PRIOR:	Move(0,0,-15);	break;
		//case VK_NEXT:	Move(0,0,15);	break;

		case VK_F11: 
			isOnMenu=TRUE; 
			isMenuUsing=FALSE;
			break;
		}
	}
	return 0;
}

// ********************************************************************************

void Univers::ExitQuestion()
{
	MsgBox *msgBox = new MsgBox;
	ce.Push(ce.masterContainer);
		msgBox->Create();
		msgBox->XDockCenter(160);
		msgBox->YDockCenter(100);
		msgBox->Move(0,0,100);
		msgBox->txtInfo.SetText("Do you realy want to exit");
		msgBox->btnCancel.SetText("No");
		msgBox->btnOk.SetText("Yes");
		msgBox->btnOk.onButton.click.connect(this,OnExit);
	ce.Pop();
}

int Univers::OnExit ()
{
	Debug0Param("Univers::OnExit");
	ce.timerRefresh.onTimer.connect(this,OnApplyExit);
	return 1;
}

int Univers::OnApplyExit(Timer *)
{
	Debug0Param("Univers::OnApplyExit");
	ce.timerRefresh.onTimer.disconnect(this);
	ce.Stop(); 
	return 1;
}

int Univers::OnKeyUp (StateKey *key,int face)
{
	DebugKey2("Univers::OnKeyUp",key,face);
	switch (key->code)
	{
	case VK_ESCAPE: ExitQuestion(); return 1;
	case VK_F1: ce.ScreenShot(ssRGB); return 1;
	case VK_F2: ce.ScreenShot(ssDepth); return 1;
	case VK_F3: ce.ScreenShot(ssLuminance); return 1;
	case VK_F4: ce.ScreenShot(ssBlending); return 1;
	case VK_HOME: HomePosition(); return 1;
	case VK_F11: 
		if (isOnMenu)
		{		
			isOnMenu=FALSE;
			if (!isMenuUsing)
			{
				if (!tipKey)
				{
					tipKey = new Tip;
					ce.Push(ce.masterContainer);
					tipKey->Create();
					tipKey->XDockRight(10,220);
					tipKey->YDockCenter(300);
					tipKey->text.SetAlign(align_begin,align_center);
					UpdateTip();
					ce.Pop();
					//ou MessageBox(ce.screenMain.hWnd,temp,"ShortCutKey - Help",0);
				}
				else 
				{
					delete tipKey;
					tipKey=NULL;
				}
				return 1;
			}
		}
		break;
	}
	return 0;
}

void Univers::UpdateTip()
{
	if (!tipKey) return;
	MyString temp;
	MyString sTrue ("True\n");
	MyString sFalse("False\n");
	temp<<"System Key - Help\n\n";
	temp<<"F11+R:\tRegion\t\t"<<(ce.screenMain.region.enable?sTrue:sFalse);
	temp<<"F11+F:\tFull screen\t"<<(ce.screenMain.enableFullScreen?sTrue:sFalse);
	temp<<"F11+T:\tTitle screen\t"<<(ce.screenMain.enableTitle?sTrue:sFalse);
	temp<<"F11+L:\tLightView\t\t"<<(ce.enableLightView?sTrue:sFalse);
	temp<<"F11+B:\tBackground\t"<<(ce.enableBackGround?sTrue:sFalse);
	temp<<"F11+A:\tAntialiasing\t"<<(ce.enableAntiAliasing?sTrue:sFalse);
	temp<<"F11+P:\tPicking\t\t"<<(ce.enablePicking?sTrue:sFalse);
	temp<<"F11+E:\tRepere\t\t"<<(ce.enableRepere?sTrue:sFalse);
	temp<<"F11+S:\tFPS\t\t"<<(ce.enableFPS?sTrue:sFalse);
	temp<<"F11+M:\tMode Polygone\t\n       ";ce.polygonMode.ToString(temp); temp<<"\n";
	temp<<"F11+D:\tDebuger\n";
	temp<<"F11+C:\tArborecence debug";
	tipKey->text.SetText(temp);
}

void Univers::HomePosition()
{
	faceNode->elem->angle.Zero();
	faceNode->elem->position.Zero();
}

int Univers::OnMouseMove(StateMouse *mouse,int deltaX,int deltaY,int face)
{
	//DebugMouse2("Univers::OnMouseMove",mouse,face);

	if (mouse->GetNbButtonPush()==1)
	{
		if (mouse->midBtn) RotateQuaternion(deltaX,deltaY,0);
		if (mouse->leftBtn) Move(-deltaX,deltaY,0);
		if (mouse->midBtn||mouse->leftBtn) return 1;
	}
	return 0;
}

int Univers::OnMouseWheel(StateMouse *mouse,int face)
{
	DebugMouse2("Univers::OnWheel",mouse,face);
	Move(0,0,-mouse->wheelDelta/10.0);
	return 1;
}

int Univers::OnLButtonDown(StateMouse *mouse,int face)
{
	DebugMouse2("Univers::OnLButtonDown",mouse,face);
	oldMouse=*mouse;
	ce.cursor.ApplyTypes(cursorMoving,cursorArrow);
	return 0;
}

int Univers::OnLButtonUp(StateMouse *mouse,int face)
{
	DebugMouse2("Univers::OnLButtonU",mouse,face);
	if (ce.cursor.curType==cursorMoving) ce.cursor.curType=cursorArrow;
	ce.cursor.leftType=cursorMoving;
	ce.cursor.ApplyTypes();
	return 0;
}
int Univers::OnMButtonDown(StateMouse *mouse,int face)
{
	DebugMouse2("Univers::OnMButtonDown",mouse,face);
	oldMouse=*mouse;
	ce.cursor.ApplyTypes(cursorRepere,cursorNoChange,cursorArrow);
/*	RECT rect; // clip le curseur 
	rect.left=mouse->pos.x-10;
	rect.top=mouse->pos.y-10;
	rect.right=mouse->pos.x+10;
	rect.bottom=mouse->pos.y+10;
	ClipCursor(&rect);*/
	return 0;
}

int Univers::OnMButtonUp(StateMouse *mouse,int face)
{
	DebugMouse2("Univers::OnMButtonUp",mouse,face);
	ClipCursor(NULL);
	if (ce.cursor.curType==cursorRepere) ce.cursor.curType=cursorArrow;
	ce.cursor.midType=cursorRepere;
	ce.cursor.ApplyTypes();
	return 0;
}

//! A chaque rafraichisement graphique, verifie s'il y a des deplacements rapides
int Univers::OnRefresh(Timer *timer)
{
	//Debug0Param("Univers::OnRefresh");
	if (!StateKey::GetState(VK_SHIFT))
	{
		double delta = timer->GetDeltaTime(TRUE)*500.0; // but 20 texel / seconde
		double x=0,y=0,z=0;
		if (StateKey::GetState(VK_UP))		y+=delta;
		if (StateKey::GetState(VK_DOWN))	y-=delta;
		if (StateKey::GetState(VK_LEFT))	x-=delta;
		if (StateKey::GetState(VK_RIGHT))	x+=delta;
		if (StateKey::GetState(VK_PRIOR))	z-=delta;
		if (StateKey::GetState(VK_NEXT))	z+=delta;
		if (x||y||z) Move(x,y,z);
	}
	return 0;
}

int Univers::OnOverIn(Control *oldControl,int face)
{
	Debug2Param("Univers::OnOverIn(oldCtrl:%s,face:%d)",(char*)oldControl->name,face);
	ce.timerRefresh.onTimer.connect(this,OnRefresh);
	ce.cursor.ApplyTypes(cursorArrow,cursorMoving,cursorRepere,cursorNothing);
	return 0;
}

int Univers::OnOverOut(Control *newControl,int face)
{
	Debug2Param("Univers::OnOverOut(newCtrl:%s,face:%d)",(char*)newControl->name,face);
	ce.timerRefresh.onTimer.disconnect(this);
	return 0;
}

// ********************************************************************************

void Univers::ToString(MyString &str)
{
	str.AddFormat("name: %s",name);
}

// ********************************************************************************
