/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "valueEditor.h"
#include "resource.h"
#include "windowTools.h"
#include "MyObject.h"
#include "face.h"
#include "light.h"
#include "controlEngine.h"
#include "screen.h"
#include "control.h"

extern ControlEngine ce;

// implementation de la class ParamValueEditors ************************************

ParamValueEditor::ParamValueEditor()
{
	Zero();
}

void ParamValueEditor::Zero()
{
	object=NULL;
	rect.bottom=rect.left=rect.right=rect.top=0;
	hWndParent=NULL;
	independant=FALSE;
}

void ParamValueEditor::operator=(ParamValueEditor &source)
{
	rect.bottom=source.rect.bottom;
	rect.left=source.rect.left;
	rect.right=source.rect.right;
	rect.top=source.rect.top;
	hWndParent=source.hWndParent;
	independant=source.independant;
}

ParamValueEditor * ParamValueEditor::GetCopy()
{
	ParamValueEditor *newOne = new ParamValueEditor;
	newOne->object = new Object();
	*newOne=*this;
	*newOne->object = *object;
	return newOne;
}

// implementation de la class ValueNamed ************************************

template <class T> 
ValueNamed<T>::ValueNamed()
{
	Zero();
}

template <class T> 
void ValueNamed<T>::Zero()
{
	value = 0;
	description = "";
}

template <class T> 
void ValueNamed<T>::operator=(ValueNamed &source)
{
	value = source.value;
	description = source.description;
}

template <class T> 
void ValueNamed<T>::Set(T _value,char *_desciption)
{
	value = _value;
	description = _desciption;
}

// implementation de la class ValueEditor ************************************

template <class T> 
ValueEditor<T>::ValueEditor()
{
	Zero();
}

template <class T> 
void ValueEditor<T>::Zero()
{
	hItem=NULL;
	info="";
	variable="";
	valueText="";
	minValue=0;
	maxValue=1;
	value = NULL;
	specials.i.SuprAll();
}

template <class T> 
void ValueEditor<T>::operator=(ValueEditor &source)
{
	hItem=source.hItem;
	info=source.info;
	variable=source.variable;
	minValue=source.minValue;
	maxValue=source.maxValue;
	value=source.value;
	valueText=source.valueText;
	specials = source.specials;
}

template <class T>
ValueEditor<T> * ValueEditor<T>::GetCopy()
{
	ValueEditor<T> *newOne = new ValueEditor<T>;
	*newOne=*this;
	return newOne;
}

template <class T> 
void ValueEditor<T>::InterpretedText(MyString &interpretedText,char *_valueText,T *_value)
{
	if (_valueText) valueText=_valueText;
	if (_value) value=_value;

	interpretedText="";

	for (int i=0;i<valueText.GetSize();i++)
		if (valueText[i]=='\t') interpretedText<<*value;
		else interpretedText<<valueText[i];
}

template <class T> 
void ValueEditor<T>::Set(char *_variable,char *_info,T _minValue,T _maxValue,HTREEITEM _hItem)
{
	hItem=_hItem;
	info=_info;
	variable=_variable;
	minValue=_minValue;
	maxValue=_maxValue;
}

template <class T> 
void ValueEditor<T>::MAJ_TreeItem(HWND hWnd)
{
	MyString text;
	InterpretedText(text);
	TVITEM item;
	item.mask = TVIF_HANDLE | TVIF_TEXT;
	item.hItem = hItem;
	item.pszText=text;
	TreeView_SetItem(GetDlgItem(hWnd,IDC_TREE_ELEM),&item);
}

template <class T> 
void ValueEditor<T>::AddSpecial(T special,char *description)
{
	ValueNamed<T> valSpecial;
	valSpecial.Set(special,description);
	specials.i+=valSpecial;
}

template <class T> 
void ValueEditor<T>::MAJ_ValueMin(HWND hWnd)
{
	MAJ_Value(hWnd,minValue);
}

template <class T> 
void ValueEditor<T>::MAJ_ValueMax(HWND hWnd)
{
	MAJ_Value(hWnd,maxValue);
}

template <class T> 
void ValueEditor<T>::MAJ_ValueMids(HWND hWnd)
{
	MAJ_Value(hWnd,(T)((minValue+maxValue)/2.0));
}

template <class T> 
void ValueEditor<T>::MAJ_Value(HWND hWnd,T _value,BOOL refresh)
{
	if (value) *value=_value;
	if (refresh) 
	{
		MAJ_Value(hWnd);
		MAJ_Specials(hWnd);
		MAJ_Slider(hWnd);
	}
}

template <class T> 
void ValueEditor<T>::MAJ_Value(HWND hWnd)
{
	MyString text;
	if (value) text<<*value;
	Edit_SetText(GetDlgItem(hWnd,IDC_EDIT_VALUE),text);
}

template <class T> 
void ValueEditor<T>::Get_Value(HWND hWnd)
{
	char text[64];
	MyString textTmp;
	Edit_GetText(GetDlgItem(hWnd,IDC_EDIT_VALUE),text,64);
	textTmp = text;
	MAJ_Value(hWnd,(T)textTmp.ToDouble(),FALSE);
}

template <class T> 
void ValueEditor<T>::MAJ_Slider(HWND hWnd)
{
	if (!(this&&value)) return;
	BOOL isActive = !(!minValue && maxValue==-1);
	Static_Enable(GetDlgItem(hWnd,IDC_SLIDER_PAD),isActive);
	double xPos = 0;
	xPos = (100.0f*(*value-minValue))/(maxValue-minValue);
	SendMessage(GetDlgItem(hWnd,IDC_SLIDER_PAD), TBM_SETPOS, TRUE, (LPARAM)xPos);
}

template <class T> 
void ValueEditor<T>::MAJ_Specials(HWND hWnd,int num)
{
	HWND hWndCtrl = GetDlgItem(hWnd,IDC_COMBO_SPECIALS);
	ComboBox_ResetContent(hWndCtrl);
	MyString text;
	BOOL active = this&&value?TRUE:FALSE;
	if (active&&specials.GetNbElem())
	{
		ComboBox_Enable(hWndCtrl,TRUE);
		ComboBox_AddString(hWndCtrl,"<Nothing>");
		int numSel = 0;
		int numCur = 0;
		T valueCur = num==-1?*value:specials.i[num].value;

		for (specials.i=0;specials.i.More();specials.i++) 
		{
			text="";
			numCur++;
			T valueSpe = specials.i()->value;
			text<<specials.i()->description<<" (="<<valueSpe<<")";
			if (valueCur==valueSpe) numSel=numCur;
			ComboBox_AddString(hWndCtrl,text);
		}
		ComboBox_SetCurSel(hWndCtrl,numSel);
		if (num!=-1) 
		{ 
			*value=valueCur; 
			MAJ_Value(hWnd); 
		}
	}
	else ComboBox_Enable(GetDlgItem(hWnd,IDC_COMBO_SPECIALS),FALSE);
}

template <class T> 
void ValueEditor<T>::MAJ(HWND hWnd)
{
	HWND hWndMin = GetDlgItem(hWnd,IDC_BUTTON_MIN);
	HWND hWndMax = GetDlgItem(hWnd,IDC_BUTTON_MAX);
	HWND hWndMids = GetDlgItem(hWnd,IDC_BUTTON_MIDS);
	HWND hWndValue= GetDlgItem(hWnd,IDC_EDIT_VALUE);
	HWND hWndInfo= GetDlgItem(hWnd,IDC_STATIC_VARIABLE);
	HWND hWndStaticValue= GetDlgItem(hWnd,IDC_STATIC1);

	BOOL active = this&&value?TRUE:FALSE;
	if (active) 
	{
		MyString text;
		text<<"Value: ["<<minValue<<";"<<maxValue<<"]";
		Static_SetText(hWndStaticValue,text);
		Static_Enable(hWndStaticValue,TRUE);
		Edit_Enable(hWndValue,TRUE);
		MAJ_Value(hWnd);
		Static_Enable(hWndMin,TRUE);
		Static_Enable(hWndMids,TRUE);
		Static_Enable(hWndMax,TRUE);
		Static_SetText(hWndInfo,info);
	}
	else
	{
		Static_SetText(hWndStaticValue,"Value: []");
		Edit_SetText(hWndValue,"");
		Static_Enable(hWndValue,FALSE);
		Static_Enable(hWndMin,FALSE);
		Static_Enable(hWndMids,FALSE);
		Static_Enable(hWndMax,FALSE);
		Static_SetText(hWndInfo,"Select a value to edit it");
	}
	MAJ_Specials(hWnd);
	MAJ_Slider(hWnd);
}

// implementation de la class ValueEditors ************************************

#define IS_NOTHING	0
#define IS_LIGHT	1
#define IS_FOG		2
#define IS_TEXTURE	10 // [10;+inf]

ValueEditors::ValueEditors()
{
}

ValueEditors::~ValueEditors()
{
	Zero();
}

void ValueEditors::Update(Object *object)
{
	if (object)
	{
		VE_int = (CBase_VE_int *)object->thisParent;
		VE_double = (CBase_VE_double *) object->thisParent;
		VE_float = (CBase_VE_float *) object->thisParent;
	}
}

void ValueEditors::Zero()
{
	VE_int = NULL;
	VE_float = NULL;
	VE_double = NULL;
	list.i.SuprAll();

}

#define INSERT_ITEM_TYPED(type,variable,info,paramCur,minValue,maxValue,valueCur) \
	VE_##type##_cur.elem.Zero(); \
	VE_##type##_cur.param = paramCur; \
	VE_##type##_cur.elem.InterpretedText(interpretedText,message,valueCur); \
	elemTree.item.pszText = interpretedText;  \
	elemTree.item.lParam = num++;  \
	hParentItem = TreeView_InsertItem(hWnd,&elemTree); \
	VE_##type##_cur.elem.Set(variable,info,minValue,maxValue,hParentItem) ;

#define INSERT_ITEM_INT(variable,info,param,minValue,maxValue,valueCur) \
	INSERT_ITEM_TYPED(int,variable,info,param,minValue,maxValue,valueCur)
#define INSERT_ITEM_FLOAT(variable,info,param,minValue,maxValue,valueCur) \
	INSERT_ITEM_TYPED(float,variable,info,param,minValue,maxValue,valueCur)
#define INSERT_ITEM_DOUBLE(variable,info,param,minValue,maxValue,valueCur) \
	INSERT_ITEM_TYPED(double,variable,info,param,minValue,maxValue,valueCur)

#define INSERT_INT 	list.i+=VE_int_cur;
#define INSERT_FLOAT list.i+=VE_float_cur;
#define INSERT_DOUBLE list.i+=VE_double_cur;

#define INSERT_ITEM \
	elemTree.item.pszText = message; \
	elemTree.item.lParam = 0; \
	hParentItem = TreeView_InsertItem(hWnd,&elemTree); 
#define INSERT_ITEM_NUM(num) \
	elemTree.item.pszText = message; \
	elemTree.item.lParam = num; \
	hParentItem = TreeView_InsertItem(hWnd,&elemTree);

#define NUM_SELECTED_RENDER		65535
#define NUM_SELECTED_LIGHT		65534
#define NUM_SELECTED_TEXTURE	65533
#define NUM_SELECTED_FONT		65532
#define NUM_SELECTED_PICKING	65531
#define NUM_SELECTED_NOTHING	65530
#define NUM_SELECTED_TRASH		65529

void ValueEditors::MakeDebugStruct(HWND hWnd)
{
	MyString message;
	HTREEITEM hParentItem;
	TVINSERTSTRUCT elemTree;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = NULL;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;

	message="Config";
	INSERT_ITEM_NUM(MAKELPARAM(NUM_SELECTED_NOTHING,0));
	{
		elemTree.hParent = hParentItem;
		message="Render";
		INSERT_ITEM_NUM(MAKELPARAM(NUM_SELECTED_RENDER,0));
		message="Lights";
		INSERT_ITEM_NUM(MAKELPARAM(NUM_SELECTED_LIGHT,0));
		message="Textures";
		INSERT_ITEM_NUM(MAKELPARAM(NUM_SELECTED_TEXTURE,0));
		message="Fonts";
		INSERT_ITEM_NUM(MAKELPARAM(NUM_SELECTED_FONT,0));
		elemTree.hParent = NULL;
	}
	message="Picking";
	INSERT_ITEM_NUM(MAKELPARAM(NUM_SELECTED_PICKING,0));
;
}

void ValueEditors::MakeDebugFaceStruct(HWND hWnd)
{
	MakeDebugStruct(hWnd);
	MakeDebugFaceStruct(hWnd,NULL,ce.primitive.tree.GetRoot());
	MakeDebugFaceStruct(hWnd,NULL,ce.trash.GetRoot());
}

void ValueEditors::MakeDebugFaceStruct(HWND hWnd,HTREEITEM hParentItem,FaceNode *node)
{
	MyString message;
	TVINSERTSTRUCT elemTree;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = hParentItem;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;
	message=node->elem->name;
	INSERT_ITEM_NUM(MAKELPARAM(node->elem->idControl,node->elem->idFace));
	for (node->childs.i=0;node->childs.i.More();node->childs.i++) 
		MakeDebugFaceStruct(hWnd,hParentItem,node->childs.i.GetElem());
}

void ValueEditors::MakeDebugControlStruct(HWND hWnd)
{
	MakeDebugStruct(hWnd);
	MakeDebugControlStruct(hWnd,NULL,ce.primitive.tree.GetRoot(),-1);
	HTREEITEM hParentItem;
	TVINSERTSTRUCT elemTree;
	MyString message;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = NULL;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;
	message = "Trash";
	INSERT_ITEM_NUM(MAKELPARAM(NUM_SELECTED_TRASH,0));
	MakeDebugControlStruct(hWnd,hParentItem,ce.trash.GetRoot(),-1);
}

void ValueEditors::MakeDebugControlStruct(HWND hWnd,HTREEITEM hParentItem,FaceNode *node,int oldIdControl)
{
	if (oldIdControl!=node->elem->idControl)
	{
		MyString message;
		TVINSERTSTRUCT elemTree;
		elemTree.hInsertAfter = TVI_LAST;
		elemTree.hParent = hParentItem;
		elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;
		elemTree.item.lParam = node->elem->idControl;
		Control *control = Control::Find(node->elem->idControl);
		if (control)
		{
			message=control->name;
			elemTree.item.pszText = message;
			hParentItem = TreeView_InsertItem(hWnd,&elemTree);
		}
	}
	for (node->childs.i=0;node->childs.i.More();node->childs.i++) 
		MakeDebugControlStruct(hWnd,hParentItem,node->childs.i.GetElem(),node->elem->idControl);
}

void ValueEditors::MakeElemControlStruct(HWND hWnd,Control *control)
{
	if (!control) return;

	Zero();
	CBase_VE_double VE_double_cur;
	CBase_VE_int VE_int_cur;
	CBase_VE_float VE_float_cur;
	VE_double_cur.param = 12;
	MyString interpretedText;

	HTREEITEM hParentItem,hParentItem2,hParentItem3,hParentItem4;
	TVINSERTSTRUCT elemTree;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = NULL;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;

	int num = 1;
	MyString message;
	message.Format("Name: %s [id]: %d",(char *)control->name,control->id);
	INSERT_ITEM;

	if (control->parent)
	{
		message.Format("Parent [id,face,name]: %d , %d , %s",control->parent->id,control->parentFace,(char *)control->parent->name);
		INSERT_ITEM;
	}

	message.Format("Mode [enable,view,sleeped]: %d , %d , %d",control->isEnable,control->isView,control->sleepedDelay);
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="isEnable: \t";
		INSERT_ITEM_INT("control.isEnable","control dois il etre utilis",FALSE,0,-1,&control->isEnable);
		VE_int_cur.elem.AddSpecial(FALSE,"false");
		VE_int_cur.elem.AddSpecial(TRUE,"true");
		INSERT_INT;
		message="isView: \t";
		INSERT_ITEM_INT("control.isView","control dois il s'afficher",FALSE,0,-1,&control->isView);
		VE_int_cur.elem.AddSpecial(FALSE,"false");
		VE_int_cur.elem.AddSpecial(TRUE,"true");
		INSERT_INT;
		message="sleepedDelay: \t";
		INSERT_ITEM_INT("control.sleepedDelay","temps d'inaction avant evenement OnSleeped",FALSE,0,10000,&control->sleepedDelay);
		INSERT_INT;
		elemTree.hParent = NULL;
	}

	message.Format("Events (%d links)",control->onControl.GetNbLinks()); 
	INSERT_ITEM;
	{
		hParentItem2 = hParentItem;
		elemTree.hParent = hParentItem;
		message.Format("onControl (%d links)",control->onControl.GetNbLinks());
		INSERT_ITEM;
		{
			hParentItem3 = hParentItem;
			elemTree.hParent = hParentItem;
			message.Format("mouse (%d links)",control->onControl.mouse.GetNbLinks());
			INSERT_ITEM;
			{
				hParentItem4 = hParentItem;
				elemTree.hParent = hParentItem;
				message.Format("left (%d links)",control->onControl.mouse.left.GetNbLinks());
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message.Format("down (%d links)",control->onControl.mouse.left.down.size());
					INSERT_ITEM;
					message.Format("up (%d links)",control->onControl.mouse.left.up.size());
					INSERT_ITEM;
					message.Format("click (%d links)",control->onControl.mouse.left.click.size());
					INSERT_ITEM;
					message.Format("doubleClick (%d links)",control->onControl.mouse.left.doubleClick.size());
					INSERT_ITEM;
					elemTree.hParent = hParentItem4;
				}
				message.Format("mid (%d links)",control->onControl.mouse.mid.GetNbLinks());
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message.Format("down (%d links)",control->onControl.mouse.mid.down.size());
					INSERT_ITEM;
					message.Format("up (%d links)",control->onControl.mouse.mid.up.size());
					INSERT_ITEM;
					message.Format("click (%d links)",control->onControl.mouse.mid.click.size());
					INSERT_ITEM;
					message.Format("doubleClick (%d links)",control->onControl.mouse.mid.doubleClick.size());
					INSERT_ITEM;
					elemTree.hParent = hParentItem4;
				}
				message.Format("right (%d links)",control->onControl.mouse.right.GetNbLinks());
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message.Format("down (%d links)",control->onControl.mouse.right.down.size());
					INSERT_ITEM;
					message.Format("up (%d links)",control->onControl.mouse.right.up.size());
					INSERT_ITEM;
					message.Format("click (%d links)",control->onControl.mouse.right.click.size());
					INSERT_ITEM;
					message.Format("doubleClick (%d links)",control->onControl.mouse.right.doubleClick.size());
					INSERT_ITEM;
					elemTree.hParent = hParentItem4;
				}
				message.Format("move (%d links)",control->onControl.mouse.move.size());
				INSERT_ITEM;
				//message.Format("down (%d links)",control->onControl.mouse.down.size());
				//INSERT_ITEM;
				//message.Format("up (%d links)",control->onControl.mouse.up.size());
				//INSERT_ITEM;
				message.Format("wheel (%d links)",control->onControl.mouse.wheel.size());
				INSERT_ITEM;
				elemTree.hParent = hParentItem3;
			}
			message.Format("key (%d links)",control->onControl.key.GetNbLinks());
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message.Format("down (%d links)",control->onControl.key.down.size());
				INSERT_ITEM;
				message.Format("up (%d links)",control->onControl.key.up.size());
				INSERT_ITEM;
				message.Format("click (%d links)",control->onControl.key.click.size());
				INSERT_ITEM;
				message.Format("doubleClick (%d links)",control->onControl.key.doubleClick.size());
				INSERT_ITEM;
				elemTree.hParent = hParentItem3;
			}
			message.Format("over (%d links)",control->onControl.over.GetNbLinks());
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message.Format("out (%d links)",control->onControl.over.out.size());
				INSERT_ITEM;
				message.Format("in (%d links)",control->onControl.over.in.size());
				INSERT_ITEM;
				elemTree.hParent = hParentItem3;
			}
			message.Format("faceover (%d links)",control->onControl.faceOver.GetNbLinks());
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message.Format("out (%d links)",control->onControl.faceOver.out.size());
				INSERT_ITEM;
				message.Format("in (%d links)",control->onControl.faceOver.in.size());
				INSERT_ITEM;
				elemTree.hParent = hParentItem3;
			}
			message.Format("sleeped (%d links)",control->onControl.sleeped.size());
			INSERT_ITEM;

			elemTree.hParent = hParentItem2;
		}	
		elemTree.hParent = NULL;
	}
}

void ValueEditors::MakeRenderStruct(HWND hWnd)
{
	Zero();
	CBase_VE_double VE_double_cur;
	CBase_VE_int VE_int_cur;
	CBase_VE_float VE_float_cur;
	VE_double_cur.param = 12;
	MyString interpretedText;

	HTREEITEM hParentItem,hParentItem2;
	TVINSERTSTRUCT elemTree;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = NULL;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;

	int num = 1;
	MyString message;
	message<<clear<<"Region [enable,nbPoly,nbOr]: "<<ce.screenMain.region.enable<<" , "<<ce.screenMain.region.points.GetNbElem()<<" , "<<ce.screenMain.region.nbOr;
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="enable: \t";
		INSERT_ITEM_INT("region.enable","active le regionage",FALSE,0,1,&ce.screenMain.region.enable);
		INSERT_INT;
		message<<clear<<"box [xMin,xMax,yMin,yMax]: "<<ce.screenMain.region.xMin<<" , "<<ce.screenMain.region.xMax<<" , "<<ce.screenMain.region.yMin<<" , "<<ce.screenMain.region.yMax;
		INSERT_ITEM;
		elemTree.hParent = NULL;
	}
	message<<clear<<"Windows [title,fullScr]: "<<ce.screenMain.enableTitle<<" , "<<ce.screenMain.enableFullScreen;
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="enableTitle: \t";
		INSERT_ITEM_INT("enableTitle","active le titre de la fenetre",FALSE,0,1,&ce.screenMain.enableTitle);
		INSERT_INT;
		message="enableFullScreen: \t";
		INSERT_ITEM_INT("enableFullScreen","active le mode plein ecran",FALSE,0,1,&ce.screenMain.enableFullScreen);
		INSERT_INT;
		message="enableAntiAliasing: \t";
		INSERT_ITEM_INT("enableAntiAliasing","active le mode anti-aliasing (active uniquement si la face l'utiulise)",FALSE,0,1,&ce.enableAntiAliasing);
		INSERT_INT;
		message="antiAliasingQuality: \t";
		INSERT_ITEM_INT("antiAliasingQuality","precision de l'anti-aliasing",FALSE,0,7,&ce.antiAliasingQuality);
		INSERT_INT;
		message="enableCursor: \t";
		INSERT_ITEM_INT("enableCursor","active l'affichage d'un curseur",FALSE,0,1,&ce.enableCursor);
		INSERT_INT;
		message="enableRepere: \t";
		INSERT_ITEM_INT("enableRepere","active l'affichage d'un repere 3d",FALSE,0,1,&ce.enableRepere);
		INSERT_INT;
		message="enableLightView: \t";
		INSERT_ITEM_INT("enableLightView","active les boite representant les lumieres",FALSE,0,1,&ce.enableLightView);
		INSERT_INT;
		message="enableBackGround: \t";
		INSERT_ITEM_INT("enableBackGround","active le fond",FALSE,0,1,&ce.enableBackGround);
		INSERT_INT;
		elemTree.hParent = NULL;
	}

	message<<clear<<"Tools [inertia,monitor]: "<<ce.enableInertia<<" , "<<ce.monitor.enable;
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="enableInertia: \t";
		INSERT_ITEM_INT("enableInertia","active le mode d'inertie",FALSE,0,1,&ce.enableInertia);
		INSERT_INT;
		message="enableViewInertia: \t";
		INSERT_ITEM_INT("enableViewInertia","active l'affichage du mode d'inertie",FALSE,0,1,&ce.enableViewInertia);
		INSERT_INT;
		message="enableMonitor: \t";
		INSERT_ITEM_INT("enableMonitor","active l'utilisation du moniteur de performance",FALSE,0,1,&ce.monitor.enable);
		INSERT_INT;
		elemTree.hParent = NULL;
	}

	message<<clear<<"Fog [enable,start,end]: "<<ce.fog.isEnable<<" , "<<ce.fog.start<<" , "<<ce.fog.end;
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="enable: \t";
		INSERT_ITEM_INT("fog.enable","active le brouillard",IS_FOG,0,1,&ce.fog.isEnable);
		INSERT_INT;
		message="start: \t";
		INSERT_ITEM_FLOAT("fog.start","distance z de debut du brouillard",IS_FOG,0,100,&ce.fog.start);
		INSERT_FLOAT;
		message="end: \t";
		INSERT_ITEM_FLOAT("fog.end","distance z de fin de brouillard",IS_FOG,0,100,&ce.fog.end);
		INSERT_FLOAT;
		message="density: \t";
		INSERT_ITEM_FLOAT("fog.density","densit du brouillard (utilis en mode EXP ou EXP2)",IS_FOG,0,0.5,&ce.fog.density);
		INSERT_FLOAT;
		message="mode: \t";
		INSERT_ITEM_INT("fog.mode","type de calcul de brouillard",IS_FOG,0,-1,(int *)&ce.fog.mode);
		VE_float_cur.elem.AddSpecial(GL_LINEAR,"Linear");
		VE_float_cur.elem.AddSpecial(GL_EXP,"Exp");
		VE_float_cur.elem.AddSpecial(GL_EXP2,"Exp2");
		INSERT_INT;
		message="color ";
		ce.fog.color.ToString(message);
		INSERT_ITEM;
		{
			hParentItem2 = hParentItem;
			elemTree.hParent = hParentItem;
			message="enable: \t";
			INSERT_ITEM_INT("fog.color.enable","active la couleur",IS_FOG,0,1,&ce.fog.color.isEnable);
			INSERT_INT;
			message="r: \t";
			INSERT_ITEM_FLOAT("fog.color.r","Composante rouge",IS_FOG,0,1,&ce.fog.color.r);
			INSERT_FLOAT;
			message="g: \t";
			INSERT_ITEM_FLOAT("fog.color.g","Composante verte",IS_FOG,0,1,&ce.fog.color.g);
			INSERT_FLOAT;
			message="b: \t";
			INSERT_ITEM_FLOAT("fog.color.b","Composante bleu",IS_FOG,0,1,&ce.fog.color.b);
			INSERT_FLOAT;
			message="a: \t";
			INSERT_ITEM_FLOAT("fog.color.a","Composante alpha (pour blending)",IS_FOG,0,1,&ce.fog.color.a);
			INSERT_FLOAT;
			elemTree.hParent = hParentItem;
		}
		elemTree.hParent = NULL;
	}
}

void ValueEditors::MakeLightsStruct(HWND hWnd)
{
	Zero();
	CBase_VE_double VE_double_cur;
	CBase_VE_int VE_int_cur;
	CBase_VE_float VE_float_cur;
	MyString interpretedText;

	HTREEITEM hParentItem,hParentItem2,hParentItem3;
	TVINSERTSTRUCT elemTree;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = NULL;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;

	int num = 1;
	MyString message;
	Light *lightCur;
	BOOL *lightMoveCur;
	MyString lightName,paramName;
	for (int i=0;i<4;i++)
	{
		lightCur=&ce.lights[i];
		lightMoveCur=&ce.lightsMove[i];

		lightName<<clear<<"light"<<i;
		message<<clear<<lightName<<" [enable]: "<<lightCur->isEnable;
		INSERT_ITEM;
		{
			hParentItem2 = hParentItem;
			elemTree.hParent = hParentItem;
			message="enable: \t";
			paramName<<clear<<lightName<<".enable";
			INSERT_ITEM_INT(paramName,"active la lumiere",IS_LIGHT,0,1,&lightCur->isEnable);
			INSERT_INT;

			message="Material";
			INSERT_ITEM;
			{
				hParentItem3 = hParentItem;
				elemTree.hParent = hParentItem;
				message="ambient: ";
				lightCur->material.ambient.ToString(message);
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					paramName<<clear<<lightName<<".ambient.enable";
					INSERT_ITEM_INT(paramName,"enable l'utilisation des couleurs",IS_LIGHT,0,1,&lightCur->material.ambient.isEnable);
					INSERT_INT;
					message="r: \t";
					paramName<<clear<<lightName<<".ambient.r";
					INSERT_ITEM_FLOAT(paramName,"Composante rouge",IS_LIGHT,0,1,&lightCur->material.ambient.r);
					INSERT_FLOAT;
					message="g: \t";
					paramName<<clear<<lightName<<".ambient.g";
					INSERT_ITEM_FLOAT(paramName,"Composante verte",IS_LIGHT,0,1,&lightCur->material.ambient.g);
					INSERT_FLOAT;
					message="b: \t";
					paramName<<clear<<lightName<<".ambient.b";
					INSERT_ITEM_FLOAT(paramName,"Composante bleu",IS_LIGHT,0,1,&lightCur->material.ambient.b);
					INSERT_FLOAT;
					message="a: \t";
					paramName<<clear<<lightName<<".ambient.a";
					INSERT_ITEM_FLOAT(paramName,"Composante alpha (pour blending)",IS_LIGHT,0,1,&lightCur->material.ambient.a);
					INSERT_FLOAT;
					elemTree.hParent = hParentItem3;
				}

				message="diffuse: ";
				lightCur->material.diffuse.ToString(message);
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					paramName<<clear<<lightName<<".diffuse.enable";
					INSERT_ITEM_INT(paramName,"enable l'utilisation des couleurs",IS_LIGHT,0,1,&lightCur->material.diffuse.isEnable);
					INSERT_INT;
					message="r: \t";
					paramName<<clear<<lightName<<".diffuse.r";
					INSERT_ITEM_FLOAT(paramName,"Composante rouge",IS_LIGHT,0,1,&lightCur->material.diffuse.r);
					INSERT_FLOAT;
					message="g: \t";
					paramName<<clear<<lightName<<".diffuse.g";
					INSERT_ITEM_FLOAT(paramName,"Composante verte",IS_LIGHT,0,1,&lightCur->material.diffuse.g);
					INSERT_FLOAT;
					message="b: \t";
					paramName<<clear<<lightName<<".diffuse.b";
					INSERT_ITEM_FLOAT(paramName,"Composante bleu",IS_LIGHT,0,1,&lightCur->material.diffuse.b);
					INSERT_FLOAT;
					message="a: \t";
					paramName<<clear<<lightName<<".diffuse.a";
					INSERT_ITEM_FLOAT(paramName,"Composante alpha (pour blending)",IS_LIGHT,0,1,&lightCur->material.diffuse.a);
					INSERT_FLOAT;
					elemTree.hParent = hParentItem3;
				}

				message="specular: ";
				lightCur->material.specular.ToString(message);
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					paramName<<clear<<lightName<<".specular.enable";
					INSERT_ITEM_INT(paramName,"enable l'utilisation des couleurs",IS_LIGHT,0,1,&lightCur->material.specular.isEnable);
					INSERT_INT;
					message="r: \t";
					paramName<<clear<<lightName<<".specular.r";
					INSERT_ITEM_FLOAT(paramName,"Composante rouge",IS_LIGHT,0,1,&lightCur->material.specular.r);
					INSERT_FLOAT;
					message="g: \t";
					paramName<<clear<<lightName<<".specular.g";
					INSERT_ITEM_FLOAT(paramName,"Composante verte",IS_LIGHT,0,1,&lightCur->material.specular.g);
					INSERT_FLOAT;
					message="b: \t";
					paramName<<clear<<lightName<<".specular.b";
					INSERT_ITEM_FLOAT(paramName,"Composante bleu",IS_LIGHT,0,1,&lightCur->material.specular.b);
					INSERT_FLOAT;
					message="a: \t";
					paramName<<clear<<lightName<<".specular.a";
					INSERT_ITEM_FLOAT(paramName,"Composante alpha (pour blending)",IS_LIGHT,0,1,&lightCur->material.specular.a);
					INSERT_FLOAT;	
					elemTree.hParent = hParentItem2;
				}
			}
			message="position: ";
			lightCur->position.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				paramName<<clear<<lightName<<".position.enable";
				INSERT_ITEM_INT(paramName,"enable l'utilisation du positionement",IS_LIGHT,0,1,&lightCur->position.isEnable);
				INSERT_INT;
				message="x: \t";
				paramName<<clear<<lightName<<".position.x";
				INSERT_ITEM_FLOAT(paramName,"position x",IS_LIGHT,-1024,1024,&lightCur->position.x);
				VE_float_cur.elem.AddSpecial(180,"paralelle");
				INSERT_FLOAT;
				message="y: \t";
				paramName<<clear<<lightName<<".position.y";
				INSERT_ITEM_FLOAT(paramName,"position y",IS_LIGHT,-1024,1024,&lightCur->position.y);
				INSERT_FLOAT;
				message="z: \t";
				paramName<<clear<<lightName<<".position.z";
				INSERT_ITEM_FLOAT(paramName,"position z",IS_LIGHT,-1024,1024,&lightCur->position.z);
				INSERT_FLOAT;
				message="w: \t";
				paramName<<clear<<lightName<<".position.w";
				INSERT_ITEM_FLOAT(paramName,"directionnel ou ponctuel",IS_LIGHT,0,-1,&lightCur->position.w);
				VE_float_cur.elem.AddSpecial(0,"directionnel");
				VE_float_cur.elem.AddSpecial(1,"ponctuel");
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}
				
			message="direction: ";
			lightCur->direction.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				paramName<<clear<<lightName<<".direction.enable";
				INSERT_ITEM_INT(paramName,"enable l'utilisation des direction",IS_LIGHT,0,1,&lightCur->direction.isEnable);
				INSERT_INT;
				message="x: \t";
				paramName<<clear<<lightName<<".direction.x";
				INSERT_ITEM_FLOAT(paramName,"direction x",IS_LIGHT,-1024,1024,&lightCur->direction.x);
				INSERT_FLOAT;
				message="y: \t";
				paramName<<clear<<lightName<<".direction.y";
				INSERT_ITEM_FLOAT(paramName,"direction y",IS_LIGHT,-1024,1024,&lightCur->direction.y);
				INSERT_FLOAT;
				message="z: \t";
				paramName<<clear<<lightName<<".direction.z";
				INSERT_ITEM_FLOAT(paramName,"direction z",IS_LIGHT,-1024,1024,&lightCur->direction.z);
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message="cutoff: ";
			lightCur->cutoff.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				paramName<<clear<<lightName<<".cutoff.enable";
				INSERT_ITEM_INT(paramName,"enable l'utilisation de l'ouverture",IS_LIGHT,0,1,&lightCur->cutoff.isEnable);
				INSERT_INT;
				message="value: \t";
				paramName<<clear<<lightName<<".cutoff.value";
				INSERT_ITEM_FLOAT(paramName,"valeur de l'ouverture",IS_LIGHT,0,90,&lightCur->cutoff.elem);
				VE_int_cur.elem.AddSpecial(180,"paralelle");
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message="exponent: ";
			lightCur->exponent.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				paramName<<clear<<lightName<<".exponent.enable";
				INSERT_ITEM_INT(paramName,"enable l'utilisation du exponent",IS_LIGHT,0,1,&lightCur->exponent.isEnable);
				INSERT_INT;
				message="value: \t";
				paramName<<clear<<lightName<<".exponent.value";
				INSERT_ITEM_FLOAT(paramName,"valeur du exponent ",IS_LIGHT,0,128,&lightCur->exponent.elem);
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message="attenuation";
			BOOL atLeast = FALSE;
			if (lightCur->attenuationCst.isEnable) 
			{ 
				message<<": "<<lightCur->attenuationCst.elem; 
				atLeast = TRUE; 
			}
			if (lightCur->attenuationLin.isEnable) 
			{ 
				if (atLeast) message<<"+";
				else message<<": ";
				message<<lightCur->attenuationLin.elem<<"*x"; 
				atLeast = TRUE; 
			}
			if (lightCur->attenuationQuad.isEnable) 
			{ 
				if (atLeast) message<<"+";
				else message<<": ";
				message<<lightCur->attenuationQuad.elem<<"*x2"; 
				atLeast = TRUE; 
			}
			INSERT_ITEM;
			{
				hParentItem2 = hParentItem;
				elemTree.hParent = hParentItem;
				message<<clear<<"constant [enable,value]:"<<lightCur->attenuationCst.isEnable<<" , "<<lightCur->attenuationCst.elem;
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					paramName<<clear<<lightName<<".attenuationCst.enable";
					INSERT_ITEM_INT(paramName,"enable l'utilisation de l'attenuation",IS_LIGHT,0,1,&lightCur->attenuationCst.isEnable);
					INSERT_INT;
					message="value: \t";
					paramName<<clear<<lightName<<".attenuationCst.value";
					INSERT_ITEM_FLOAT(paramName,"valeur de l'attenuation",IS_LIGHT,0,90,&lightCur->attenuationCst.elem);
					VE_float_cur.elem.AddSpecial(180,"paralelle");
					INSERT_FLOAT;
					elemTree.hParent = hParentItem2;
				}
				message<<clear<<"lineaire [enable,value]:"<<lightCur->attenuationLin.isEnable<<" , "<<lightCur->attenuationLin.elem;
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					paramName<<clear<<lightName<<".attenuationLin.enable";
					INSERT_ITEM_INT(paramName,"enable l'utilisation de l'attenuation",IS_LIGHT,0,1,&lightCur->attenuationLin.isEnable);
					INSERT_INT;
					message="value: \t";
					paramName<<clear<<lightName<<".attenuationLin.value";
					INSERT_ITEM_FLOAT(paramName,"valeur de l'attenuation",IS_LIGHT,0,90,&lightCur->attenuationLin.elem);
					VE_float_cur.elem.AddSpecial(180,"paralelle");
					INSERT_FLOAT;
					elemTree.hParent = hParentItem2;
				}
				message<<clear<<"quadratique [enable,value]:"<<lightCur->attenuationQuad.isEnable<<" , "<<lightCur->attenuationQuad.elem;
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					paramName<<clear<<lightName<<".attenuationQuad.enable";
					INSERT_ITEM_INT(paramName,"enable l'utilisation de l'attenuation",IS_LIGHT,0,1,&lightCur->attenuationQuad.isEnable);
					INSERT_INT;
					message="value: \t";
					paramName<<clear<<lightName<<".attenuationQuad.value";
					INSERT_ITEM_FLOAT(paramName,"valeur de l'attenuation",IS_LIGHT,0,90,&lightCur->attenuationQuad.elem);
					VE_float_cur.elem.AddSpecial(180,"paralelle");
					INSERT_FLOAT;
					elemTree.hParent = hParentItem2;
				}
			}
			elemTree.hParent=NULL;
		}
	}
}

void ValueEditors::MakeTexturesStruct(HWND hWnd)
{
	Zero();
	CBase_VE_double VE_double_cur;
	CBase_VE_int VE_int_cur;
	CBase_VE_float VE_float_cur;
	MyString interpretedText;

	HTREEITEM hParentItem,hParentItem2;
	TVINSERTSTRUCT elemTree;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = NULL;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;

	int num = 1;
	MyString message,textureNum,temp;

	int numTexture = -1;
	for (Texture::list.i=0;Texture::list.i.More();Texture::list.i++)
	{
		numTexture++;
		Texture *texture = Texture::list.i.GetElem();

		textureNum.Format("texture%d",numTexture);
		message.Format("%s [name,id]: \"%s\" , %d",(char*)textureNum,(char*)texture->name,texture->id);
		INSERT_ITEM;
		{
			hParentItem2 = hParentItem;
			elemTree.hParent = hParentItem;
			//message="source: \"";message<<texture->source<<"\"";
			message.Format("source: \"%s\"",(char*)texture->source);
			INSERT_ITEM;
			message.Format("size [x,y]: %d , %d",texture->xSize,texture->ySize);
			INSERT_ITEM;
			message="useResizing: \t";
			temp=textureNum;temp<<".useResizing";
			INSERT_ITEM_INT(temp,"utilse le redimentionement (parametre scale) ou changement a la source",FALSE,0,1,&texture->useResizing);
			INSERT_INT;
			message="scale [x,y]: ";message<<texture->xScale<<" , "<<texture->yScale;
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="x: \t";
				temp=textureNum;temp<<".xScale";
				INSERT_ITEM_DOUBLE(temp,"scaling de la texture dans les x (<1 si texture original non exposant de 2)",FALSE,0,1,&texture->xScale);
				INSERT_DOUBLE;
				message="y: \t";
				temp=textureNum;temp<<".yScale";
				INSERT_ITEM_DOUBLE(temp,"scaling de la texture dans les y (<1 si texture original non exposant de 2)",FALSE,0,1,&texture->yScale);
				INSERT_DOUBLE;
				elemTree.hParent = hParentItem2;
			}

			message="border ";
			texture->border.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				temp=textureNum;temp<<".border.enable";
				INSERT_ITEM_INT(temp,"enable l'utilisation des couleurs",IS_TEXTURE+numTexture,0,1,&texture->border.isEnable);
				INSERT_INT;
				message="r: \t";
				temp=textureNum;temp<<".border.r";
				INSERT_ITEM_FLOAT(temp,"Composante rouge",IS_TEXTURE+numTexture,-1,1,&texture->border.r);
				INSERT_FLOAT;
				message="g: \t";
				temp=textureNum;temp<<".border.g";
				INSERT_ITEM_FLOAT(temp,"Composante verte",IS_TEXTURE+numTexture,-1,1,&texture->border.g);
				INSERT_FLOAT;
				message="b: \t";
				temp=textureNum;temp<<".border.b";
				INSERT_ITEM_FLOAT(temp,"Composante bleu",IS_TEXTURE+numTexture,-1,1,&texture->border.b);
				INSERT_FLOAT;
				message="a: \t";
				temp=textureNum;temp<<".border.a";
				INSERT_ITEM_FLOAT(temp,"Composante alpha (pour blending)",IS_TEXTURE+numTexture,-1,1,&texture->border.a);
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message="envColor ";
			texture->envColor.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				temp=textureNum;temp<<".envColor.enable";
				INSERT_ITEM_INT(temp,"enable l'utilisation des couleurs",IS_TEXTURE+numTexture,0,1,&texture->envColor.isEnable);
				INSERT_INT;
				message="r: \t";
				temp=textureNum;temp<<".envColor.r";
				INSERT_ITEM_FLOAT(temp,"Composante rouge",IS_TEXTURE+numTexture,-1,1,&texture->envColor.r);
				INSERT_FLOAT;
				message="g: \t";
				temp=textureNum;temp<<".envColor.g";
				INSERT_ITEM_FLOAT(temp,"Composante verte",IS_TEXTURE+numTexture,-1,1,&texture->envColor.g);
				INSERT_FLOAT;
				message="b: \t";
				temp=textureNum;temp<<".envColor.b";
				INSERT_ITEM_FLOAT(temp,"Composante bleu",IS_TEXTURE+numTexture,-1,1,&texture->envColor.b);
				INSERT_FLOAT;
				message="a: \t";
				temp=textureNum;temp<<".envColor.a";
				INSERT_ITEM_FLOAT(temp,"Composante alpha (pour blending)",IS_TEXTURE+numTexture,-1,1,&texture->envColor.a);
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message="filter [min,mag]: ";message<<texture->minFilter<<" , "<<texture->magFilter;
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="min: \t";
				temp=textureNum;temp<<".minFilter";
				INSERT_ITEM_INT(temp,"type de traitement si la taille de la texture orginal est plus petite qu'a l'affichage",IS_TEXTURE+numTexture,0,-1,(int *)&texture->minFilter);
				VE_int_cur.elem.AddSpecial(GL_NEAREST,"Nearest");
				VE_int_cur.elem.AddSpecial(GL_LINEAR,"Linear");	 
				INSERT_INT;
				message="mag: \t";
				temp=textureNum;temp<<".magFilter";
				INSERT_ITEM_INT(temp,"type de traitement si la taille de la texture original est plus grande qu'a l'affichage",IS_TEXTURE+numTexture,0,-1,(int *)&texture->magFilter);
				VE_int_cur.elem.AddSpecial(GL_NEAREST,"Nearest");
				VE_int_cur.elem.AddSpecial(GL_LINEAR,"Linear");	 
				INSERT_INT;
				elemTree.hParent = hParentItem2;
			}
			message="wrap [Hori,Verti]: ";message<<texture->wrapS<<" , "<<texture->wrapT;
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="Horizontal: \t";
				temp<<clear<<textureNum<<".wrapS";
				INSERT_ITEM_INT(temp,"type d'envelopement horizontal",IS_TEXTURE+numTexture,0,-1,(int *)&texture->wrapS);
				VE_int_cur.elem.AddSpecial(GL_CLAMP ,"Clamp");
				VE_int_cur.elem.AddSpecial(GL_REPEAT,"Repeat");	 
				INSERT_INT;
				message="Vertical: \t";
				temp<<clear<<textureNum;temp<<".wrapT";
				INSERT_ITEM_INT(temp,"type d'envelopement vertical",IS_TEXTURE+numTexture,0,-1,(int *)&texture->wrapT);
				VE_int_cur.elem.AddSpecial(GL_CLAMP ,"Clamp");
				VE_int_cur.elem.AddSpecial(GL_REPEAT,"Repeat");	 
				INSERT_INT;
				elemTree.hParent = hParentItem2;
			}
			elemTree.hParent = NULL;
		}
	}
}

/*!
\todo Ajouter le nom du fichier ttf, type, taille etc ...
*/

void ValueEditors::MakeFontsStruct(HWND hWnd)
{
	Zero();
	CBase_VE_double VE_double_cur;
	CBase_VE_int VE_int_cur;
	CBase_VE_float VE_float_cur;
	MyString interpretedText;

	HTREEITEM hParentItem;//,hParentItem2;
	TVINSERTSTRUCT elemTree;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = NULL;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;

	int num = 1;
	MyString message,fontTxt,temp;

	int numFont = -1;

	MyList<FTFont *> fonts;
	fonts.i+=ce.fontBitmap;
	fonts.i+=ce.fontExtrude;
	fonts.i+=ce.fontOutline;
	fonts.i+=ce.fontPixmap;
	fonts.i+=ce.fontPolygon;
	fonts.i+=ce.fontTexture;
	for (fonts.i=0;fonts.i.More();fonts.i++)
	{
		FTFont *font = fonts.i.GetElem();
		numFont++;
		int size = font->FaceSize();
		float ascender = font->Ascender();
		float descender = font->Descender();

		fontTxt.Format("font%d",numFont);
		message.Format("%s [name,asc,des]: \"%s\" , %0.2f , %0.2f",(char*)fontTxt,ascender,descender);
		INSERT_ITEM;
/*			{
			hParentItem2 = hParentItem;
			elemTree.hParent = hParentItem;

			message="useResizing: \t";
			temp=textureNum;temp<<".useResizing";
			INSERT_ITEM_INT(temp,"utilse le redimentionement (parametre scale) ou changement a la source",FALSE,0,1,&texture->useResizing);
			INSERT_INT;
			elemTree.hParent = NULL;
		}*/
	}
}

BOOL ValueEditors::MakeElemFaceStruct(HWND hWnd,FaceNode *node)
{
	if (!node) return FALSE;
	Zero();
	CBase_VE_double VE_double_cur;
	CBase_VE_int VE_int_cur;
	CBase_VE_float VE_float_cur;
	MyString interpretedText;

	Face *face = node->elem;
	HTREEITEM hParentItem,hParentItem2;
	TVINSERTSTRUCT elemTree;
	elemTree.hInsertAfter = TVI_LAST;
	elemTree.hParent = NULL;
	elemTree.item.mask = TVIF_TEXT | TVIF_PARAM;

	int num = 1;
	MyString message;
	message.Format("Name: %s [idControl,idFace]: %d,%d",(char*)face->name,face->idControl,face->idFace);
	INSERT_ITEM;
	message<<clear<<"Angle [x,y,z]: "<<face->angle.x<<" , "<<face->angle.y<<" , "<<face->angle.z;
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="angle.x: \t";
		INSERT_ITEM_DOUBLE("Angle.x","Angle X de l'univers",FALSE,-180,180,&face->angle.x);
		INSERT_DOUBLE;
		message="angle.y: \t";
		INSERT_ITEM_DOUBLE("Angle.y","Angle Y de l'univers",FALSE,-180,180,&face->angle.y);
		INSERT_DOUBLE;
		message="angle.z: \t";
		INSERT_ITEM_DOUBLE("Angle.z","Angle Z de l'univers",FALSE,-180,180,&face->angle.z);
		INSERT_DOUBLE;
		message="ratioXY: \t";
		INSERT_ITEM_DOUBLE("RatioXY","ration entre les X et Y",FALSE,-180,180,&face->ratioXY);
		INSERT_DOUBLE;
		elemTree.hParent = NULL;
	}
	message.Format("Mode [view,aalias,fog,blend,line]: %d , %d , %d , %d , %d",face->view,face->antialias,face->fog,face->blending,face->line);
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="view: \t";
		INSERT_ITEM_INT("view","Face doit etre visualis ?",FALSE,0,1,&face->view);
		INSERT_INT;
		message="antialias: \t";
		INSERT_ITEM_INT("antialias","Antialising active?",FALSE,0,1,&face->antialias);
		INSERT_INT;
		message="fog: \t";
		INSERT_ITEM_INT("fog","Le brouillard concerne il cette face ? (si brouillard general est activ)",FALSE,0,1,&face->fog);
		INSERT_INT;
		message="blending: \t";
		INSERT_ITEM_INT("blending","Mode de transparance activ ? (utilise les valeur alphas)",FALSE,0,1,&face->blending);
		INSERT_INT;

		message="line: \t";
		INSERT_ITEM_FLOAT("line","Taille du contour de la surface en ligne, booleen si antialiasing de ligne activ",FALSE,0,10,&face->line);
		INSERT_FLOAT;

		message="lineMaterials";
		INSERT_ITEM;
		{
			hParentItem2 = hParentItem;
			elemTree.hParent = hParentItem;
			message="typeFace: \t";
			INSERT_ITEM_INT("typeFace","Type deface conserne",FALSE,0,-1,&face->lineMaterial.typeFace);
			VE_int_cur.elem.AddSpecial(GL_FRONT_AND_BACK,"front&back");
			VE_int_cur.elem.AddSpecial(GL_FRONT,"front");
			VE_int_cur.elem.AddSpecial(GL_BACK,"back");
			INSERT_INT;

			message="ambient: ";
			face->lineMaterial.ambient.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				INSERT_ITEM_INT("lineMaterial.ambient.enable","enable l'utilisation des coureurs",FALSE,0,1,&face->lineMaterial.ambient.isEnable);
				INSERT_INT;
				message="r: \t";
				INSERT_ITEM_FLOAT("lineMaterial.ambient.r","Composante rouge",FALSE,0,1,&face->lineMaterial.ambient.r);
				INSERT_FLOAT;
				message="g: \t";
				INSERT_ITEM_FLOAT("lineMaterial.ambient.g","Composante verte",FALSE,0,1,&face->lineMaterial.ambient.g);
				INSERT_FLOAT;
				message="b: \t";
				INSERT_ITEM_FLOAT("lineMaterial.ambient.b","Composante bleu",FALSE,0,1,&face->lineMaterial.ambient.b);
				INSERT_FLOAT;
				message="a: \t";
				INSERT_ITEM_FLOAT("lineMaterial.ambient.a","Composante alpha (pour blending)",FALSE,0,1,&face->lineMaterial.ambient.a);
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message="diffuse: ";
			face->lineMaterial.diffuse.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				INSERT_ITEM_INT("lineMaterial.diffuse.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->lineMaterial.diffuse.isEnable);
				INSERT_INT;
				message="r: \t";
				INSERT_ITEM_FLOAT("lineMaterial.diffuse.r","Composante rouge",FALSE,0,1,&face->lineMaterial.diffuse.r);
				INSERT_FLOAT;
				message="g: \t";
				INSERT_ITEM_FLOAT("lineMaterial.diffuse.g","Composante verte",FALSE,0,1,&face->lineMaterial.diffuse.g);
				INSERT_FLOAT;
				message="b: \t";
				INSERT_ITEM_FLOAT("lineMaterial.diffuse.b","Composante bleu",FALSE,0,1,&face->lineMaterial.diffuse.b);
				INSERT_FLOAT;
				message="a: \t";
				INSERT_ITEM_FLOAT("lineMaterial.diffuse.a","Composante alpha (pour blending)",FALSE,0,1,&face->lineMaterial.diffuse.a);
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message="specular: ";
			face->lineMaterial.specular.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				INSERT_ITEM_INT("lineMaterial.specular.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->lineMaterial.specular.isEnable);
				INSERT_INT;
				message="r: \t";
				INSERT_ITEM_FLOAT("lineMaterial.specular.r","Composante rouge",FALSE,0,1,&face->lineMaterial.specular.r);
				INSERT_FLOAT;
				message="g: \t";
				INSERT_ITEM_FLOAT("lineMaterial.specular.g","Composante verte",FALSE,0,1,&face->lineMaterial.specular.g);
				INSERT_FLOAT;
				message="b: \t";
				INSERT_ITEM_FLOAT("lineMaterial.specular.b","Composante bleu",FALSE,0,1,&face->lineMaterial.specular.b);
				INSERT_FLOAT;
				message="a: \t";
				INSERT_ITEM_FLOAT("lineMaterial.specular.a","Composante alpha (pour blending)",FALSE,0,1,&face->lineMaterial.specular.a);
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message="emission: ";
			face->lineMaterial.emission.ToString(message);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				INSERT_ITEM_INT("lineMaterial.emission.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->lineMaterial.emission.isEnable);
				INSERT_INT;
				message="r: \t";
				INSERT_ITEM_FLOAT("lineMaterial.emission.r","Composante rouge",FALSE,0,1,&face->lineMaterial.emission.r);
				INSERT_FLOAT;
				message="g: \t";
				INSERT_ITEM_FLOAT("lineMaterial.emission.g","Composante verte",FALSE,0,1,&face->lineMaterial.emission.g);
				INSERT_FLOAT;
				message="b: \t";
				INSERT_ITEM_FLOAT("lineMaterial.emission.b","Composante bleu",FALSE,0,1,&face->lineMaterial.emission.b);
				INSERT_FLOAT;
				message="a: \t";
				INSERT_ITEM_FLOAT("lineMaterial.emission.a","Composante alpha (pour blending)",FALSE,0,1,&face->lineMaterial.emission.a);
				INSERT_FLOAT;
				elemTree.hParent = hParentItem2;
			}

			message<<clear<<"shiness [enable,value]: "<<face->lineMaterial.shiness.isEnable<<" , "<<face->lineMaterial.shiness.elem;
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="enable: \t";
				INSERT_ITEM_INT("lineMaterial.shiness.enable","enable l'utilisation de la luminosit",FALSE,0,1,&face->lineMaterial.shiness.isEnable);
				INSERT_INT;
				message="value: \t";
				INSERT_ITEM_INT("lineMaterial.shiness.value","luminosit",FALSE,0,128,&face->lineMaterial.shiness.elem);
				INSERT_INT;
				elemTree.hParent = hParentItem2;
			}
			elemTree.hParent = NULL;
		}
		elemTree.hParent = NULL;
	}
	if (face->text)
	{
		message.Format("Text [value,scale]: \"%s\" , %.2lf",(char*)face->text,face->text->scale);
		INSERT_ITEM;
		{
			hParentItem2 = hParentItem;
			elemTree.hParent = hParentItem;

			//message="Font: \t";
			//INSERT_ITEM_STRING("text.font.name","Nom de la fonte utilis",FALSE,&face->text->font->name);
			//INSERT_STRING;

			message.Format("align [x,y]: %d , %d",face->text->xAlign,face->text->yAlign);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="x: \t";
				INSERT_ITEM_INT("text.align.x","type d'alignement sur les X",FALSE,0,-1,(int *)&face->text->xAlign);
				VE_int_cur.elem.AddSpecial(align_begin,"begin");
				VE_int_cur.elem.AddSpecial(align_center,"center");
				VE_int_cur.elem.AddSpecial(align_end,"end");
				VE_int_cur.elem.AddSpecial(align_justify,"justify");
				INSERT_INT;
				message="y: \t";
				INSERT_ITEM_INT("text.align.y","type d'alignement sur les Y",FALSE,0,-1,(int *)&face->text->yAlign);
				VE_int_cur.elem.AddSpecial(align_begin,"begin");
				VE_int_cur.elem.AddSpecial(align_center,"center");
				VE_int_cur.elem.AddSpecial(align_end,"end");
				VE_int_cur.elem.AddSpecial(align_justify,"justify");
				INSERT_INT;
				message="xJustifyLimit: \t";
				INSERT_ITEM_DOUBLE("text.xJustifyLimit","maximum de la justification en X",FALSE,0,50,&face->text->xJustifyLimit);
				INSERT_DOUBLE;
				message="yJustifyLimit: \t";
				INSERT_ITEM_DOUBLE("text.yJustifyLimit","maximum de la justification en Y",FALSE,0,50,&face->text->yJustifyLimit);
				INSERT_DOUBLE;
				elemTree.hParent = hParentItem2;
			}				
			//message="value: \t";
			//INSERT_ITEM_STRING("value","Text a afficher",FALSE,&face->text->value);
			//INSERT_STRING;
			
			message.Format("wrapping [line,word,letter]: %d , %d , %d",face->text->lineWrap,face->text->wordWrap,face->text->letterWrap);
			INSERT_ITEM;
			{
				elemTree.hParent = hParentItem;
				message="line: \t";
				INSERT_ITEM_INT("text.lineWrap","les lignes sont elle secable ?(retour automatique)",FALSE,0,-1,(int *)&face->text->lineWrap);
				VE_int_cur.elem.AddSpecial(FALSE,"false");
				VE_int_cur.elem.AddSpecial(TRUE,"true");
				INSERT_INT;
				message="word: \t";
				INSERT_ITEM_INT("text.wordWrap","les mots sont ils secable ?(retour automatique)",FALSE,0,-1,(int *)&face->text->wordWrap);
				VE_int_cur.elem.AddSpecial(FALSE,"false");
				VE_int_cur.elem.AddSpecial(TRUE,"true");
				INSERT_INT;
				message="letter: \t";
				INSERT_ITEM_INT("text.letterWrap","les lettres sont elle secable ?",FALSE,0,-1,(int *)&face->text->letterWrap);
				VE_int_cur.elem.AddSpecial(FALSE,"false");
				VE_int_cur.elem.AddSpecial(TRUE,"true");
				INSERT_INT;
				elemTree.hParent = hParentItem2;
			}

			message="scale: \t";
			INSERT_ITEM_DOUBLE("text.scale","taille relative de la fonte",FALSE,0,4,&face->text->scale);
			INSERT_DOUBLE;

			message="depth: \t";
			INSERT_ITEM_FLOAT("text.depth","epaisseur (dans les cas d'une fonte extrude)",FALSE,0,20,&face->text->depth);
			INSERT_FLOAT;

			message="material";
			INSERT_ITEM;
			{
				hParentItem2 = hParentItem;
				elemTree.hParent = hParentItem;
				message="typeFace: \t";
				INSERT_ITEM_INT("typeFace","Type deface conserne",FALSE,0,-1,&face->text->material.typeFace);
				VE_int_cur.elem.AddSpecial(GL_FRONT_AND_BACK,"front&back");
				VE_int_cur.elem.AddSpecial(GL_FRONT,"front");
				VE_int_cur.elem.AddSpecial(GL_BACK,"back");
				INSERT_INT;

				message="ambient: ";
				face->text->material.ambient.ToString(message);
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					INSERT_ITEM_INT("material.ambient.enable","enable l'utilisation des coureurs",FALSE,0,1,&face->text->material.ambient.isEnable);
					INSERT_INT;
					message="r: \t";
					INSERT_ITEM_FLOAT("material.ambient.r","Composante rouge",FALSE,0,1,&face->text->material.ambient.r);
					INSERT_FLOAT;
					message="g: \t";
					INSERT_ITEM_FLOAT("material.ambient.g","Composante verte",FALSE,0,1,&face->text->material.ambient.g);
					INSERT_FLOAT;
					message="b: \t";
					INSERT_ITEM_FLOAT("material.ambient.b","Composante bleu",FALSE,0,1,&face->text->material.ambient.b);
					INSERT_FLOAT;
					message="a: \t";
					INSERT_ITEM_FLOAT("material.ambient.a","Composante alpha (pour blending)",FALSE,0,1,&face->text->material.ambient.a);
					INSERT_FLOAT;
					elemTree.hParent = hParentItem2;
				}

				message="diffuse: ";
				face->text->material.diffuse.ToString(message);
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					INSERT_ITEM_INT("material.diffuse.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->text->material.diffuse.isEnable);
					INSERT_INT;
					message="r: \t";
					INSERT_ITEM_FLOAT("material.diffuse.r","Composante rouge",FALSE,0,1,&face->text->material.diffuse.r);
					INSERT_FLOAT;
					message="g: \t";
					INSERT_ITEM_FLOAT("material.diffuse.g","Composante verte",FALSE,0,1,&face->text->material.diffuse.g);
					INSERT_FLOAT;
					message="b: \t";
					INSERT_ITEM_FLOAT("material.diffuse.b","Composante bleu",FALSE,0,1,&face->text->material.diffuse.b);
					INSERT_FLOAT;
					message="a: \t";
					INSERT_ITEM_FLOAT("material.diffuse.a","Composante alpha (pour blending)",FALSE,0,1,&face->text->material.diffuse.a);
					INSERT_FLOAT;
					elemTree.hParent = hParentItem2;
				}

				message="specular: ";
				face->text->material.specular.ToString(message);
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					INSERT_ITEM_INT("material.specular.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->text->material.specular.isEnable);
					INSERT_INT;
					message="r: \t";
					INSERT_ITEM_FLOAT("material.specular.r","Composante rouge",FALSE,0,1,&face->text->material.specular.r);
					INSERT_FLOAT;
					message="g: \t";
					INSERT_ITEM_FLOAT("material.specular.g","Composante verte",FALSE,0,1,&face->text->material.specular.g);
					INSERT_FLOAT;
					message="b: \t";
					INSERT_ITEM_FLOAT("material.specular.b","Composante bleu",FALSE,0,1,&face->text->material.specular.b);
					INSERT_FLOAT;
					message="a: \t";
					INSERT_ITEM_FLOAT("material.specular.a","Composante alpha (pour blending)",FALSE,0,1,&face->text->material.specular.a);
					INSERT_FLOAT;
					elemTree.hParent = hParentItem2;
				}

				message="emission: ";
				face->text->material.emission.ToString(message);
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					INSERT_ITEM_INT("material.emission.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->text->material.emission.isEnable);
					INSERT_INT;
					message="r: \t";
					INSERT_ITEM_FLOAT("material.emission.r","Composante rouge",FALSE,0,1,&face->text->material.emission.r);
					INSERT_FLOAT;
					message="g: \t";
					INSERT_ITEM_FLOAT("material.emission.g","Composante verte",FALSE,0,1,&face->text->material.emission.g);
					INSERT_FLOAT;
					message="b: \t";
					INSERT_ITEM_FLOAT("material.emission.b","Composante bleu",FALSE,0,1,&face->text->material.emission.b);
					INSERT_FLOAT;
					message="a: \t";
					INSERT_ITEM_FLOAT("material.emission.a","Composante alpha (pour blending)",FALSE,0,1,&face->text->material.emission.a);
					INSERT_FLOAT;
					elemTree.hParent = hParentItem2;
				}

				message<<clear<<"shiness [enable,value]: "<<face->text->material.shiness.isEnable<<" , "<<face->text->material.shiness.elem;
				INSERT_ITEM;
				{
					elemTree.hParent = hParentItem;
					message="enable: \t";
					INSERT_ITEM_INT("material.shiness.enable","enable l'utilisation de la luminosit",FALSE,0,1,&face->text->material.shiness.isEnable);
					INSERT_INT;
					message="value: \t";
					INSERT_ITEM_INT("material.shiness.value","luminosit",FALSE,0,128,&face->text->material.shiness.elem);
					INSERT_INT;
					elemTree.hParent = hParentItem2;
				}
				elemTree.hParent = NULL;
			}
			elemTree.hParent = NULL;
		}
	}
	message="Curve";
	INSERT_ITEM;
	{
		hParentItem2 = hParentItem;
		elemTree.hParent = hParentItem;
		message.Format("center [x,y,z]: %.2f , %.2f , %.2f",face->curve.center.x,face->curve.center.y,face->curve.center.z);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			message="x: \t";
			INSERT_ITEM_DOUBLE("curve.center.x","position du centre en x de l'incurvation",FALSE,0,1,&face->curve.center.x);
			INSERT_DOUBLE;
			message="y: \t";
			INSERT_ITEM_DOUBLE("curve.center.y","position du centre en y de l'incurvation",FALSE,0,1,&face->curve.center.y);
			INSERT_DOUBLE;
			message="z: \t";
			INSERT_ITEM_DOUBLE("curve.center.z","position du centre en z de l'incurvation",FALSE,-1024,1024,&face->curve.center.z);
			INSERT_DOUBLE;
			elemTree.hParent = hParentItem2;
		}
		message<<clear<<"nbCut [x,y]: "<<face->curve.nbCut.x<<" , "<<face->curve.nbCut.y;
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			message="x: \t";
			INSERT_ITEM_INT("curve.nbCut.x","nb subdivision en sous-face en x",FALSE,1,50,&face->curve.nbCut.x);
			INSERT_INT;
			message="y: \t";
			INSERT_ITEM_INT("curve.nbCut.y","nb subdivision en sous-face en y",FALSE,1,50,&face->curve.nbCut.y);
			INSERT_INT;
			elemTree.hParent = hParentItem2;
		}
		elemTree.hParent = NULL;
	}
	message.Format("Center [x,y => cx,cy]: %d , %d => %.2lf , %.2lf",face->center.x,face->center.y,face->computeCenter.x,face->computeCenter.y);
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="x: \t";
		INSERT_ITEM_INT("center.x","position dans les x",FALSE,0,-1,&face->center.x);
		VE_int_cur.elem.AddSpecial(-1,"gauche");
		VE_int_cur.elem.AddSpecial(0,"centr");
		VE_int_cur.elem.AddSpecial(1,"droit");
		INSERT_INT;
		message="y: \t";
		INSERT_ITEM_INT("center.y","position dans les y",FALSE,0,-1,&face->center.y);
		VE_int_cur.elem.AddSpecial(-1,"haut");
		VE_int_cur.elem.AddSpecial(0,"centr");
		VE_int_cur.elem.AddSpecial(1,"bas");
		INSERT_INT;
		elemTree.hParent = NULL;
	}
	message="Position";
	INSERT_ITEM;
	{
		hParentItem2 = hParentItem;
		elemTree.hParent = hParentItem;
		if (face->position.x.IsDynamic())
			 message.Format("position.x: %.2lf * %.2lf + %.2lf = %.2lf",face->position.x.percentValue,*face->position.x.baseValue,face->position.x.constValue,face->computePosition.x);
		else message.Format("position.x: %.2lf",face->position.x.constValue);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			if (face->position.x.IsDynamic())
			{
				message="percent: \t";
				INSERT_ITEM_DOUBLE("position.x.percentValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,-0.5,0.5,&face->position.x.percentValue);
				INSERT_DOUBLE;
				message="base: \t";
				INSERT_ITEM_DOUBLE("position.x.baseValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,-1024,1024,face->position.x.baseValue);
				INSERT_DOUBLE;
			}
			message="const: \t";
			INSERT_ITEM_DOUBLE("position.x.constValue","composante constante de la position (pos = pourcentage*variable+constante)",FALSE,-1024,1024,&face->position.x.constValue);
			INSERT_DOUBLE;
			elemTree.hParent = hParentItem2;
		}
		if (face->position.y.IsDynamic())
				message.Format("position.y: %.2lf * %.2lf + %.2lf = %.2lf",face->position.y.percentValue,*face->position.y.baseValue,face->position.y.constValue,face->computePosition.y);
		else message.Format("position.y: %.2lf",face->position.y.constValue);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			if (face->position.y.IsDynamic())
			{
				message="percent: \t";
				INSERT_ITEM_DOUBLE("position.y.percentValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,-0.5,0.5,&face->position.y.percentValue);
				INSERT_DOUBLE;
				message="base: \t";
				INSERT_ITEM_DOUBLE("position.y.baseValue","composante base de la taille (taille = pourcentage*base+constante)",FALSE,-1024,1024,face->position.y.baseValue);
				INSERT_DOUBLE;
			}				
			message="const: \t";
			INSERT_ITEM_DOUBLE("position.y.constValue","composante constante de la position (pos = pourcentage*variable+constante)",FALSE,-1024,1024,&face->position.y.constValue);
			INSERT_DOUBLE;
			elemTree.hParent = hParentItem2;
		}
		if (face->position.z.IsDynamic())
				message.Format("position.z: %.2lf * %.2lf + %.2lf = %.2lf",face->position.z.percentValue,*face->position.z.baseValue,face->position.z.constValue,face->computePosition.z);
		else message.Format("position.z: %.2lf",face->position.z.constValue);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			if (face->position.z.IsDynamic())
			{
				message="percent: \t";
				INSERT_ITEM_DOUBLE("position.z.percentValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,-0.5,0.5,&face->position.z.percentValue);
				INSERT_DOUBLE;
				message="base: \t";
				INSERT_ITEM_DOUBLE("position.z.baseValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,-1024,1024,face->position.z.baseValue);
				INSERT_DOUBLE;
			}				
			message="const: \t";
			INSERT_ITEM_DOUBLE("position.z.constValue","composante constante de la position (pos = pourcentage*variable+constante)",FALSE,-1024,1024,&face->position.z.constValue);
			INSERT_DOUBLE;
			elemTree.hParent = hParentItem2;
		}
		elemTree.hParent = NULL;
	}
	message="Size";
	INSERT_ITEM;
	{
		hParentItem2 = hParentItem;
		elemTree.hParent = hParentItem;
		if (face->size.x.IsDynamic())
				message.Format("size.x: %.2lf * %.2lf + %.2lf = %.2lf",face->size.x.percentValue,*face->size.x.baseValue,face->size.x.constValue,face->computeSize.x);
		else message.Format("size.x: %.2lf",face->size.x.constValue);

		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			if (face->size.x.IsDynamic())
			{
				message="percent: \t";
				INSERT_ITEM_DOUBLE("size.x.percentValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,0,1,&face->size.x.percentValue);
				INSERT_DOUBLE;
				message="base: \t";
				INSERT_ITEM_DOUBLE("size.x.baseValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,0,2048,face->size.x.baseValue);
				INSERT_DOUBLE;
			}
			message="const: \t";
			INSERT_ITEM_DOUBLE("size.x.constValue","composante constante de la taille (taille = pourcentage*base+constante)",FALSE,0,2048,&face->size.x.constValue);
			INSERT_DOUBLE;
			elemTree.hParent = hParentItem2;
		}
		if (face->size.y.IsDynamic())
				message.Format("size.y: %.2lf * %.2lf + %.2lf = %.2lf",face->size.y.percentValue,*face->size.y.baseValue,face->size.y.constValue,face->computeSize.y);
		else message.Format("size.y: %.2lf",face->size.y.constValue);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			if (face->size.y.IsDynamic())
			{
				message="percent: \t";
				INSERT_ITEM_DOUBLE("size.y.percentValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,0,1,&face->size.y.percentValue);
				INSERT_DOUBLE;
				message="base: \t";
				INSERT_ITEM_DOUBLE("size.y.baseValue","composante pourcentage de la taille (taille = pourcentage*base+constante)",FALSE,0,2048,face->size.y.baseValue);
				INSERT_DOUBLE;
			}
			message="const: \t";
			INSERT_ITEM_DOUBLE("size.y.constValue","composante constante de la taille (taille = pourcentage*variable+constante)",FALSE,0,2048,&face->size.y.constValue);
			INSERT_DOUBLE;
			elemTree.hParent = hParentItem2;
		}
		elemTree.hParent = NULL;
	}
	message.Format("Scale [x,y,z]: %.3lf , %.3lf , %.3lf",face->scale.x,face->scale.y,face->scale.z);
	INSERT_ITEM;
	{
		elemTree.hParent = hParentItem;
		message="x: \t";
		INSERT_ITEM_DOUBLE("scale.x","zoom en x",FALSE,0.25,4,&face->scale.x);
		INSERT_DOUBLE;
		message="y: \t";
		INSERT_ITEM_DOUBLE("scale.y","zoom en y",FALSE,0.25,4,&face->scale.y);
		INSERT_DOUBLE;
		message="z: \t";
		INSERT_ITEM_DOUBLE("scale.z","zoom en z",FALSE,0.25,4,&face->scale.z);
		INSERT_DOUBLE;
		elemTree.hParent = NULL;
	}
	message="Material";
	INSERT_ITEM;
	{
		hParentItem2 = hParentItem;
		elemTree.hParent = hParentItem;
		message="typeFace: \t";
		INSERT_ITEM_INT("typeFace","Type deface conserne",FALSE,0,-1,&face->material.typeFace);
		VE_int_cur.elem.AddSpecial(GL_FRONT_AND_BACK,"front&back");
		VE_int_cur.elem.AddSpecial(GL_FRONT,"front");
		VE_int_cur.elem.AddSpecial(GL_BACK,"back");
		INSERT_INT;

		message="ambient: ";
		face->material.ambient.ToString(message);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			message="enable: \t";
			INSERT_ITEM_INT("material.ambient.enable","enable l'utilisation des coureurs",FALSE,0,1,&face->material.ambient.isEnable);
			INSERT_INT;
			message="r: \t";
			INSERT_ITEM_FLOAT("material.ambient.r","Composante rouge",FALSE,0,1,&face->material.ambient.r);
			INSERT_FLOAT;
			message="g: \t";
			INSERT_ITEM_FLOAT("material.ambient.g","Composante verte",FALSE,0,1,&face->material.ambient.g);
			INSERT_FLOAT;
			message="b: \t";
			INSERT_ITEM_FLOAT("material.ambient.b","Composante bleu",FALSE,0,1,&face->material.ambient.b);
			INSERT_FLOAT;
			message="a: \t";
			INSERT_ITEM_FLOAT("material.ambient.a","Composante alpha (pour blending)",FALSE,0,1,&face->material.ambient.a);
			INSERT_FLOAT;
			elemTree.hParent = hParentItem2;
		}

		message="diffuse: ";
		face->material.diffuse.ToString(message);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			message="enable: \t";
			INSERT_ITEM_INT("material.diffuse.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->material.diffuse.isEnable);
			INSERT_INT;
			message="r: \t";
			INSERT_ITEM_FLOAT("material.diffuse.r","Composante rouge",FALSE,0,1,&face->material.diffuse.r);
			INSERT_FLOAT;
			message="g: \t";
			INSERT_ITEM_FLOAT("material.diffuse.g","Composante verte",FALSE,0,1,&face->material.diffuse.g);
			INSERT_FLOAT;
			message="b: \t";
			INSERT_ITEM_FLOAT("material.diffuse.b","Composante bleu",FALSE,0,1,&face->material.diffuse.b);
			INSERT_FLOAT;
			message="a: \t";
			INSERT_ITEM_FLOAT("material.diffuse.a","Composante alpha (pour blending)",FALSE,0,1,&face->material.diffuse.a);
			INSERT_FLOAT;
			elemTree.hParent = hParentItem2;
		}

		message="specular: ";
		face->material.specular.ToString(message);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			message="enable: \t";
			INSERT_ITEM_INT("material.specular.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->material.specular.isEnable);
			INSERT_INT;
			message="r: \t";
			INSERT_ITEM_FLOAT("material.specular.r","Composante rouge",FALSE,0,1,&face->material.specular.r);
			INSERT_FLOAT;
			message="g: \t";
			INSERT_ITEM_FLOAT("material.specular.g","Composante verte",FALSE,0,1,&face->material.specular.g);
			INSERT_FLOAT;
			message="b: \t";
			INSERT_ITEM_FLOAT("material.specular.b","Composante bleu",FALSE,0,1,&face->material.specular.b);
			INSERT_FLOAT;
			message="a: \t";
			INSERT_ITEM_FLOAT("material.specular.a","Composante alpha (pour blending)",FALSE,0,1,&face->material.specular.a);
			INSERT_FLOAT;
			elemTree.hParent = hParentItem2;
		}

		message="emission: ";
		face->material.emission.ToString(message);
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			message="enable: \t";
			INSERT_ITEM_INT("material.emission.enable","enable l'utilisation des couleurs",FALSE,0,1,&face->material.emission.isEnable);
			INSERT_INT;
			message="r: \t";
			INSERT_ITEM_FLOAT("material.emission.r","Composante rouge",FALSE,0,1,&face->material.emission.r);
			INSERT_FLOAT;
			message="g: \t";
			INSERT_ITEM_FLOAT("material.emission.g","Composante verte",FALSE,0,1,&face->material.emission.g);
			INSERT_FLOAT;
			message="b: \t";
			INSERT_ITEM_FLOAT("material.emission.b","Composante bleu",FALSE,0,1,&face->material.emission.b);
			INSERT_FLOAT;
			message="a: \t";
			INSERT_ITEM_FLOAT("material.emission.a","Composante alpha (pour blending)",FALSE,0,1,&face->material.emission.a);
			INSERT_FLOAT;
			elemTree.hParent = hParentItem2;
		}

		message<<clear<<"shiness [enable,value]: "<<face->material.shiness.isEnable<<" , "<<face->material.shiness.elem;
		INSERT_ITEM;
		{
			elemTree.hParent = hParentItem;
			message="enable: \t";
			INSERT_ITEM_INT("material.shiness.enable","enable l'utilisation de la luminosit",FALSE,0,1,&face->material.shiness.isEnable);
			INSERT_INT;
			message="value: \t";
			INSERT_ITEM_INT("material.shiness.value","luminosit",FALSE,0,128,&face->material.shiness.elem);
			INSERT_INT;
			elemTree.hParent = hParentItem2;
		}
		elemTree.hParent = NULL;
	}
	message="Texture";
	if (face->texture) 
	{	message<<" [id,name]: "<<face->texture->id<<" , "<<face->texture->name;
		INSERT_ITEM;
		{
			hParentItem2 = hParentItem;
			elemTree.hParent = hParentItem;
			message="id: \t";
			INSERT_ITEM_INT("texture.id","Numero de la texture",IS_TEXTURE,0,15,&face->texture->id);
			VE_int_cur.elem.AddSpecial(0,"not use");
			INSERT_INT;
			elemTree.hParent = NULL;
		}
	}
	return TRUE;
}	

ValueEditors valueEditors;

LRESULT WINAPI DebugStructProc(HWND hDlg,UINT message,WPARAM wParam,LPARAM lParam)
{
	static HWND hWndTreeStruct;
	static HWND hWndTreeElem;
	static FaceNode *node;
	static ParamValueEditor *param;
	static HWND hWndValueEditor;
	static RECT rect;
	static BOOL mode_control_enable;
	if (param) valueEditors.Update(param->object);

	switch(message)
	{
	case WM_INITDIALOG: 
		param=NULL;
		hWndValueEditor=NULL;
		GetWindowRect(hDlg,&rect);
		hWndTreeStruct = GetDlgItem(hDlg,IDC_TREE_STRUCT);
		hWndTreeElem = GetDlgItem(hDlg,IDC_TREE_ELEM);
		Button_SetCheck(GetDlgItem(hDlg,IDC_RADIO_FACEMODE),TRUE);

		mode_control_enable = FALSE;
		valueEditors.MakeDebugFaceStruct(hWndTreeStruct);
		node = ce.primitive.tree.GetRoot();
		valueEditors.MakeElemFaceStruct(hWndTreeElem,node);
		TreeView_ExpandAll_Level(hWndTreeStruct,TRUE,2);
		TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);
		return TRUE;

	case WM_MOVE:
		{
			int deltaX = -rect.left;
			int deltaY = -rect.top;
			GetWindowRect(hDlg,&rect);
			deltaX+=rect.left;
			deltaY+=rect.top;

			if (param&&(!param->independant))
			{
				RECT rectElem;
				GetWindowRect(GetDlgItem(hDlg,IDC_TREE_ELEM),&rectElem);
				SetDeltaWindowPos(hWndValueEditor,deltaX,deltaY);
			}
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
		case IDCANCEL:
			EndDialog(hWndValueEditor,LOWORD(wParam));
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;

		case IDC_RADIO_FACE_MODE:
			mode_control_enable = FALSE;
			TreeView_DeleteAllItems(hWndTreeStruct);
			TreeView_DeleteAllItems(hWndTreeElem);
			valueEditors.MakeDebugFaceStruct(hWndTreeStruct);
			node = ce.primitive.tree.GetRoot();
			valueEditors.MakeElemFaceStruct(hWndTreeElem,node);
			TreeView_ExpandAll_Level(hWndTreeStruct,TRUE,2);
			TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);
			break;

		case IDC_RADIO_CONTROLS_MODE:
			mode_control_enable = TRUE;
			TreeView_DeleteAllItems(hWndTreeStruct);
			TreeView_DeleteAllItems(hWndTreeElem);
			valueEditors.MakeDebugControlStruct(hWndTreeStruct);
			node = ce.primitive.tree.GetRoot();
			valueEditors.MakeElemControlStruct(hWndTreeElem,ce.masterContainer);
			TreeView_ExpandAll_Level(hWndTreeStruct,TRUE,2);
			TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);
			break;
		}
		break;

	case WM_NCMOUSEMOVE:
	case WM_MOUSEMOVE:
		ShowMouse(TRUE); // car normalement iniber par la genetre GL
		return TRUE;

	case WM_NOTIFY:
		{
			int numSelectedControl;
			int numSelectedFace;
			LPNMHDR pnmh = (LPNMHDR) lParam; 
			LPNMTREEVIEW pnmtv = (LPNMTREEVIEW) lParam;
			switch (wParam)
			{
			case IDC_TREE_STRUCT: 
				switch (pnmh->code)
				{
				case TVN_SELCHANGED:
					{
						LPARAM param =pnmtv->itemNew.lParam;
						numSelectedControl = LOWORD(param);
						numSelectedFace = HIWORD(param);
						if ((numSelectedFace&0x8000)==0x8000) // gestion des nombre negatifs
							numSelectedFace-=0x10000;
					}
					switch(numSelectedControl)
					{
					case NUM_SELECTED_RENDER: // config - Render
						TreeView_DeleteAllItems(hWndTreeElem);
						valueEditors.MakeRenderStruct(hWndTreeElem);
						TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);						
						break;
					case NUM_SELECTED_LIGHT: // config - Lights
						TreeView_DeleteAllItems(hWndTreeElem);
						valueEditors.MakeLightsStruct(hWndTreeElem);
						TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);						
						break;
					case NUM_SELECTED_FONT: // config - Font
						TreeView_DeleteAllItems(hWndTreeElem);
						valueEditors.MakeFontsStruct(hWndTreeElem);
						TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);						
						break;
					case NUM_SELECTED_TEXTURE: // config - Textures
						TreeView_DeleteAllItems(hWndTreeElem);
						valueEditors.MakeTexturesStruct(hWndTreeElem);
						TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);						
						break;
					case NUM_SELECTED_PICKING: // picking 
					case NUM_SELECTED_NOTHING: // config - rien de selectione
						TreeView_DeleteAllItems(hWndTreeElem);
						TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);						
						break;
					default:
						if (mode_control_enable)
						{
							Control *control = Control::Find(numSelectedControl);
							TreeView_DeleteAllItems(hWndTreeElem);
							valueEditors.MakeElemControlStruct(hWndTreeElem,control);
							TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);	
						}
						else
						{
							node = ce.primitive.Find(numSelectedControl,numSelectedFace);
							TreeView_DeleteAllItems(hWndTreeElem);
							valueEditors.MakeElemFaceStruct(hWndTreeElem,node);
							TreeView_ExpandAll_Level(hWndTreeElem,TRUE,2);	
						}
					}
				}
				break;
			case IDC_TREE_ELEM: 
				switch (pnmh->code)
				{
				case TVN_SELCHANGED:
					int num = (int)pnmtv->itemNew.lParam;

					if (param&&param->independant) param=NULL;
					if (!param) 
					{
						param = new ParamValueEditor;
						param->hWndParent = hDlg;
						param->independant = FALSE;
						param->object = NULL;
						hWndValueEditor = CreateDialogParam(ce.hInst,MAKEINTRESOURCE(IDD_VALUE_EDITOR),NULL,(DLGPROC)ValueEditorProc,(LPARAM)param);
						ShowWindow(hWndValueEditor,SW_SHOW);
						RECT rectElem;
						GetWindowRect(GetDlgItem(hDlg,IDC_TREE_ELEM),&rectElem);
						SetDeltaWindowPos(hWndValueEditor,rectElem.left,rectElem.bottom);
					}
					if (num) 
					{
						param->object = valueEditors.list.i(num-1);
						valueEditors.Update(param->object);

						MyString title;
						switch(param->object->type)
						{
							case CBASE_INDEX_INT:
								title<<"Value: "<<valueEditors.VE_int->elem.variable<<" - "<<node->elem->name;
								valueEditors.VE_int->elem.MAJ(hWndValueEditor); 
								break;
							case CBASE_INDEX_FLOAT:
								title<<"Value: "<<valueEditors.VE_float->elem.variable<<" - "<<node->elem->name;
								valueEditors.VE_float->elem.MAJ(hWndValueEditor); 
								break;
							case CBASE_INDEX_DOUBLE:
								title<<"Value: "<<valueEditors.VE_double->elem.variable<<" - "<<node->elem->name;
								valueEditors.VE_double->elem.MAJ(hWndValueEditor); 
								break;
						}
						SetWindowText(hWndValueEditor,title);
					}
					else 
					{
						param->object = NULL;
						ValueEditor<GLint> *temp=NULL;
						temp->MAJ(hWndValueEditor);
					}

				}
				break;
			}
		}
		break;
	}
	return FALSE;
}

void ApplyParamObj(LONG type)
{
	switch(type)
	{
	case IS_FOG: ce.fog.Use(); break;//fog.Enable(ce.fog.enable); break;
	case IS_LIGHT: ce.MajLights(); break;
	default:
		if (type>=IS_TEXTURE)
		{
			int num = type-IS_TEXTURE;
			Texture::list.i[num]->Config(); 
		}
	}
}

/*!
\todo mecanisme de desalocation a faire (mais peut inportant pour le moment) 
*/
LRESULT WINAPI ValueEditorProc(HWND hDlg,UINT message,WPARAM wParam,LPARAM lParam)
{
	ParamValueEditor *param = (ParamValueEditor *) GetWindowLongPtr (hDlg,GWLP_USERDATA);

	BOOL activeParam = param&&param->object;
	if (activeParam) valueEditors.Update(param->object);

	switch (message) 
	{
	case WM_INITDIALOG:
		param = (ParamValueEditor *) lParam; 
		SetWindowLongPtr (hDlg, GWLP_USERDATA , (LONG_PTR)param);
		if (param) GetWindowRect(hDlg,&param->rect);
		return TRUE;

	case WM_SIZE:
		{
			if (!param) return TRUE;
			int deltaX,deltaY;
			GetDeltaWindow(hDlg,&param->rect,&deltaX,&deltaY);
			SetDeltaWindowSize(GetDlgItem(hDlg,IDC_STATIC_VARIABLE),deltaX,deltaY);
			SetDeltaWindowSize(GetDlgItem(hDlg,IDC_EDIT_VALUE),deltaX,0);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_EDIT_VALUE),0,deltaY);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_COMBO_SPECIALS),0,deltaY);
			SetDeltaWindowSize(GetDlgItem(hDlg,IDC_COMBO_SPECIALS),deltaX,0);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_STATIC1),0,deltaY);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_STATIC2),0,deltaY);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_BUTTON_MIN),deltaX,deltaY);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_BUTTON_MIDS),deltaX,deltaY);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_BUTTON_MAX),deltaX,deltaY);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_BUTTON_MOVE),deltaX,deltaY);
			SetDeltaWindowPos(GetDlgItem(hDlg,IDC_SLIDER_PAD),0,deltaY);
			SetDeltaWindowSize(GetDlgItem(hDlg,IDC_SLIDER_PAD),deltaX,0);
			InvalidateRect(hDlg,NULL,TRUE);
		}
		return TRUE;

	case WM_HSCROLL:
		if (!activeParam) break;
		if ((HWND)lParam==GetDlgItem(hDlg,IDC_SLIDER_PAD))
		{
			int xPos;
			switch (LOWORD(wParam))
			{
				case SB_THUMBTRACK:
					xPos = HIWORD(wParam); // de 0 a 100
					float xValue = xPos/100.0f;
					if (param->object)
					{
						GLint valueInt;
						GLdouble valueDouble;
						GLfloat valueFloat;
						switch (param->object->type)
						{
						case CBASE_INDEX_INT:
							valueInt=valueEditors.VE_int->elem.minValue+(GLint)(xValue*(valueEditors.VE_int->elem.maxValue-valueEditors.VE_int->elem.minValue));
							valueEditors.VE_int->elem.MAJ_Value(hDlg,valueInt);
							break;
						case CBASE_INDEX_FLOAT:
							valueFloat=valueEditors.VE_float->elem.minValue+xValue*(valueEditors.VE_float->elem.maxValue-valueEditors.VE_float->elem.minValue);
							valueEditors.VE_float->elem.MAJ_Value(hDlg,valueFloat);
							break;
						case CBASE_INDEX_DOUBLE:
							valueDouble=valueEditors.VE_double->elem.minValue+xValue*(valueEditors.VE_double->elem.maxValue-valueEditors.VE_double->elem.minValue);
							valueEditors.VE_double->elem.MAJ_Value(hDlg,valueDouble);
							break;
						}
						ApplyParamObj(param->object->param);
					}
					break;
			}
		}
		break;

	case WM_NCMOUSEMOVE:
	case WM_MOUSEMOVE:
		ShowMouse(TRUE); // car normalement iniber par la genetre GL
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
		case IDCANCEL: EndDialog(hDlg, LOWORD(wParam)); return TRUE;
		case IDC_COMBO_SPECIALS:
			if (!activeParam) break;
			switch (HIWORD(wParam)) 
			{
			case LBN_SELCHANGE :
				{
					int num = ComboBox_GetCurSel(GetDlgItem(hDlg,IDC_COMBO_SPECIALS));
					switch (param->object->type) {
						case CBASE_INDEX_INT: valueEditors.VE_int->elem.MAJ_Specials(hDlg,num-1); break;
						case CBASE_INDEX_FLOAT: valueEditors.VE_float->elem.MAJ_Specials(hDlg,num-1); break;
						case CBASE_INDEX_DOUBLE: valueEditors.VE_double->elem.MAJ_Specials(hDlg,num-1); break;
					}
					ApplyParamObj(param->object->param); // pas sur que ca serve a quelque chose !
				}
				return TRUE;
			} 
			break;

		case IDC_BUTTON_MIN: 
			if (!activeParam) break;
			switch (param->object->type) {
			case CBASE_INDEX_INT: valueEditors.VE_int->elem.MAJ_ValueMin(hDlg); break;
			case CBASE_INDEX_FLOAT: valueEditors.VE_float->elem.MAJ_ValueMin(hDlg); break;
			case CBASE_INDEX_DOUBLE: valueEditors.VE_double->elem.MAJ_ValueMin(hDlg); break;
			}
			ApplyParamObj(param->object->param);
			break;
		case IDC_BUTTON_MAX: 
			if (!activeParam) break;
			switch (param->object->type) {
			case CBASE_INDEX_INT: valueEditors.VE_int->elem.MAJ_ValueMax(hDlg); break;
			case CBASE_INDEX_FLOAT: valueEditors.VE_float->elem.MAJ_ValueMax(hDlg); break;
			case CBASE_INDEX_DOUBLE: valueEditors.VE_double->elem.MAJ_ValueMax(hDlg); break;
			}
			ApplyParamObj(param->object->param);
			break;
		case IDC_BUTTON_MIDS: 
			if (!activeParam) break;
			switch (param->object->type) {
			case CBASE_INDEX_INT: valueEditors.VE_int->elem.MAJ_ValueMids(hDlg); break;
			case CBASE_INDEX_FLOAT: valueEditors.VE_float->elem.MAJ_ValueMids(hDlg); break;
			case CBASE_INDEX_DOUBLE: valueEditors.VE_double->elem.MAJ_ValueMids(hDlg); break;
			}
			ApplyParamObj(param->object->param);
			break;
		case IDC_EDIT_VALUE: 
			if (!activeParam) break;
			switch (param->object->type) {
			case CBASE_INDEX_INT: valueEditors.VE_int->elem.Get_Value(hDlg); valueEditors.VE_int->elem.MAJ_Specials(hDlg); valueEditors.VE_int->elem.MAJ_Slider(hDlg); valueEditors.VE_int->elem.MAJ_TreeItem(param->hWndParent); break;
			case CBASE_INDEX_FLOAT: valueEditors.VE_float->elem.Get_Value(hDlg); valueEditors.VE_float->elem.MAJ_Specials(hDlg); valueEditors.VE_float->elem.MAJ_Slider(hDlg); valueEditors.VE_float->elem.MAJ_TreeItem(param->hWndParent); break;
			case CBASE_INDEX_DOUBLE: valueEditors.VE_double->elem.Get_Value(hDlg); valueEditors.VE_double->elem.MAJ_Specials(hDlg); valueEditors.VE_double->elem.MAJ_Slider(hDlg); valueEditors.VE_double->elem.MAJ_TreeItem(param->hWndParent); break;
			}
			ApplyParamObj(param->object->param);
			break;
		case IDC_BUTTON_MOVE: 
			if (!activeParam) break;
			param->independant = TRUE;
			Button_Enable(GetDlgItem(hDlg,IDC_BUTTON_MOVE),FALSE);
			ParamValueEditor *paramIndependant = param->GetCopy();
			SetWindowLongPtr(hDlg, GWLP_USERDATA , (LONG_PTR)paramIndependant); 
			break;
		}
		break;
	}
	return FALSE;
}