/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file valueEditor.h
\brief Gestion de l'editeur de valeur
*/

#ifndef INCLUDE_VALUE_EDITOR
#define INCLUDE_VALUE_EDITOR

#pragma warning( disable : 4355 ) // pour les this codebase

#include "3d.h"
#include "face.h"
#include "MyObject.h"
#include "control.h"

#define SPEED			6         
#define SPEED_PAD		45    
#define SPEED_DELTA		1         
#define SPEED_PAD_DELTA	10   
#define DELTA_FOG       0.5

/*!
\brief Parametre des valeurs de l'editeur
*/
class ParamValueEditor
{
public:
	Object *object; // valueEditor
	RECT rect;
	HWND hWndParent;
	BOOL independant;

	ParamValueEditor();
	void Zero();
	void operator=(ParamValueEditor &source);
	ParamValueEditor * GetCopy();
};

/*!
\brief Valeur nome pour le mode Editor
*/
template <class T> 
class ValueNamed
{
public:
	T value;
	MyString description;

public:
	ValueNamed();
	void Zero();
	void operator=(ValueNamed &source);
	void Set (T _value,char *_desciption);
};

/*!
\brief valeur de l'element en saisie
*/
template <class T> 
class ValueEditor
{
public:
	MyString valueText;
	MyString info;
	MyString variable;
	T minValue;
	T maxValue;
	T *value;
	HTREEITEM hItem;

	typedef ValueNamed<T> ValueNamedCour;
	MyList<ValueNamedCour> specials;

public:
	ValueEditor();
	void Zero();
	void operator=(ValueEditor &source);
	ValueEditor<T> * GetCopy();
	void InterpretedText(MyString &interpretedText,char *_valueText=NULL,T *_value=NULL);
	void Set(char *_variable,char *_info,T _minValue,T _maxValue,HTREEITEM _hItem);
	void MAJ_TreeItem(HWND hWnd);
	void AddSpecial(T special,char *description);
	void MAJ_ValueMin(HWND hWnd);
	void MAJ_ValueMax(HWND hWnd);
	void MAJ_ValueMids(HWND hWnd);
	void MAJ_Value(HWND hWnd,T _value,BOOL refresh=TRUE);
	void MAJ_Value(HWND hWnd);
	void Get_Value(HWND hWnd);
	void MAJ_Slider(HWND hWnd);
	void MAJ_Specials(HWND hWnd,int num=-1);
	void MAJ(HWND hWnd);
};

typedef ValueEditor<GLint> ValueEditor_int;
typedef ValueEditor<GLfloat> ValueEditor_float;
typedef ValueEditor<GLdouble> ValueEditor_double;

#define CBase_VE_int BaseObj<ValueEditor_int,CBASE_INDEX_INT>
#define CBase_VE_float BaseObj<ValueEditor_float,CBASE_INDEX_FLOAT>
#define CBase_VE_double BaseObj<ValueEditor_double,CBASE_INDEX_DOUBLE>

/*!
\brief Gestion des l'ensemble de ValueEditor
*/
class ValueEditors
{
public:
	MyList<Object> list;
	CBase_VE_double *VE_double;
	CBase_VE_int *VE_int;
	CBase_VE_float *VE_float;

public:
	ValueEditors();
	~ValueEditors();
	void Update(Object *object);
	void Zero();
	void MakeDebugStruct(HWND hWnd);
	void MakeDebugFaceStruct(HWND hWnd);
	void MakeDebugFaceStruct(HWND hWnd,HTREEITEM hParentItem,FaceNode *node);
	void MakeDebugControlStruct(HWND hWnd);
	void MakeDebugControlStruct(HWND hWnd,HTREEITEM hParentItem,FaceNode *node,int oldIdControl);
	void MakeRenderStruct(HWND hWnd);
	void MakeLightsStruct(HWND hWnd);
	void MakeTexturesStruct(HWND hWnd);
	void MakeFontsStruct(HWND hWnd);
	BOOL MakeElemFaceStruct(HWND hWnd,FaceNode *node);
	void MakeElemControlStruct(HWND hWnd,Control *control);

};

LRESULT WINAPI DebugStructProc(HWND hDlg,UINT message,WPARAM wParam,LPARAM lParam);
LRESULT WINAPI ValueEditorProc(HWND hDlg,UINT message,WPARAM wParam,LPARAM lParam);

#endif
