/* *************************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
************************************************************************************* */

#include "3d.h"
#include "input.h"
#include "controlEngine.h"
#include "window.h"
#include "mylist.h"

extern ControlEngine ce;

// ********************************************************************************

Window::Window() 
{
	// init des variable
	name="Window";
	titleDock=top;

	// gestion des evenements
	canResize=TRUE;
	canRotate=TRUE;

	title.onTitle.close.connect(this,OnClose);
	onControl.mouse.left.down.connect(this,OnLMRButton);
	onControl.mouse.mid.down.connect(this,OnLMRButton);
	onControl.mouse.right.down.connect(this,OnLMRButton);
}

Window::~Window()
{
}

// ********************************************************************************

#define DEFAULT_XSIZE_WINDOW 400
#define DEFAULT_YSIZE_WINDOW 300

/*! \brief Creation de l'aspect graphique

Ex.:
\code
// cre un container de 400*300*10 au centre
faceNode = ce.primitive.AddFace(debugInfo,id,0,0,0.5,0,0.5,0,400,0,300,0,FALSE);
ce.primitive.Push(0);
Window window;
window.Create(10);
ce.primitive.Pop();

\endcode
*/

void Window::Create(double depth)
{
	SetParent();

	faceNode = ce.primitive.AddBox(name,id,depth,TRUE); 
	faceNode->elem->material.SetDS(0.5,0.5,0.5);
	
	ce.Push(this);
	ce.primitive.GoTo(FACE_FRONT);
	// en haut centr avec marge de 2 pixel
	title.Create(this);
	switch (titleDock)
	{
	case top:
		title.faceNode->elem->center.Set(0,-1);
		title.faceNode->elem->position.y.Set(-2,0.5,&faceNode->elem->computeSize.y);
		title.faceNode->elem->size.x.Set(-4,1,&faceNode->elem->computeSize.x);
		title.faceNode->elem->size.y=24;
		// en dessous du titre de 2 pixel, bordure de 2 pixel
		client.Create();
		client.faceNode->elem->center.Set(0,1);
		client.faceNode->elem->position.y.Set(2,-0.5,&faceNode->elem->computeSize.y);
		client.faceNode->elem->size.x.Set(-4,1,&faceNode->elem->computeSize.x);
		client.faceNode->elem->size.y.Set(-28,1,&faceNode->elem->computeSize.y);
		break;
	case bottom:
		title.faceNode->elem->center.Set(0,1);
		title.faceNode->elem->position.y.Set(2,-0.5,&faceNode->elem->computeSize.y);
		title.faceNode->elem->size.x.Set(-4,1,&faceNode->elem->computeSize.x);
		title.faceNode->elem->size.y=24;
		// au dessus du titre de 2 pixel, bordure de 2 pixel
		client.Create();
		client.faceNode->elem->center.Set(0,-1);
		client.faceNode->elem->position.y.Set(-2,0.5,&faceNode->elem->computeSize.y);
		client.faceNode->elem->size.x.Set(-4,1,&faceNode->elem->computeSize.x);
		client.faceNode->elem->size.y.Set(-28,1,&faceNode->elem->computeSize.y);
		break;
	case left:
		title.faceNode->elem->center.Set(0,-1);
		title.faceNode->elem->angle.Set(0,0,90);
		title.faceNode->elem->position.x.Set(2,-0.5,&faceNode->elem->computeSize.x);
		title.faceNode->elem->size.x.Set(-4,1,&faceNode->elem->computeSize.y);
		title.faceNode->elem->size.y=24;
		// au dessus du titre de 2 pixel, bordure de 2 pixel
		client.Create();
		client.faceNode->elem->center.Set(1,0);
		client.faceNode->elem->position.x.Set(30,-0.5,&faceNode->elem->computeSize.x);
		client.faceNode->elem->size.x.Set(-32,1,&faceNode->elem->computeSize.x);
		client.faceNode->elem->size.y.Set(-2,1,&faceNode->elem->computeSize.y);
		break;
	case right:
		title.faceNode->elem->center.Set(0,-1);
		title.faceNode->elem->angle.Set(0,0,-90);
		title.faceNode->elem->position.x.Set(-2,0.5,&faceNode->elem->computeSize.x);
		title.faceNode->elem->size.x.Set(-4,1,&faceNode->elem->computeSize.y);
		title.faceNode->elem->size.y=24;
		// au dessus du titre de 2 pixel, bordure de 2 pixel
		client.Create();
		client.faceNode->elem->center.Set(-1,0);
		client.faceNode->elem->position.x.Set(-30,0.5,&faceNode->elem->computeSize.x);
		client.faceNode->elem->size.x.Set(-32,1,&faceNode->elem->computeSize.x);
		client.faceNode->elem->size.y.Set(-2,1,&faceNode->elem->computeSize.y);
		break;
	}
	ce.Pop();
}

// ********************************************************************************

// on a demander la fermeture
int Window::OnClose() 
{
	Debug0Param("Window::OnClose()");
	Delete();
	return 1;
}

int Window::OnLMRButton(StateMouse *mouse,int face)
{
	DebugMouse2("Window::OnLMRButton",mouse,face);
	title.GotFocus();
	return 1;
}

void Window::ToString(MyString &str)
{
	str.AddFormat("name: %s canResize:%d",name,canResize);
}

// ----------------------------------------------------------------------------

ResizingAndRotateControl::ResizingAndRotateControl()
{
	trackButton = NULL;
	onControl.mouse.move.connect(this,OnHover);
}

ResizingAndRotateControl::~ResizingAndRotateControl()
{
//	onControl.mouse.move.disconnect();
}

//! Sur survole de la fenetre affiche le "trackButton" si a proximit d'un bord

int ResizingAndRotateControl::OnHover(StateMouse *mouse,int deltaX,int deltaY,int numFace)
{
	//DebugMouse2("ResizingAndRotateControl::OnHover",mouse,numFace);
	double dx = 10; // maximum 2 texel du bord en X pour etre activ
	double dy = 10; // maximum 2 texel du bord en Y pour etre activ
	// gestion dans le cas ou la face et plus petite que les 10 texel ==> /2 de la taille original
	Face *face = faceNode->Elems(numFace);
	if (face->computeSize.x<(3.0*dx)) dx=face->computeSize.x/3.0;
	if (face->computeSize.y<(3.0*dy)) dy=face->computeSize.y/3.0;

	Point2D<double> localPos;
	Point2D<double> pos(mouse->pos.x,mouse->pos.y);
	localPos = GetLocalPos(numFace,pos);
	BOOL isOnLeft = fabs(localPos.x)<=dx;
	BOOL isOnRight = (localPos.x>=0)&&(fabs(face->computeSize.x-localPos.x)<=dx);
	BOOL isOnTop = fabs(localPos.y)<=dy;
	BOOL isOnBottom = (localPos.y>=0)&&(fabs(face->computeSize.y-localPos.y)<=dy);
	if (isOnLeft||isOnRight||isOnTop||isOnBottom)
	{	// affichage de la TrackButton
		if (!trackButton)
		{
			ce.Push(this);
			ce.primitive.GoTo(numFace);
			trackButton = new TrackButton; 
			trackButton->SetResizeAndRotate(canResize,canRotate);
			if (trackButton->Create(20))
			{
				trackButton->faceNode->elem->position.z=-8;
				trackButton->SetResizeAndRotate(canResize,canRotate);
				trackButton->onControl.over.out.connect(this,OnTrackButtonOver);
			}
			ce.Pop();
		}
		int orientation = 
			(isOnLeft?left:0) |
			(isOnRight?right:0) |
			(isOnTop?top:0) |
			(isOnBottom?bottom:0);
		trackButton->SetMode((DockPos)orientation);

		if (isOnTop) trackButton->YDockTop(0,16,TRUE);
		else if (isOnBottom) trackButton->YDockBottom(0,16,TRUE);
		else trackButton->YDockTop(localPos.y,16,TRUE); 

		if (isOnLeft) trackButton->XDockLeft(0,16,TRUE);
		else if (isOnRight) trackButton->XDockRight(0,16,TRUE);
		else trackButton->XDockLeft(localPos.x,16,TRUE); 
		return 1;
	}
	if (trackButton) ce.timerRefresh.onTimer.connect(this,OnNextFrame);
	return 0;
}

int ResizingAndRotateControl::OnNextFrame(Timer *timer)
{
	timer->onTimer.disconnect(this);
	ce.pickingCursor.ResetCurrent();
	delete trackButton; 
	trackButton = NULL; 
	return 0;
}

int ResizingAndRotateControl::OnTrackButtonOver(Control *newControl,int face)
{
	Debug2Param("ResizingAndRotateControl::OnTrackButtonOver(newCtrl:%s,int face)",newControl,face);
	ce.timerRefresh.onTimer.connect(this,OnNextFrame);
	return 0;
}