/* ***********************************************************************************
	Writer:		Sebastien Bloc
	Copyright:	2003-2004
	eMail:		sebastien.bloc@free.fr
	URL:		http://mignonsoft.free.fr

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	http://www.gnu.org/copyleft/gpl.html
*********************************************************************************** */

/*! \file window.h
\brief Control Window
*/

#ifndef INCLUDE_WINDOW_CTRL_H
#define INCLUDE_WINDOW_CTRL_H

#include "MyString.h"
#include "MyDebug.h"
#include "title.h"
#include "container.h"
#include "cubedock.h"
#include "trackButton.h"

/*! Class de gestion de resize et rotate d'un control

Ceci est une "interface" de control et lui confere une trackButton de gestion
Ex d'un control qui peut etre resize
\code
class MyControl : public ResizingAndRotateControl
{
	MyControl() 
	{
		canResize=TRUE;
		canRotate=FALSE;
		...
	}
}
\endcode
*/

class ResizingAndRotateControl : public Control
{
private:
	int OnTrackButtonOver(Control *,int);
	TrackButton *trackButton;	//!< control d'affichage
	int OnNextFrame(Timer *timer);

public:
	BOOL canResize;
	BOOL canRotate;

	ResizingAndRotateControl();
	~ResizingAndRotateControl();
	int OnHover(StateMouse *mouse,int deltaX,int deltaY,int numFace); // La fenetre est survoll
};

/*!
\brief Une "Window" est form d'un Title et d'un zone Cliente

\todo TrackButton control a faire
*/
class Window : public ResizingAndRotateControl 
{
public:
	DockPos titleDock; //!< Ou est plac le titre: haut,bas, gauche, droit ?
	Title title;
	Container client;

	Window();
	~Window();
	virtual void Create() { Create(DEFAULT_DEPTH_WINDOW); }
	void Create(double depth);
	void ToString(MyString &str);

	int OnMouseWheel(StateMouse *mouse,int face);
	int OnClose(); // on a demander la fermeture
	int OnOver(StateMouse *mouse,int face);
	int OnLMRButton(StateMouse *,int);

};

#endif